/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package es.prodevelop.gvsig.mobile.app;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.string.StringUtilities;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;
import es.prodevelop.gvsig.mobile.gui.mainframe.SplashDialog;
import es.prodevelop.gvsig.mobile.icons.IconResourceReader;

/**
 * 
 * This singleton-based class represents the application itself.
 * Receives the app params from the <code>Launcher</code> and instantiates the 
 * main window.
 * 
 * @author jcarras
 * @see es.prodevelop.gvsig.mobile.app.Launcher
 *
 */
public class GvsigMobileApp {

	
	private static HashMap params = new HashMap();
	private int width = 0;
	private int height = 0;
	private static MainWindow theWindow = null;
	public static MainWindow getTheWindow() {
		return theWindow;
	}

	// private static SplashDialog splash_dlg = null;
	private static GvsigMobileApp theApp = null; //Singleton Pattern
	// private Locale locale;
	private static Logger logger = Logger.getLogger(GvsigMobileApp.class);
	
	public static final String LANGUAGE_PARAM = "l"; // "language"
	public static final String SCREEN_SIZE_PARAM = "s";
	public static final String APP_PATH_PARAM = "p"; // "path"
	public static final String APP_VM = "m"; // "virtual machine"
	public static String RUNNING_VM_ID = ""; // "virtual machine"
	public static final String J9_VM_ID = "J9"; // "virtual machine"
	public static final String PHONEME_VM_ID = "PMEA"; // "virtual machine"
	public static String APP_FOLDER = "";
	public static String GVM_VERSION = "";
	public static String GVM_BN = "";
	
	
	public static String WINDOWS_CE_VERSION = "";
	public static boolean INCLUDE_PAN_TOOL = false;
	
	/**
	 * Hidden constructor (singleton class) 
	 *
	 */
	private GvsigMobileApp() {
		ISO_639_FILE = IconResourceReader.getWorkingDirFile("resources", "iso_639.tab");
		
	}
	
	/**
	 * Singleton access to unique instance
	 *  
	 * @return the unique instance of this class
	 */
	public static GvsigMobileApp getApp(){
		if (theApp==null){
			theApp=new GvsigMobileApp();
		}
		return theApp;	
	}
	
	private static int[] stringToIntArray(String val,String separator) throws NumberFormatException{

		String[] tokens = StringUtilities.split(val, separator);
		int[] result = new int[tokens.length];
		for (int i=0;i<tokens.length;i++){
			result[i]=(int) Integer.parseInt(tokens[i]);
		}
		return result;
	}
	
	/**
	 * This method starts the application. It's called by the Launcher
	 *
	 */
	public void execute(){
		
		// Dimension scr_dim = Toolkit.getDefaultToolkit().getScreenSize();
		
		// Frame splash_back = new Frame();
		// splash_back.setSize(scr_dim.width, scr_dim.height);
		// splash_back.setTitle("gvSIG Mobile...");
		// splash_back.setVisible(true);
		// splash_dlg = new SplashDialog(splash_back, "splash_logo.png");
		

		
		
		// ============== GETTING WINDOWS MOBILE VERSION ==================
		INCLUDE_PAN_TOOL = true;

		String osname = System.getProperty("os.name");
		logger.warn("OS NAME: " + osname);
		logger.warn("VM: " + RUNNING_VM_ID);
		
//		if ((RUNNING_VM_ID.compareToIgnoreCase(PHONEME_VM_ID) == 0)
//				&& (osname.toLowerCase().indexOf("windows") != -1)) {
//			
//			String osversion = System.getProperty("os.version");
//			logger.warn("OS VERSION: " + osversion);
//			if (osversion != null) {
//				WINDOWS_CE_VERSION = StringUtilities.split(osversion, " ")[0];
//				logger.warn("WINCE VERSION: " + WINDOWS_CE_VERSION);
//				try {
//					double osv = Double.parseDouble(WINDOWS_CE_VERSION);
//					if (osv >= 5) {
//						INCLUDE_PAN_TOOL = true;
//					} else {
//						INCLUDE_PAN_TOOL = false;
//					}
//				} catch (Exception ex) {
//					logger.error("While getting windows mobile version: " + ex.getMessage());
//				}
//			} else {
//				logger.warn("Did not find os.version property.");
//				INCLUDE_PAN_TOOL = true;
//			}
//		} else {
//			INCLUDE_PAN_TOOL = true;
//		}
		
		if ((RUNNING_VM_ID.compareToIgnoreCase(PHONEME_VM_ID) == 0)) {
			INCLUDE_PAN_TOOL = false;
		} else {
			INCLUDE_PAN_TOOL = true;
		}
		logger.warn("Add pan tool: " + INCLUDE_PAN_TOOL);
			
		
		
		// app persistence ----------------
		Utils.createWorkspaceFolder("persistence"); //$NON-NLS-1$
		
		//int force_static = GvsigMobileGUIConstants.TOOLBAR_HEIGHT;

		if (getParams().containsKey(SCREEN_SIZE_PARAM)){
			try{
				int[] wH = stringToIntArray((String)getParams().get(SCREEN_SIZE_PARAM), "x");
				if (wH!=null && wH.length==2){
					setWidth(wH[0]);
					setHeight(wH[1]);
				} else {
					logger.warn("Incorrect parameter s");
				}
			} catch (NumberFormatException ex){
				logger.warn("Incorrect parameter s");
			}
		} else {
			// not -s parameter found: we assume that we must set:
			// w = screen_w
			// h = screen_h - (26 / 320) * screen_h
			Dimension dims = Toolkit.getDefaultToolkit().getScreenSize();
			logger.debug("getScreenSize = " + dims.toString());
			setWidth(dims.width);
			setHeight(dims.height - (26 * dims.height / 320));
		}
		
		logger.debug("initializing constants " + GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
		
		//MainWindow creation
		if (theWindow==null){
			if ((width > 0) && (height > 0)) {
				theWindow = new MainWindow(width, height);
			} else {
				theWindow = new MainWindow();
			}
		} else {
			theWindow.setVisible(true); // .show();
		}
	}

	/**
	 * Gets application params in a hashmap
	 * @return application params in a hashmap
	 */
	public static HashMap getParams() {
		return params;
	}

	/**
	 * Sets application params in a hashmap
	 * @param p the app params
	 */
	public void setParams(HashMap p) {
		params = p;
		setParamsInDependencies(params);
		String[] ver_bn = getVersionAndBuildNumber();
		GVM_VERSION=ver_bn[0];
		GVM_BN=ver_bn[1];
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
	}
	
	/**
	 * List of locales
	 */
	public static File ISO_639_FILE = null;
	




	
	private void setParamsInDependencies(HashMap p) {
		String path = (String) p.get(APP_PATH_PARAM);
		APP_FOLDER = path;
		ResourceReader.resourceDir = path + File.separator + "resources";
		// AboutDialog.setResourceDir(path + File.separator + "resources");
	}
	
    private String[] getVersionAndBuildNumber() {
    	
    	String[] resp = new String[2];
    	resp[0] = "[" + Messages.getText("Unknown") + "]";
    	resp[1] = "[" + Messages.getText("Unknown") + "]";
    	
    	Properties properties =  new Properties();
		InputStream is = null;
		try {
			File bnf = new File(GvsigMobileApp.APP_FOLDER + File.separator + "build.number");
			if (!bnf.exists())
				bnf = new File(GvsigMobileApp.APP_FOLDER + File.separator + 
						"appGvsigMobile" + File.separator + "build.number");
			is = new FileInputStream(bnf);
		} catch (FileNotFoundException ex) {
			logger.error("While reading build.number file: " + ex.getMessage());
			return resp;
		}
		
		if (is != null) {
			try {
				properties.load(is);
			} catch (IOException e) {
				logger.error("While loading build.number file: " + e.getMessage());
				return resp;
			}
		} else {
			logger.error("While reading build.number file: input stream is NULL (?)");
			return resp;
		}
		
		resp[0] = (String) properties.get("version.id");
		resp[1] = (String) properties.get("build.number");
		
		return resp;
	}
}
