/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.app;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.HashMap;

import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

import es.prodevelop.gvsig.mobile.common.ProcessUtils;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.ecw.EcwReader;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.shp.ShpReader;
import es.prodevelop.gvsig.mobile.fmap.util.Defaults;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.download.Downloader;
import es.prodevelop.gvsig.mobile.fmap.util.string.StringUtilities;
import gnu.io.RXTXVersion;

/**
 * Launcher class. Instantiates the applicaton class and executes it.
 * 
 * @see es.prodevelop.gvsig.mobile.app.GvsigMobileApp
 * 
 * @author jldominguez
 *
 */
public class Launcher {

	protected static GvsigMobileApp app = GvsigMobileApp.getApp();
	
	/**
	 * Application base folder. It will be set from the app params.
	 */
	public static String APP_BASE_FOLDER = "";
	
	/**
	 * log file name. Full path: APP_BASE_FOLDER/log/LOG_FILE_NAME 
	 */
	public static String LOG_FILE_NAME = "gvsig-mobile.log.txt";
	public static String LOG_FILE_NAME2 = "gvsig-mobile.log.console.txt";


	private static HashMap getParams(String[] args){
		HashMap params = new HashMap();
		
		// Argument search
		System.out.println("App params:");
		for(int i = 0; i < args.length; i++) {
			
			String i_param = args[i];
			// coordinate this with the exe launcher:
			i_param = StringUtilities.replace(i_param, "%20", " ");
			
			System.out.println("Param: " + i + " : " + i_param);
			int pos = i_param.indexOf("=");
			if(pos>=0) {
				// replaceAll does not exist
				String remove_quotes = i_param.substring(pos+1, i_param.length());
				remove_quotes = StringUtilities.replace(remove_quotes, "\"", "");
				params.put(i_param.substring(0, pos).toLowerCase(), remove_quotes);
				
			}
		}
		return params;
	}

	/**
	 * Main method.
	 * 
	 * @param args Valid args are: language=<code> size=<width>x<height> path=<path> Example:
	 * language=en size=400x300 path=c:\gvSIGMobile
	 * 
	 */
	public static void main(String[] pargs) {

		
		// DLLs loaded in the first place to avoid memory problems
		try {
			loadDLLs(pargs);
		} catch (Throwable th) {
			System.err.println("While loading libraries: " + th.getMessage());
		}
		
		System.setProperty ("gnu.io.log.mode", "SILENT_MODE");

		

		long curr = System.currentTimeMillis();

		File fil = new File("..");
		String direct = fil.getAbsolutePath();
		
		if (app.getParams().containsKey(GvsigMobileApp.APP_PATH_PARAM)){
			direct = (String)app.getParams().get(GvsigMobileApp.APP_PATH_PARAM);
			direct = StringUtilities.replace(direct, "\"", "");
		}
		
		Defaults.setWorkspace(direct);

		Utils.createWorkspaceFolder("log");
		String outlogpath = direct + File.separator + "log"  + File.separator + curr + "." + LOG_FILE_NAME;
		String outlogpath2 = direct + File.separator + "log"  + File.separator + curr + "." + LOG_FILE_NAME2;
		System.out.println("Log file: " + outlogpath);

		try {
    		PatternLayout l = new PatternLayout("%5p [%t] - %m%n");
    		FileAppender fa = new FileAppender(l, outlogpath, false);
    		Logger.getRootLogger().setLevel(Level.WARN);
    		//Logger.getRootLogger().setLevel(Level.ERROR);
    		Logger.getRootLogger().addAppender(fa);
    		
    		PrintStream ps = new PrintStream(
    				new BufferedOutputStream(
    				new FileOutputStream(outlogpath2)));
    		
    		System.setOut(ps);
    		System.setErr(ps);

    		System.err.println("SYSTEM.ERR TEST...");
    		System.out.println("SYSTEM.OUT TEST...");
		} catch (Exception ex) {
			System.err.println("Error while initializing logging: " + ex.getMessage());
		}

		// start the app
		app.execute();
	}

	private static void loadDLLs(String[] args) {
		
		System.out.println("Forcing initialization of Utils...");
		Utils.touch();

		if (Utils.USING_PDA) {

			System.out.println("Forcing load of Utils DLL...");
			ProcessUtils.touch();
			String sd_card = ProcessUtils.getSDCardPath();
			if (sd_card.length() == 0) {
				Downloader.CACHED_FILES_ROOT_PATH =
					File.separator + "gvSIGMobileCache";
			} else {
				Downloader.CACHED_FILES_ROOT_PATH =
					File.separator + sd_card + File.separator + "gvSIGMobileCache";
			}
		} else {
			Downloader.CACHED_FILES_ROOT_PATH =
				System.getProperty("user.home") + File.separator + "gvSIGMobileCache";
		}
		System.out.println("Forcing load of ECW DLL...");
		EcwReader.touch();
		System.out.println("Forcing load of SHP DLL...");
		ShpReader.touch();
		
		app.setParams(getParams(args));

		if (app.getParams().containsKey(GvsigMobileApp.APP_VM)){
			String vm = (String)app.getParams().get(GvsigMobileApp.APP_VM);
			System.out.println("VM specified: " + vm);
			GvsigMobileApp.RUNNING_VM_ID = vm;
			if (GvsigMobileApp.PHONEME_VM_ID.equals(vm)){
				String vers = RXTXVersion.getVersion();
				System.out.println("Forcing load of RXTX DLL. Version " + vers);
			}
		} else {
			System.out.println("VM not specified !");
		}
	}
	


}
