package es.prodevelop.gvsig.mobile.edition.gui;

import java.awt.Frame;
import java.awt.Label;
import java.awt.TextField;
import java.awt.event.MouseEvent;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.location.GPSManager;

import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.gui.dialog.AcceptCancelDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;
import es.prodevelop.gvsig.mobile.gui.dialog.MessageDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;
import es.prodevelop.gvsig.mobile.gui.tab.ExpansibleScrolledPanel;
import es.prodevelop.gvsig.mobile.gui.tab.TabbedDialog;

public class GPSEditionConfigDialog extends TabbedDialog{

	private static final long serialVersionUID = -1280063533258759713L;

	private static Logger logger = Logger.getLogger(GPSEditionConfigDialog.class);

	private FLyrVect layer = null;

	private GPSManager gpsMananger = GPSManager.getGPSManager();

	private static final int LABELS_WIDTH_1 = 90;
	private static final int LABELS_WIDTH_2 = 120;

	//ROW 1
	private Label toleranceLabel = null;
	//ROW 2
	private Label projectionLabel = null;
	private TextField projectionTxtFld = null;
	private Label unitsLabel = null;
	private TextField unitsTxtFld = null;
	//ROW 3
	private Label toleranceXLabel = null;
	private TextField toleranceXTxtFld = null;
	private Label toleranceYLabel = null;
	private TextField toleranceYTxtFld = null;
	private MainWindow parent=null;

	public GPSEditionConfigDialog(Frame parent_frame, FLyrVect lyr){
		super(parent_frame, Messages.getText("GPSPrefsDialog.84"));
		parent = (MainWindow) parent_frame;
		layer = lyr;
		init();
	}


	private void init(){
		//ROW 1
		toleranceLabel = new Label(Messages.getText("GPSPrefsDialog.84")); //$NON-NLS-1$
		toleranceLabel.setSize(LABELS_WIDTH_1, DialogConstants.CONTROL_HEIGHT);

		//ROW 2
		projectionLabel = new Label(Messages.getText("GPSPrefsDialog.85")); //$NON-NLS-1$
		projectionLabel.setSize(LABELS_WIDTH_1, DialogConstants.CONTROL_HEIGHT);

		projectionTxtFld= new TextField(layer.getProjection().getAbrev());
		projectionTxtFld.setSize(LABELS_WIDTH_2, DialogConstants.CONTROL_HEIGHT);
		projectionTxtFld.setEnabled(false);

		unitsLabel= new Label(Messages.getText("GPSPrefsDialog.86")); //$NON-NLS-1$
		unitsLabel.setSize(LABELS_WIDTH_1, DialogConstants.CONTROL_HEIGHT);

		unitsTxtFld = new TextField(layer.getProjection().getUnitsAbbrev());
		unitsTxtFld.setSize(LABELS_WIDTH_2, DialogConstants.CONTROL_HEIGHT);
		unitsTxtFld.setEnabled(false);

		//ROW 3
		toleranceXLabel = new Label(Messages.getText("GPSPrefsDialog.87"));
		toleranceXLabel.setSize(LABELS_WIDTH_1, DialogConstants.CONTROL_HEIGHT);

		toleranceXTxtFld = new TextField();
		toleranceXTxtFld.setSize(LABELS_WIDTH_2, DialogConstants.CONTROL_HEIGHT);
		toleranceXTxtFld.setText("0");

		toleranceYLabel = new Label(Messages.getText("GPSPrefsDialog.88"));
		toleranceYLabel.setSize(LABELS_WIDTH_1, DialogConstants.CONTROL_HEIGHT);

		toleranceYTxtFld = new TextField();
		toleranceYTxtFld.setSize(LABELS_WIDTH_2, DialogConstants.CONTROL_HEIGHT);
		toleranceYTxtFld.setText("0");

		//TAB 1
		ExpansibleScrolledPanel ep1 = new ExpansibleScrolledPanel();
		ExpansibleRow row;

		row = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		// add controls to row, the last one horizontally expandable
		row.add(projectionLabel);
		row.add(projectionTxtFld);
		row.add(unitsLabel);
		row.add(unitsTxtFld);
		ep1.add(row);
		// add row to dialog, not expandable
		row = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row.add(toleranceXLabel);
		row.add(toleranceXTxtFld);
		row.add(toleranceYLabel);
		row.add(toleranceYTxtFld);
		ep1.add(row);


		this.addTab(Messages.getText("GPSPrefsDialog.84"), ep1);

	}


	public void mousePressed(MouseEvent e) {
		if(e.getSource().equals(okButton)){
			double lon = 0; 
			double lat = 0;
			try{
				lon = Double.parseDouble(toleranceXTxtFld.getText());
				lat = Double.parseDouble(toleranceYTxtFld.getText());
				
				if(unitsTxtFld.getText().equals("")){
					if(lon < -180 || lon > 180 || lat < -90 || lat > 90){
						logger.error("Incorrect values to tolerance.");
						
						AcceptCancelDialog.informUser(
								Messages.getText("MainWindow.100"), 
								Messages.getText("GPSEditionConfigDialog.1"),
								parent);
					}
					else {
						gpsMananger.setToleranceX(lon);
						gpsMananger.setToleranceY(lat);
						super.mousePressed(e);
					}
				}else{
					gpsMananger.setToleranceX(lon);
					gpsMananger.setToleranceY(lat);
					super.mousePressed(e);
				}
			}catch(NumberFormatException ex){
				AcceptCancelDialog.informUser(
						Messages.getText("MainWindow.100"), 
						Messages.getText("GPSEditionConfigDialog.1"),
						parent);
			}
			
			parent.doAfterGPSEditionDialog();
		}
		
		if(e.getSource().equals(cancelButton))
			super.mousePressed(e);

	}




}
