/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ib��ez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integraci�n de Tecnolog�as SL	*
 *   Conde Salvatierra de �lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*
 ************************************************/

package es.prodevelop.gvsig.mobile.gui.canvas;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Panel;
import java.awt.Rectangle;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.location.tracklog.TracklogManager;

import es.prodevelop.geodetic.utils.conversion.GeoUtils;
import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.MapContext;
import es.prodevelop.gvsig.mobile.fmap.Refreshable;
import es.prodevelop.gvsig.mobile.fmap.core.FPolygon2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolyline2D;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.acetate.MapCursor;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.ecw.EcwReader;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectionEvent;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectionListener;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectionSupport;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.util.Defaults;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.cancel.Cancellable;
import es.prodevelop.gvsig.mobile.fmap.viewport.ExtentEvent;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPortListener;
import es.prodevelop.gvsig.mobile.gui.bar.StatusBar;
import es.prodevelop.gvsig.mobile.gui.dialog.AcceptCancelDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.control.LayersTable;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;
import es.prodevelop.gvsig.mobile.gui.tool.GroupToolbar;
import es.prodevelop.gvsig.mobile.gui.tool.SplashWidget;
import es.prodevelop.gvsig.mobile.gui.tool.TBButton;
import es.prodevelop.gvsig.mobile.gui.tool.TBWidget;
import es.prodevelop.gvsig.mobile.gui.tool.ToolBar;
import es.prodevelop.gvsig.mobile.gui.tool.ToolBarGroupButton;
import es.prodevelop.gvsig.mobile.gui.tool.maplistener.MapToolListener;
import es.prodevelop.gvsig.mobile.gui.tool.util.Animable;
import es.prodevelop.gvsig.mobile.project.Project;

/**
 * This class is the <i>canvas</i> of the application, where layers and drawn.
 * It receives events from: the viewport (extent events), the mapcontext (selection events)
 * and the GPS device (when new GPS data is available).
 *
 * It holds also a reference to the tools (toolbars) and owns the project and the TOC.
 *
 * @see es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort
 * @see es.prodevelop.gvsig.mobile.fmap.MapContext
 * @see org.gvsig.mobile.location.listener.GPSManagementEventsListener
 * 
 * @author jldominguez
 *
 */
public class MapControl extends Panel implements ViewPortListener, Refreshable, Animable,
SelectionListener, ComponentListener {

	private static Logger logger = Logger.getLogger(MapControl.class);
	
	// public MapCursor gps_cursor;
	private MapCursorSupport theMapCursorSupport;

	private SplashWidget processingSplash;
	private Cancellable canc = new Cancellable();
	public Project project = new Project();
	private LayersTable toc;

	private Graphics2D mapGraphics = null;

	// --------- cached image
	private BufferedImage image = null;
	private boolean panMode = false;
	private boolean dontRedraw = false;
	private Image panModeImage = null;
	private Graphics panModeImageGraphics = null;
	// -----------------------

	private TransparentCanvas symbolsCanvas;

	/**
	 * A reference to the group button tool bar
	 */
	public GroupToolbar groupButtons  = new GroupToolbar();
	private MapToolListener toolListener;
	private MainWindow mainWindow;
	// -----------------------
	private int imageOffsetX = 0;
	private int imageOffsetY = 0;

	public void step(int qty){
		groupButtons.step(qty);
	}

	/**
	 * Overriden method to refresh component.
	 */
	public void update(Graphics g){
		
		if (g != null) {
			paint(g);
		} else {
			logger.warn("Did not update graphics (is NULL)");
		}
		
	}

	/**
	 * Set a new tool bar as the one selected.
	 *
	 * @param t the tool bar to be set as selected
	 * @param w canvas width (unused parameter)
	 */
	public void setToolBar(ToolBar t, int w) {

		ToolBarGroupButton tbgb = null;
		for (int i=0; i<groupButtons.size(); i++) {
			tbgb = (ToolBarGroupButton) groupButtons.get(i);
			tbgb.setSelected(false);
		}

		t.getGroupButton().setSelected(true);

		int count = getComponentCount();
		for (int i=0; i<count; i++) {
			Component c = getComponent(i);
			if (c instanceof TBWidget) ((TBWidget) c).setVisible(false);
			//TODO Make more generic*/
		}

		//Set Toolbar widgets to visible
		t.setButtonsVisible(true);

		// visible empty + help:
		t.getHelpButton().setVisible(true);
	}

	private void refreshGroupButtons(int a, int b) {
		refresh(0, a * GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_HEIGHT,
				GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_WIDTH,
				GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_HEIGHT);
		refresh(0, b * GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_HEIGHT,
				GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_WIDTH,
				GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_HEIGHT);
	}

	/**
	 * Constructor. Initializes the TOC, all the listeners and the GPS device.
	 *
	 * @param mw the main window that owns this control
	 * @param w the width of the main window that owns this control
	 */
	public MapControl(MainWindow mw, int w) {

		mainWindow = mw;
		toc = new LayersTable(
				w - (GvsigMobileGUIConstants.ICON_SIZE_SMALL + 2 * TocButton.MARGIN),
				this);

		// the toc listens to extent changes to update scale visibility icons:
		project.getMapContext().getViewPort().addViewPortListener(toc);

		setSize(mw.getWidth(), mw.getHeight());

		// ----------------------- tools
		toolListener = new MapToolListener(this);
		addMouseListener(toolListener);
		addMouseMotionListener(toolListener);

		addKeyListener(mw);
		// -----------------------
		addComponentListener(this);


		initialize();
	}

	private void initialize() {
		logger.debug(Utils.time() + "Start MapControl Initialize");
		setBackground(Color.WHITE);
		project.getMapContext().addViewPortLisneter(this);

		theMapCursorSupport = new MapCursorSupport(this);
		// gps_cursor = new MapCursor(IconResourceReader.GPS_CURSOR_ICON[0]);
		// gps_cursor.setVisible(false);
		theMapCursorSupport.addCursor(this);


//		processingSplash = new SplashWidget(IconResourceReader.PROCESSING_ICON,
//				GvsigMobileGUIConstants.PROCESS_ICON_SIZE,
//				GvsigMobileGUIConstants.PROCESS_ICON_SIZE);
		
		processingSplash = new SplashWidget(
				Messages.getText("splash"),
				Color.DARK_GRAY,Color.WHITE, getWidth());
		
		
		processingSplash.setVisible(false);
		
		processingSplash.setPos((getWidth() - processingSplash.getWidth())/2,
				( getHeight() - processingSplash.getHeight())/2);
		

		setLayout(null);
		add(processingSplash);
	}

	/**
	 * Gets parent main window.
	 *
	 * @return parent main window
	 */
	public MainWindow getParentFrame() {
		return mainWindow;
	}

	/**
	 * Synchronized method to avoid simultaneous calls from different threads, which would cause
	 * problems due to the use of temporary transformations in Graphics object.
	 * 
	 * @param g
	 */
	private synchronized void synchronizedPaint(Graphics g) {
		if (dontRedraw){
			processingSplash.paint(getGraphics());
		} else  {

			if (panModeImageGraphics == null) {
				// too soon to paint (?)
				// (to avoid null pointer exception sometimes on PC)
				return;
			}

			panModeImageGraphics.clearRect(0, 0, getWidth(), getHeight());
			if (!panMode) {
				imageOffsetX=0;
				imageOffsetY=0;
			}
			panModeImageGraphics.drawImage(image, imageOffsetX, imageOffsetY, null);

			if (!panMode) {

				try {
					getMapContext().getAcetateLayer().draw(
							(Graphics2D) panModeImageGraphics,
							getMapContext().getViewPort());
				} catch (FMapDriverException e) {
					logger.error("While drawing acetate: " + e.getMessage());
				}
			}

			// GPS Cursor moves while panning too
			theMapCursorSupport.drawCursor(panModeImageGraphics, imageOffsetX, imageOffsetY);
			paintToolsImagesHere(panModeImageGraphics);


			if(panMode){

				if (symbolsCanvas != null) {
					panModeImageGraphics.translate(imageOffsetX,imageOffsetY);
					symbolsCanvas.paint(panModeImageGraphics);
					panModeImageGraphics.translate(-imageOffsetX,-imageOffsetY);
				}
			} else {
				symbolsCanvas.paint(panModeImageGraphics);
			}
			
			g.drawImage(panModeImage, 0, 0, null);
		}
	}
	/**
	 * Overriden paint method. Deals with: pan mode (drag-and-drop the map), paints the tools
	 * and the GPS cursor if necessary.
	 */
	public void paint(Graphics g) {
		synchronizedPaint(g);
	}

	/**
	 * Method used to warn this components that the pan tool starts to be in use,
	 * (mouse down), so drawing method will behave in a different way.
	 *
	 * @param set whether or not the pan tool is in use
	 */
	public void setToolsImageForPanDrag(boolean set) {
		panMode = set;
	}

	private void paintToolsImagesHere(Graphics ig) {

		Component item;
		int offx, offy;

		for (int i = 0; i < getComponentCount(); i++) {
			item = getComponent(i);
			if (item.isVisible() && !(item instanceof MapCursor)) {
				offx = item.getX();
				offy = item.getY();
				ig.translate(offx, offy);
				item.paint(ig);
				ig.translate(-offx, -offy);
			}
		}

		// ------------- paint empty buttons --------------
		ToolBar selToolBar = mainWindow.getSelectedToolBar();
		if (selToolBar != null) selToolBar.paintEmptyButtons(ig);
		// ------------- paint empty buttons --------------

	}


	/**
	 * This method initializes the graphics of this component.
	 * @param initial Whether it is the application startup
	 */
	public void startGraphics(boolean initial) {
		logger.debug(Utils.time() + "startGraphics init");
		
		int _w = getWidth();
		int _h = getHeight();

		if ((_w < 1) || (_h < 1)) {
			logger.error("OM - MapControl start graphics - RETURNED before instantiating image!!");
			return;
		}
		
		image = Utils.createBufferedImage(_w, _h);
		mapGraphics = (Graphics2D) image.getGraphics();
		// mapGraphics.crea
		mapGraphics.setClip(0, 0, _w, _h);
		project.getMapContext().setViewPortImageSize(getSize());
		panModeImage = Utils.createBufferedImage(_w, _h); // createImage(_w, _h);
		// try { Thread.sleep(50); } catch (InterruptedException e) { }
		panModeImageGraphics = panModeImage.getGraphics();
		logger.debug(Utils.time() + "image created");
		
		// ---- transparent canvas:
		if(symbolsCanvas == null){
			symbolsCanvas = TransparentCanvas.getInstance();
			logger.debug(Utils.time() + " Instancia del Canvas " + symbolsCanvas.toString());
			symbolsCanvas.setViewPort(project.getMapContext().getViewPort());
		}
		else{
			if(initial){
				symbolsCanvas.clear();
				symbolsCanvas.clearEdition();
				logger.debug(Utils.time() + " Canvas Inicial " + symbolsCanvas.toString());				
			}
			else{
				symbolsCanvas.setViewPort(project.getMapContext().getViewPort());
			    logger.debug(Utils.time() + " Establecido el ViewPort " + symbolsCanvas.toString());
			}
		}
//		if (initial|| (symbolsCanvas == null)) {
//			symbolsCanvas = new TransparentCanvas(project.getMapContext().getViewPort());
//		} else {
//			symbolsCanvas.setViewPort(project.getMapContext().getViewPort());
//		}
	}

	/**
	 * Gets current projection (gets the data from the project)
	 * @return the current projection
	 */
	public IProjection getProjection() {
		return project.getMapContext().getProjection();
	}

	/**
	 * Sets the current projection (sets the data in the project)
	 * @param proj the new projection
	 */
	public void setProjection(IProjection proj) {
		project.getMapContext().setProjection(proj);
	}

	/**
	 * Used to cancel the drawing. Currently unused.
	 *
	 */
	public void cancelDrawing() {
		canc.setCanceled(true);
	}

	/**
	 * Main method to draw the map.
	 *
	 * @param doClear whether the canvas has to be cleared before drawing.
	 * @return whether there was more than one layer
	 */
	public boolean draw(boolean doClear) {

		boolean do_something = (project.getMapContext().getLayerCount() > 0);

		if (true) {
			clearCanvas();
		}
		if (do_something) {
			setProcessing(true);

			project.getMapContext().draw(mapGraphics, canc);

		}

		setProcessing(false);
		refresh(false);
		return do_something;
	}

	/**
	 * Get the viewport
	 *
	 * @return the viewport
	 */
	public ViewPort getViewPort() {
		return project.getMapContext().getViewPort();
	}

	/**
	 * Called when the extent changes (part of viewport listener)
	 */
	public void extentChanged(ExtentEvent e, boolean repaint) {
		if (repaint) draw(true);
	}

	/**
	 * Used to refresh the map
	 * @param update where to use the update method or just the repain
	 */
	public void refresh(boolean update) {

		if (getGraphics() == null) {
			logger.debug("Graphics is null, did not refresh ...");
			return;
		}
		
		if (update) {
			update(getGraphics());
		} else {
			repaint();
		}
	}

	/**
	 * Forces a refresh fo the <code>MapControl</code> given the top-left corner
	 * and the size
	 * @param x	X coordinate of the left border area to refresh
	 * @param y Y coordinate of the top border area to refresh
	 * @param w Width of the area to refresh
	 * @param h Height of the area to refresh
	 */
	public void refresh(int x, int y, int w, int h) {
		repaint(x, y, w, h);
	}

	/**
	 * Forces a refresh fo the <code>MapControl</code> given a rectangle with
	 * the area to refresh
	 * @param rectangle Area to be refreshed
	 */
	public void refresh(Rectangle rectangle) {
		refresh(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
	}

	/**
	 * Closes current datasources. It's called whenm application terminates.
	 *
	 */
	public void closeDataSources() {
		project.getMapContext().closeDataSources();
	}

	/**
	 * Add a new layer to the map.
	 *
	 * @param lyr the new layer to be added
	 * @param repaint whether or not to repaint after adding the layer
	 */
	public void addLayer(FLayer lyr, boolean repaint) {
		project.setModified(true);
		project.getMapContext().addLayer(lyr, repaint);

		if (lyr instanceof FLyrVect) {
			try {
				FLyrVect lyrv = (FLyrVect) lyr;
				SelectableDataSource ds = lyrv.getRecordset();
				SelectionSupport ss = ds.getSelectionSupport();
				ss.addSelectionListener(this);
			} catch (FMapDriverException e) {
				logger.error("While adding MapControl as selection listener.");
			}
		}
		toc.addItem(lyr, false);
	}

	/**
	 * Remove one layer from the current map.
	 *
	 * @param lyr the layer to be removed
	 */
	public void removeLayer(FLayer lyr) {
		project.setModified(true);
		project.getMapContext().removeLayer(lyr);
		redraw(true);
	}

	/**
	 * Swap order of the layers (arrow buttons in TOC)
	 *
	 * @param a 0-based index of one of the layers
	 * @param b 0-based index of the other layer
	 * @param repaint whether or not the map has to be repainte
	 */
	public void swap(int a, int b, boolean repaint) {
		project.getMapContext().swapFromBottom(a, b);
		if (repaint) {
			redraw(true);
		}
	}

	/**
	 * Gets number of layers in current map
	 * @return number of layers in current map
	 */
	public int getLayerCount() {
		return project.getMapContext().getLayerCount();
	}

	/**
	 * Gets i-th layer
	 * @param index index of the desired layer
	 * @return i-th layer
	 */
	public FLayer getLayer(int index) {
		return project.getMapContext().getLayer(index);
	}

	/**
	 * Gets currently selected layer (in TOC), or null if there is no selection
	 * @param only_vect whether we are interested only in vector layers
	 * @return currently selected layer (in TOC), or null if there is no selection
	 */
	public FLayer getSelectedLayer(boolean only_vect) {
		FLayer[] sel = project.getMapContext().getSelectedLayers(only_vect);

		if (sel == null) return null;
		if (sel.length == 1) return sel[0];
		return null;
	}

	/**
	 * Sets a new active button by assigning it to a new Listener
	 * @param button <code>TBButton</code> to set active and assign to the Listener
	 * @param clearStatusBar whether the status bar has to be cleared
	 */
	public void setListener(TBButton button, boolean clearStatusBar){
		toolListener.setTool(button, clearStatusBar);
	}

	/**
	 * Returns the active Tool associated to the Listener in MapToolListener
	 * <p>Only tools of type <code>TBSelectableTool</code> (subclass of
	 * <code>TBButton</code>) registers listeners, so only this
	 * objects will be real active tools
	 * @return the current tool (must be of type <code>TBButton</code>)
	 */
	public TBButton getCurrentTool(){
		return toolListener.getCurrentTool();
	}

	/**
	 * Gets the map context object
	 * @return the map context object
	 */
	public MapContext getMapContext() {
		return project.getMapContext();
	}

	/**
	 * Gets the image of the current map
	 * @return the image of the current map
	 */
	public Image getCurrentImage() {
		return image;
	}

	/**
	 * Sets a new offset for the image (used with pan tool)
	 */
	public void setOffsetAndRefresh(int offsetx, int offsety, boolean refresh) {
		imageOffsetX = offsetx;
		imageOffsetY = offsety;
		logger.debug("MapControl - setOffsetAndRefresh, imageOffsetX = " + imageOffsetX);
		logger.debug("MapControl - setOffsetAndRefresh, imageOffsetY = " + imageOffsetY);
		if (refresh) {
			update(getGraphics());
		}
	}

	/**
	 * Gets a row of pixels from the current image map.
	 *
	 * @param x x coordinate of the beginning of the row
	 * @param y y coordinate of the beginning of the row
	 * @param len length of the row
	 * @return array of RGBA values
	 */
	public int[] getCurrentRow(int x, int y, int len) {
		return image.getRGB(x, y, len, 1, null, 0, len);
	}

	/**
	 * Gets a column of pixels from the current image map.
	 *
	 * @param x x coordinate of the beginning of the column
	 * @param y y coordinate of the beginning of the column
	 * @param len length of the column
	 * @return array of RGBA values
	 */
	public int[] getCurrentColumn(int x, int y, int len) {
		return image.getRGB(x, y, 1, len, null, 0, 1);
	}

	/**
	 * Sets RGBA values in a row of the map
	 * @param x x coordinate of the beginning of the row
	 * @param y y coordinate of the beginning of the row
	 * @param pix array of RGBA values to be written
	 */
	public void setCurrentRow(int x, int y, int[] pix) {
		image.setRGB(x, y, pix.length, 1, pix, 0, pix.length);
	}

	/**
	 * Sets RGBA values in a column of the map
	 * @param x x coordinate of the beginning of the column
	 * @param y y coordinate of the beginning of the column
	 * @param pix array of RGBA values to be written
	 */
	public void setCurrentColumn(int x, int y, int[] pix) {
		image.setRGB(x, y, 1, pix.length, pix, 0, 1);
	}

	/**
	 * Compute the XOR pixels of a region of the map. Uses signed width and height
	 * to be more flexible
	 *
	 * @param x x coordinate of the origin of the zone
	 * @param y y coordinate of the origin of the zone
	 * @param signed_w signed width of the region
	 * @param signed_h signed height of the region
	 */
	public void xorAndRepaint(int x, int y, int signed_w, int signed_h) {

		int w = Math.abs(signed_w);
		int h = Math.abs(signed_h);

		int minx = Math.min(x, x + signed_w);
		int miny = Math.min(y, y + signed_h);

		int size = Math.abs(signed_w * signed_h);
		int[] pixels = new int[size];

		image.getRGB(minx, miny, w, h, pixels, 0, w);
		// ~ = NOT
		for (int i=0; i<size; i++) pixels[i] = ~(pixels[i] & 0xffffff);
		image.setRGB(minx, miny, w, h, pixels, 0, w);

		repaint(minx, miny, w, h);
	}

	/**
	 * Clears this component with the backgrouond color
	 *
	 */
	public void clearCanvas() {
		
		if (mapGraphics == null) {
			return;
		}
		
		mapGraphics.setColor(project.getBgColor());
		mapGraphics.fillRect(0, 0, getWidth(), getHeight());
	}

	/**
	 * Adds a new group button in the group button bar (left side of the window)
	 * @param gb a new group button to be added
	 * @param b whether the new group button is selected
	 */
	public void addGroupButton(ToolBarGroupButton gb, boolean b) {
		gb.setSelected(b);
		add(gb);
		groupButtons.add(gb);
	}

	public void removeGroupButton(ToolBarGroupButton gb){
		for (int i = 0; i < groupButtons.size(); i++) {
			ToolBarGroupButton aux = (ToolBarGroupButton) groupButtons.get(i);
			if(aux.equals(gb)){
				groupButtons.remove(aux.getIndex());
			}
		}
		((ToolBarGroupButton) groupButtons.get(0)).setSelected(true);
	}

	/**
	 * Called when the selection changed a map has to be refreshed (part of selection
	 * listener)
	 */
	public void selectionChanged(SelectionEvent e) {
		draw(true);
	}

	/**
	 * This method performs the "zoom to selection" action.
	 *
	 */
	public void zoomToSelection() {
		project.getMapContext().zoomToSelection();
	}

	/**
	 * This method performs the "clear selection" action.
	 *
	 */
	public void clearSelection() {
		if (project.getMapContext().clearSelection()) draw(true);
	}


	/**
	 * Used when drawing process begins or ends. The process icon will show.
	 * @param b true when the process begins, false when it ends.
	 */
	public void setProcessing(boolean b) {
		processingSplash.setVisible(b);
		dontRedraw=b;
		
		update(getGraphics());
	}

	/**
	 * Used to know if the canvas (this object) is currently in the drawing process.
	 */
	public boolean isProcessing() {
		return processingSplash.isVisible();
	}

	/**
	 * Gets an array with the layers that are infoable.
	 *
	 * @param only_vect whether only vector layers have to be included
	 *
	 * @return an array with the layers that are infoable.
	 */
	public FLayer[] getInfoableLayers(boolean only_vect) {
		return project.getMapContext().getInfoableLayers(only_vect);
	}

	/**
	 * Used to redraw the map
	 */
	public void redraw(boolean clear) {
		draw(clear);
	}

	/**
	 * Get the TOC object (table of current layers)
	 * @return the TOC object
	 */
	public LayersTable getToc() {
		return toc;
	}

	/**
	 * Sets the new multiline geometry while using the measure distance tool
	 *
	 * @param ig the new multiline geometry
	 * @param ref_minx extent of the geometry in screen pixel coordinates
	 * @param ref_miny extent of the geometry in screen pixel coordinates
	 * @param ref_maxx extent of the geometry in screen pixel coordinates
	 * @param ref_maxy extent of the geometry in screen pixel coordinates
	 * @return whether the map ndeeds to be repainted
	 */
	public boolean setDistanceGeometry(IGeometry ig,
			int ref_minx,
			int ref_miny,
			int ref_maxx,
			int ref_maxy) {

		if (symbolsCanvas == null) {
			return false;
		}

		boolean resp = symbolsCanvas.clear();
		if (ig == null) { return resp; }

		symbolsCanvas.addGeometry(ig);
		repaint(ref_minx, ref_miny, ref_maxx - ref_minx, ref_maxy - ref_miny);
		return false;
	}

	/**
	 * Tells the mapcontrol to write the distance measurements as string  in the status bar
	 *
	 */
	public void setSymbolGeometryLengthAsDistanceInStatusBar() {

		double[] d = {0, 0};
		IGeometry ig = (IGeometry) symbolsCanvas.getGeometry(0);

		if (ig != null) {

			if (ig.getInternalShape() instanceof FPolyline2D) {
				d = GeoUtils.getIteratorLengthAndLast(
						ig.getPathIterator(Utils.IDENTITY_AT),
						project.getProjection().getAbrev());
			}
		}

		int prec = 2;

		mainWindow.setStatusBarText(StatusBar.MEASURES_BAR, Messages.getText("MapControl.9") //$NON-NLS-1$
				+ " = " + Utils.getFormattedDouble(d[0], prec)
				+ Messages.getText("MapControl.10") + " = " //$NON-NLS-1$
				+ Utils.getFormattedDouble(d[1], prec) + Messages.getText("MapControl.11") //$NON-NLS-1$
				);
	}

	/**
	 *
	 * Tells the mapcontrol to write the area measurements as string  in the status bar
	 */
	public void setSymbolGeometryAreaInStatusBar() {

		double a = 0;
		IGeometry ig = (IGeometry) symbolsCanvas.getGeometry(0);

		if (ig != null) {

			if (ig.getInternalShape() instanceof FPolygon2D) {
				a = GeoUtils.getAreaInSquareMeters(
						ig.getPathIterator(Utils.IDENTITY_AT),
						project.getProjection().getAbrev());
			}

		}

		String units = Messages.getText("MapControl.14"); //$NON-NLS-1$
		int prec = 2;
		if (a >= GeoUtils.SQ_METERS_LIMIT) {
			a = a / 1000000;
			units = Messages.getText("MapControl.15"); //$NON-NLS-1$
		}
		mainWindow.setStatusBarText(StatusBar.MEASURES_BAR,
				Messages.getText("MapControl.17") + " = " + Utils.getFormattedDouble(a, prec) + units); //$NON-NLS-1$
	}

	/**
	 * Set text in status bar.
	 *
	 * @param id ID the the status bar
	 * @param txt text to be written
	 */
	public void setStatusBarText(int id, String txt) {
		mainWindow.setStatusBarText(id, txt);
	}
	
	/**
	 * Add new text to StatusBar.
	 * 
	 * @param id
	 * @param txt
	 */
	public void addStatusBarText(int id, String txt) {
		mainWindow.concatStatusBarText(id, txt);
	}

	/**
	 * Set a new icon in the status bar
	 * @param id ID of the status bar
	 * @param dir folder where the image is
	 * @param filename file nameof the icon
	 */
	public void setStatusBarIcon(int id, String dir, String filename) {
		mainWindow.setStatusBarIcon(id,ResourceReader.IMG_SKIN_PATH, filename); //$NON-NLS-1$
	}
	


	/**
	 * Set new data in status bar
	 * 
	 * @param bar_id
	 * @param data_id
	 * @param value
	 */
	public void setStatusBarDataValue(int bar_id, int data_id, Object value) {
		
		mainWindow.setStatusBarDataValue(bar_id, data_id, value);
	}


	/**
	 *
	 * Set a new subicon in the status bar
	 * @param id ID of the status bar
	 * @param dir folder where the image is
	 * @param filename file nameof the subicon
	 */
	public void setStatusBarSubIcon(int id, String dir, String filename) {
		mainWindow.setStatusBarSubIcon(id,ResourceReader.IMG_SKIN_PATH, filename); //$NON-NLS-1$
	}

	// ----------------------------------------------------------------
	// ----------------------------- GPS ------------------------------
	// ----------------------------------------------------------------

	/**
	 * Switch tracklog activation button. Active: start doing the track log,
	 * Inactive: stop
	 */
	public void switchTracklog() {
		logger.debug("Tracklog switch pushed"); //$NON-NLS-1$
		if (TracklogManager.isTracking()) {
			TracklogManager.stopTracking();
			//gpsTracklogSwitch.setLabel("t");
		} else {
			TracklogManager.startTracking();
			//gpsTracklogSwitch.setLabel("T");
		}
	}



	// ----------------------------- GPS END --------------------------
	// ----------------------------------------------------------------

	/**
	 * Saves current map as PNG (tells user output file name)
	 */
	public void saveAsPng() {
		int __w = getWidth();
		int __h = getHeight();
		BufferedImage png = Utils.createBufferedImage(__w, __h);
		paint(png.getGraphics());
		int[] pixels = png.getRGB(0, 0, __w, __h, null, 0, __w);
		byte[] png_bytes = EcwReader.paintArray(pixels, __w, __h, 0, false);


		String img_name = ("image_" + System.currentTimeMillis() % 1000000) + ".png"; //$NON-NLS-1$ //$NON-NLS-2$
		boolean done = Utils.writeBytesToFile(png_bytes,
				Defaults.WORKSPACE
				+ File.separator + "screenshots" + File.separator //$NON-NLS-1$
				+ img_name);

		if (done) {
			AcceptCancelDialog.informUser(
					Messages.getText("MapControl.33"),
					Messages.getText("MapControl.32") //$NON-NLS-2$
					+ ": " + img_name,
					mainWindow);
		} else {
			AcceptCancelDialog.informUser(Messages.getText("MapControl.33"), Messages.getText("MapControl.34"), mainWindow); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	public void componentResized(ComponentEvent arg0) {
		
		if (processingSplash!=null &&
				getWidth() > 0 &&
				processingSplash.getWidth() > 0 &&
				getHeight() > 0 &&
				processingSplash.getHeight() > 0)
			processingSplash.setPos(( getWidth() - processingSplash.getWidth())/2,
				( getHeight() - processingSplash.getHeight())/2);
	}

	public void componentShown(ComponentEvent arg0) {
		componentResized(arg0);
	}


	public void componentHidden(ComponentEvent arg0) { }
	public void componentMoved(ComponentEvent arg0) { }

	/**
	 * Gets the current project
	 * @return the current project
	 */
	public Project getProject() {
		return project;
	}

	/**
	 * Sets the current project
	 * @param project the new project
	 */
	public void setProject(Project project) {
		this.project = project;
		project.getMapContext().addViewPortLisneter(this);
		toc = new LayersTable(toc.getWidth(), this);
		project.getMapContext().getViewPort().addViewPortListener(toc);

		Iterator it = project.getMapContext().getLayers().iterator();
		while (it.hasNext()){
			FLayer lyr = (FLayer)it.next();
			if (lyr instanceof FLyrVect) {
				try {
					((FLyrVect) lyr).getRecordset().getSelectionSupport().addSelectionListener(this);
				} catch (FMapDriverException e) {
					// TODO Auto-generated catch block
					logger.error("While adding MapControl as selection listener.");
				}
			}
			toc.addItem(lyr, false);
		}
		
		startGraphics(false);
		redraw(true);
	}

	/**
	 * Gets the object where measurement symbols are drawn.
	 * @return the object where measurement symbols are drawn.
	 */
	public TransparentCanvas getSymbolsCanvas() {
		return symbolsCanvas;
	}


	/**
	 * Tells the map control to recenter the map at the given pixel. It's the screen pixel
	 * where the user has clicked while using the recenter tool
	 *
	 * @param x_ coordinates to recenter to (screen pixel reference system)
	 * @param y_ coordinates to recenter to (screen pixel reference system)
	 */
	public void recenter(int x_, int y_){
		Rectangle2D oldext = getViewPort().getAdjustedExtent();
		if (oldext == null) {
			return;
		}

		// ------------------------- starts to work
		if (isProcessing()) {
			return;
		} else {
			setProcessing(true);
		}


		int src_x = x_;
		int src_y = y_;

		logger.debug(Utils.time() + "CenterOnPointListener, point" + " =" + src_x + ", " + src_y); //$NON-NLS-1$ //$NON-NLS-2$

		Point2D geop = getViewPort().toMapPoint(src_x, src_y);

		double half_w = 0.5 * oldext.getWidth();
		double half_h = 0.5 * oldext.getHeight();
		Rectangle2D.Double newext = new Rectangle2D.Double(
				geop.getX() - half_w, geop.getY() - half_h, 2 * half_w, 2 * half_h);
		getViewPort().setExtent(newext, true);

	}


	public MapCursorSupport getMapCursorSupport() {
		return theMapCursorSupport;
	}

	public void clearEditionCanvas(){
		panModeImageGraphics.clearRect(0,0,getWidth(),getHeight());
	}

	public boolean repaintEdition(int ref_minx,
			int ref_miny,
			int ref_maxx,
			int ref_maxy){
		symbolsCanvas.clearEdition();

		repaint(ref_minx, ref_miny, ref_maxx - ref_minx, ref_maxy - ref_miny);
		return false;
	}

}
