/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;


import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.Color;
import java.awt.FileDialog;
import java.awt.Label;
import java.awt.TextField;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.geodetic.utils.conversion.GeoUtils;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ColorChooserPanel;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;
import es.prodevelop.gvsig.mobile.gui.tab.ExpansibleScrolledPanel;
import es.prodevelop.gvsig.mobile.gui.tab.TabbedDialog;
import es.prodevelop.gvsig.mobile.project.Project;
import es.prodevelop.gvsig.mobile.settings.AppSettings;

/**
 * This dialog shows app current settings and allows changes in them: default view
 * settings, startup project and language.
 * 
 * @see es.prodevelop.gvsig.mobile.settings.AppSettings
 * 
 * @author jldominguez
 *
 */

public class AppSettingsDialog extends TabbedDialog implements ItemListener  {
	
	private static Logger logger = Logger.getLogger(AppSettingsDialog.class);
    
    // frame component needed to open further dialogs
    private MainWindow parent = null;

    // ----------------------------------------------------------------
    // ---------------------- DEFAULT SETTINGS -------------------------
    // 1st row
    private Label srsLabel;
    private Choice srsChoice;
    
    // 2nd row
    private Label bboxLabel;

    // 3rd row
    private Label bboxLabel_x;
    private TextField bboxText_xmin, bboxText_xmax;
    
    // 4th row
    private Label bboxLabel_y;
    private TextField bboxText_ymin, bboxText_ymax;
    
    // 5th row
    private Label bgLabel;
    private ColorChooserPanel bgComponent;
    
    //Icon size factor
    private Label sizeLabel;
    private Choice sizeChoice;
    
    // 6th row
    private Label pathLabel;
    private TextField pathText;
    private TocButton pathButton;
    
    // 7th row is empty and vertically expandable 
    // ---------------------- DEFAULT SETTINGS -------------------------
    // ----------------------------------------------------------------
    
    // ----------------------------------------------------------------
    // ---------------------- DEFAULT PROJECT -------------------------

    private Label useDefProjectLabel;
    private Checkbox useDefProjectChk;
    
    private Label defProjectLabel;
    private TextField defProjectText;
    private TocButton defPathButton;
    private TocButton thisDefButton;
    // ---------------------- DEFAULT PROJECT -------------------------
    // ----------------------------------------------------------------
    
    // ----------------------------------------------------------------
    // ---------------------- LANGUAGE  -------------------------
    // 1st row
    private Label langLabel;
    private Choice langChoice;

    /**
     * Constructor.
     * @param parent_frame parent (main) window
     */
    public AppSettingsDialog(MainWindow parent_frame) {
    	
        super(parent_frame, Messages.getText("AppSettingsDialog.0")); //$NON-NLS-1$
        setBounds(
                parent_frame.getX(),
                parent_frame.getY(),
                parent_frame.getWidth(),
                parent_frame.getHeight());

        // Set parent frame which is needed to open further dialogs
        parent = parent_frame;
        
        // this.setUndecorated(true);
        // this.setModal(true); // WINDOW-DIALOG

        init();
    }
    
    private void init() {
    	
    	int coord_text_w =
    		(parent.getWidth() - DialogConstants.LABELS_WIDTH - 4 * DialogConstants.XSPACE - 8) / 2;
        
        // --------- create controls: ----------------
        // 1st row
        srsLabel = new Label(Messages.getText("AppSettingsDialog.1")); //$NON-NLS-1$
        srsLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        srsLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        srsChoice = getSrsChoice();
        srsChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);
        srsChoice.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        // 2nd row
        bboxLabel = new Label(Messages.getText("AppSettingsDialog.2")); //$NON-NLS-1$
        bboxLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
        bboxLabel.setAlignment(Label.CENTER);
        bboxLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        // 3rd row
        bboxLabel_x = new Label(Messages.getText("AppSettingsDialog.3")); //$NON-NLS-1$
        bboxLabel_x.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        bboxLabel_x.setFont(DialogConstants.MENU_FONT);
        bboxText_xmin = new TextField();
        bboxText_xmin.setSize(coord_text_w, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        bboxText_xmin.setFont(DialogConstants.DIALOG_SMALL_FONT);
        bboxText_xmax = new TextField();
        bboxText_xmax.setSize(coord_text_w, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        bboxText_xmax.setFont(DialogConstants.DIALOG_SMALL_FONT);
        // 4th row
        bboxLabel_y = new Label(Messages.getText("AppSettingsDialog.4")); //$NON-NLS-1$
        bboxLabel_y.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        bboxLabel_y.setFont(DialogConstants.MENU_FONT);
        bboxText_ymin = new TextField();
        bboxText_ymin.setSize(coord_text_w, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        bboxText_ymin.setFont(DialogConstants.DIALOG_SMALL_FONT);
        bboxText_ymax = new TextField();
        bboxText_ymax.setSize(coord_text_w, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        bboxText_ymax.setFont(DialogConstants.DIALOG_SMALL_FONT);
        // 5th row
        bgLabel = new Label(Messages.getText("AppSettingsDialog.5")); //$NON-NLS-1$
        bgLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        bgLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        bgComponent = new ColorChooserPanel(Color.BLACK, parent);
        
        sizeLabel = new Label(Messages.getText("Iconos")); //$NON-NLS-1$
        sizeLabel.setSize((4 * DialogConstants.LABELS_WIDTH) / 3, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        sizeLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        sizeChoice = getSizeChoice();
        sizeChoice.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        sizeChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);
        
        // 6th row
        pathLabel = new Label(Messages.getText("AppSettingsDialog.6")); //$NON-NLS-1$
        pathLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        pathLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        pathText = new TextField();
        pathText.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        pathText.setFont(DialogConstants.DIALOG_SMALL_FONT);
        pathButton = new TocButton(
        		"folder_transp.png", //$NON-NLS-1$
        		true);
        pathButton.addMouseListener(this);
        
        
        
        // tab 3 language
        langLabel = new Label(Messages.getText("Language")); //$NON-NLS-1$
        langLabel.setSize((4 * DialogConstants.LABELS_WIDTH) / 3, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        langLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        langChoice = getLangChoice();
        langChoice.setSize(DialogConstants.TABLE_LABEL_HEIGHT, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        langChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);
        
        // ---------------------
        ExpansibleScrolledPanel ep3 = new ExpansibleScrolledPanel();
        // ---------------------
        ExpansibleRow row3_1 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        row3_1.add(langLabel); row3_1.addExtensible(langChoice);
        ep3.add(row3_1);
        // ---------------------
        ExpansibleRow row_size = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        row_size.add(sizeLabel); row_size.addExtensible(sizeChoice);
        ep3.add(row_size);
        // -----------------------
        addTab(Messages.getText("General"), ep3);
        
        // ---------------------------------------
        // ---------------------------------------
        

        // ------------ tab 1 --------------
        ExpansibleScrolledPanel ep = new ExpansibleScrolledPanel();
        // ---------------------
        ExpansibleRow row1; row1 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        row1.add(srsLabel); row1.addExtensible(srsChoice);
        ep.add(row1);
        // ---------------------
        ExpansibleRow row5; row5 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        row5.add(pathLabel);
        row5.addExtensible(pathText);
        row5.add(pathButton);
        ep.add(row5);
        // ---------------------
        ExpansibleRow row6; row6 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        row6.add(bgLabel); row6.add(bgComponent);
        
        ep.add(row6);
        // ---------------------
        ExpansibleRow row2; row2 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        row2.addExtensible(bboxLabel); 
        ep.add(row2);
        // ---------------------
        ExpansibleRow row3; row3 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        row3.add(bboxLabel_x); row3.add(bboxText_xmin);
        row3.addExtensible(bboxText_xmax);
        ep.add(row3);
        // ---------------------
        ExpansibleRow row4; row4 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        row4.add(bboxLabel_y); row4.add(bboxText_ymin);
        row4.addExtensible(bboxText_ymax);
        ep.add(row4);
        // ---------------------
        ExpansibleRow row7; row7 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        ep.addExpansible(row7);
        // ---------------------
        
        // --------------------
        addTab(Messages.getText("AppSettingsDialog.8"), ep); //$NON-NLS-1$
        
        // ------------- tab 2 ------------
        useDefProjectLabel = new Label(Messages.getText("AppSettingsDialog.9")); //$NON-NLS-1$
        useDefProjectLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        useDefProjectLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        
        useDefProjectChk = new Checkbox();
        useDefProjectChk.addItemListener(this);
        useDefProjectChk.setSize(DialogConstants.TABLE_LABEL_HEIGHT, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        
        defProjectLabel = new Label(Messages.getText("AppSettingsDialog.10")); //$NON-NLS-1$
        defProjectLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        defProjectLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        
        defProjectText = new TextField();
        defProjectText.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        defProjectText.setFont(DialogConstants.DIALOG_SMALL_FONT);
        
        defPathButton = new TocButton("folder_transp.png", true); //$NON-NLS-1$
        defPathButton.addMouseListener(this);
        thisDefButton = new TocButton("pro_set_default.png", true); //$NON-NLS-1$
        thisDefButton.addMouseListener(this);

        // ---------------------
        ExpansibleScrolledPanel ep2 = new ExpansibleScrolledPanel();
        // ---------------------
        ExpansibleRow row2_1 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        row2_1.add(useDefProjectLabel); row2_1.add(useDefProjectChk);
        ep2.add(row2_1);
        // ---------------------
        ExpansibleRow row2_2 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        row2_2.add(defProjectLabel);
        row2_2.addExtensible(defProjectText);
        row2_2.add(defPathButton);
        row2_2.add(thisDefButton);
        ep2.add(row2_2);
        
        // ----------------------------------------
        addTab(Messages.getText("AppSettingsDialog.13"), ep2); //$NON-NLS-1$
        
        // ----------------------------------------
        // ----------------------------------------
        

        // ----------------- SET VALUES -------------------
        setIfPresent(srsChoice, AppSettings.defaultSRS);
        bgComponent.setColor(AppSettings.defaultBgColor);
        setIfPresent(sizeChoice, mapToChoice(AppSettings.sizeFactor));
        bboxText_xmin.setText("" + AppSettings.defaultBBox.getMinX()); //$NON-NLS-1$
        bboxText_xmax.setText("" + AppSettings.defaultBBox.getMaxX()); //$NON-NLS-1$
        bboxText_ymin.setText("" + AppSettings.defaultBBox.getMinY()); //$NON-NLS-1$
        bboxText_ymax.setText("" + AppSettings.defaultBBox.getMaxY()); //$NON-NLS-1$
        pathText.setText(AppSettings.defaultSavePath.toString());
        
        // tab 2
        File df = AppSettings.defaultProject;
        if (df == null) {
        	defProjectText.setText(""); //$NON-NLS-1$
        } else {
        	defProjectText.setText(df.getAbsolutePath());
        }
        useDefProjectChk.setState(AppSettings.useDefaultProject);
        defProjectText.setEnabled(AppSettings.useDefaultProject);
        
        // lang
        setIfPresent(langChoice, AppSettings.localeLanguage);
        
    }

    private Choice getLangChoice() {
    	
    	if (langChoice == null) {
    		langChoice = new Choice();
    		String[] available_lang = AppSettings.getAvailableLanguages();
    		for (int i=0; i<available_lang.length; i++) {
    			langChoice.add(available_lang[i]);
    		}
    	}
    	return langChoice;
	}
    

	private void setIfPresent(Choice ch, String str) {
    	
    	int ind = 0;
    	for (int i=0; i<ch.getItemCount(); i++) {
    		if (ch.getItem(i).compareToIgnoreCase(str) == 0) {
    			ind = i;
    			break;
    		}
    	}
    	ch.select(ind);
	}

	private Choice getSrsChoice() {
    	
    	if (srsChoice == null) {
    		
    		srsChoice = new Choice();
    		addKnownCodes(srsChoice);
    		// srsChoice.add("EPSG:23030");
    		// srsChoice.add("EPSG:4326");
    		
    	}
		return srsChoice;
	}

	private Choice getSizeChoice() {
    	
    	if (sizeChoice == null) {
    		
    		sizeChoice = new Choice();
    		addSizes(sizeChoice);
    		
    	}
		return sizeChoice;
	}


	
	private void addKnownCodes(Choice ch) {
		Iterator iter = GeoUtils.CRSIterator();
		while (iter.hasNext()) {
			ch.add((String) iter.next());
		}
	}

	private void addSizes(Choice ch) {
		// So far, only two sizes "a pin"
		ch.add("24x24");
		ch.add("48x48");
		
		/* Let's allow adding an icon set configured in AppSettings (app persistence)
		*  so that if a different icon folder (different from standard icon sets
		*  -img24, img48-) under resources/img is stored, it's displayed in
		*  the choice.
		*/
		// if (mapToChoice(AppSettings.defaultSizeFactor) == AppSettings.defaultSizeFactor) {
		// 	ch.add(AppSettings.defaultSizeFactor);
		// }
		
		//TODO: Manage icon sets or get folders under resources/img
	}
	public void mousePressed(MouseEvent e) {

		Object src = e.getSource();
		
		if (src == pathButton) {
			
			FileDialog dlg = new FileDialog(parent, Messages.getText("AppSettingsDialog.19"), FileDialog.LOAD); //$NON-NLS-1$
			dlg.setModal(true);
			dlg.show();

			String getFile = dlg.getDirectory();
			if (getFile == null) return;
			pathText.setText(getFile);
		}
		
		if (src == defPathButton) {
			FileDialog dlg = new FileDialog(parent, Messages.getText("AppSettingsDialog.20"), FileDialog.LOAD); //$NON-NLS-1$
			dlg.setModal(true);
			dlg.show();

			String getFile = dlg.getFile();
			
			if (getFile == null) return;
			
			String dir = dlg.getDirectory();
			String last_ch = dir.substring(dir.length() - 1);
			if (last_ch.compareTo(File.separator) == 0) {
				dir = dir.substring(0, dir.length() - 1);
			}
			pathText.setText(dir);
			
			defProjectText.setText(getFile);
		}
		
		if (src == thisDefButton) {
			String curr_prj_path = parent.getMapControl().project.getPath();
			String curr_prj_name = parent.getMapControl().project.getName();
			defProjectText.setText(curr_prj_path + File.separator + curr_prj_name + Project.extension);
		}
		
		if (src == okButton) {
			
			AcceptCancelDialog.informUser(
					Messages.getText("AppSettingsDialog.apply"),
					Messages.getText("AppSettingsDialog.restart_to_apply"),
					parent);
			super.mousePressed(e);
			parent.doStoreAppSettings(this);
			return;
		}		

		super.mousePressed(e);		
		
		
	}
	
	// --------------------- getters -------------------
	
	/**
	 * Get chosen language code.
	 * @return chosen language code
	 */
	public String getLanguage() {
		return langChoice.getSelectedItem();
	}
	
	/**
	 * @return chosen SRS (EPSG:XXXXX)
	 */
	public String getSRS() {
		return srsChoice.getSelectedItem();
	}
	
	/**
	 * @return chosen Size Factor (folder path).
	 *         maps from choice texts to folder paths
	 */
	public String getSizeFactor() {
		String choiceItem;
		
		choiceItem = sizeChoice.getSelectedItem();
		
		//Let's map to the value to be serialized 
		//(from displayed text in the choice)
		return mapFromChoice(choiceItem);
	}
	
	/**
	 * Maps from XML serialized sizeFactor to the text to be used for the choice 
	 * @param xmlSize Serialized (AppSettings)value of sizeFactor 
	 * @return text to be displayed in the choice
	 */
	private String mapToChoice(String xmlSize){
		String text = "";
		
		if (xmlSize.equalsIgnoreCase("img24")) {
			text = "24x24";
		} else if (xmlSize.equalsIgnoreCase("img48")) {
			text = "48x48";
		} else {
			text = xmlSize;
		}
		
		return text;
	
	}

	/**
	 * Maps from the sizeFactor in the combo to the text to be serialized 
	 * @param choiceSize value of sizeFactor in the choice
	 * @return text to be serialized (in AppSettings)
	 */
	private String mapFromChoice(String choiceSize){
		if (choiceSize.equals("24x24")) {
			return "img24";
		} else if (choiceSize.equals("48x48")) {
			return "img48";
		} else if (choiceSize.equals("72x72")) {
			return "img72";
		} else return choiceSize;
	
	}

	/**
	 * 
	 * @return chosen default view extent.
	 */
	public Rectangle2D getBBox() {
		
		Rectangle2D resp = new Rectangle2D.Double(-180, -90, 360, 180);
		double x1, y1, x2, y2;
		
		try {
			
			x1 = Double.parseDouble(bboxText_xmin.getText());
			y1 = Double.parseDouble(bboxText_ymin.getText());
			x2 = Double.parseDouble(bboxText_xmax.getText());
			y2 = Double.parseDouble(bboxText_ymax.getText());
			resp = new Rectangle2D.Double(x1, y1, x2 - x1, y2 - y1);
			
		} catch (Exception ex) {
			logger.error("While getting bbox from dialog: " + ex.getMessage()); //$NON-NLS-1$
		}
		
		return resp;
	}
	
	/**
	 * 
	 * @return default view's chosen backgrouond color
	 */
	public Color getBgColor() {
		return bgComponent.getColor();
	}
	
	/**
	 * 
	 * @return chosen startup project full path 
	 */
	public File getPath() {
		File resp = new File(File.separator);
		
		try {
			resp = new File(pathText.getText());
		} catch (Exception ex) {
			logger.error("While getting path from dialog: " + ex.getMessage()); //$NON-NLS-1$
		}
		return resp;
	}

	public void itemStateChanged(ItemEvent e) {
		
		Object src = e.getSource();
		
		if (src == useDefProjectChk) {
			defProjectText.setEnabled(useDefProjectChk.getState());
		}
	}
	
	/**
	 * 
	 * @return chosen startup project file 
	 */
	public File getDefaultProjectFile() {
		File f = null;
		try {
			f = new File(AppSettings.defaultSavePath + File.separator + defProjectText.getText());
		} catch (Exception bad_file_ex) {}
		return f;
	}
	
	/**
	 * 
	 * @return whether the user wants to use a startup project
	 */
	public boolean isDefaultProjectUsed() {
		return useDefProjectChk.getState();
	}
	

}

