/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.FileDialog;
import java.awt.Frame;
import java.util.ArrayList;

import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;

/**
 * A file dialog that uses the CustomizedFilenameFilter. These things dont work properly
 * in IBM J9 but do work in Sun Phoneme.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.dialog.CustomizedFilenameFilter
 * 
 * @author jldominguez
 *
 */
public class FilteredFileDialog extends FileDialog {
	
	private CustomizedFilenameFilter filter = null;

	/**
	 * Constructor
	 * @param parent parent frame
	 * @param _ext allowed extension ("jpg", etc)
	 * @param _dir initial folder
	 */
	public FilteredFileDialog(Frame parent, String _ext, String _dir) {
		super(parent);
		setDirectory(_dir);
		if (_ext==null)
			setFile("*.*");
		else
			setFile("*." + _ext);
		filter = new CustomizedFilenameFilter(_ext);
		setFilenameFilter(filter);
	}
	
	/**
	 * Constructor
	 * @param parent parent frame
	 * @param _ext allowed extension ("jpg", etc)
	 */
	public FilteredFileDialog(Frame parent, String _ext) {
		this(parent, _ext, System.getProperty("user.dir"));
	}
	
	public FilteredFileDialog(Frame parent) {
		this(parent,null,System.getProperty("user.dir"));
	}

	/**
	 * Adds an allowed extension
	 * @param ext allowed extension
	 */
	public void addAllowExtension(String ext){
		filter.addAllowExtension(ext);
	}
}
