/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.Frame;
import java.awt.Label;
import java.awt.TextField;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Point2D;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.geodetic.utils.conversion.ConversionCoords;
import es.prodevelop.gvsig.mobile.fmap.core.acetate.MapFinishTrack;
import es.prodevelop.gvsig.mobile.fmap.core.acetate.MapSymbolIcon;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.canvas.MapFinishTrackSupport;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleWindow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.LayersTable;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;
import es.prodevelop.gvsig.mobile.gui.tool.listener.EditionLineAreaToolListener;
import es.prodevelop.gvsig.mobile.gui.tool.listener.EditionPointToolListener;
import es.prodevelop.gvsig.mobile.icons.IconResourceReader;

/**
 * Visualizes one point and the direction have been taken.
 * This dialog is used to add a new point landmark and if the GPS is active, says the direction line to reach new point.
 * The new point is able to being edited by user with the same attributes as the rest of the layer features.
 * 
 * When the user clicks on "OK", The landmark will be created. 
 *  
 * @author Carlos Snchez Perin
 *
 */
public class FinishPointDialog extends ExpansibleWindow implements MouseListener, ItemListener {
	
	private static Logger logger = Logger.getLogger(FinishPointDialog.class);
	private static final long serialVersionUID = 1L;
	
	// Title
	private Label titleLabel = null;
	// 1st row
    private Label bboxLabel_x;
    private TextField bboxText_x; 
    // 2nd row
    private Label bboxLabel_y;
    private TextField bboxText_y;
    // Buttons
    private TocButton okButton, cancelButton;
	
    private LayersTable toc = null;
	private MapControl map;
	private MapSymbolIcon flagIcon;
	private MapFinishTrackSupport flagTrackSupport;
	private MapSymbolIcon actual_flag = null;
	private MapFinishTrack actual_track = null;
	
	/**
	 * Constructor. Needed to get symbols painted.
	 * @param canvas 		where the icon will be painted
	 * @param p 			parent frame, needed for graphical purposes
	 */
	public FinishPointDialog(MapControl canvas,  Frame p, EditionPointToolListener mainAddPointList, EditionLineAreaToolListener mainAddLineList) {	
		super(p);
		map = canvas;
		toc = canvas.getToc();
		setBounds(p.getX(), p.getY(), p.getWidth(), p.getHeight());
		actual_flag =(MapSymbolIcon) map.getMapContext().getAcetateLayer().getElement("GPS_FINISH_FLAG_ICON");
		actual_track =(MapFinishTrack) map.getMapContext().getAcetateLayer().getElement("GPS_FINISH_FLAG_TRACK");
		init();
		setVisible(true);
	}
	
	private void init() {
		int coord_text_w =(getWidth() - DialogConstants.LABELS_WIDTH - 4 * DialogConstants.XSPACE - 8) / 2;
		
		cancelButton = new TocButton(DialogConstants.DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME, 0, true, false);
		cancelButton.addMouseListener(this);
		okButton = new TocButton(DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, "toc_ok_dis_small.png", 1, true, false); //$NON-NLS-1$
		okButton.addMouseListener(this);
		
		
		titleLabel = new Label(Messages.getText("GPSFinishPointDialog.0")); //$NON-NLS-1$
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		titleLabel.setAlignment(1);

		    
		// 3rd row
        bboxLabel_x = new Label(Messages.getText("GPSFinishPointDialog.1")); //$NON-NLS-1$
      
        bboxLabel_x.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        //bboxLabel_x.setFont(DialogConstants.MENU_FONT);
        bboxLabel_x.setFont(DialogConstants.BIG_FONT);
        bboxText_x = new TextField();
        bboxText_x.setSize(coord_text_w, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        bboxText_x.setFont(DialogConstants.BIG_FONT);
        //bboxText_x.setFont(DialogConstants.DIALOG_SMALL_FONT);
        // 4th row
        bboxLabel_y = new Label(Messages.getText("GPSFinishPointDialog.2")); //$NON-NLS-1$
        bboxLabel_y.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        //bboxLabel_y.setFont(DialogConstants.MENU_FONT);
        bboxLabel_y.setFont(DialogConstants.BIG_FONT);
        bboxText_y = new TextField();
        bboxText_y.setSize(coord_text_w, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
        //bboxText_y.setFont(DialogConstants.DIALOG_SMALL_FONT);
        bboxText_y.setFont(DialogConstants.BIG_FONT);		 
   		titleLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
   		bboxLabel_x.setSize(DialogConstants.LABELS_WIDTH*2, DialogConstants.TABLE_LABEL_HEIGHT*2);
   		bboxLabel_x.setAlignment(Label.CENTER);
   		bboxText_x.setSize(DialogConstants.LABELS_WIDTH*2, DialogConstants.TABLE_LABEL_HEIGHT*2);
   		bboxLabel_y.setSize(DialogConstants.LABELS_WIDTH*2, DialogConstants.TABLE_LABEL_HEIGHT*2);
		bboxLabel_y.setAlignment(Label.CENTER);
		bboxText_y.setSize(DialogConstants.LABELS_WIDTH*2, DialogConstants.TABLE_LABEL_HEIGHT*2);
		if (actual_flag!=null){
   			bboxText_x.setText(new Double(actual_track.getGeoPoint().getX()).toString());
   			bboxText_y.setText(new Double(actual_track.getGeoPoint().getY()).toString());
   		}
		else{
			bboxText_x.setText("0.0");
   			bboxText_y.setText("0.0");
		}
		
		// -------------------
		
		// --------------
		ExpansibleRow row00 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row00.addExtensible(titleLabel);
		add(row00);
		// --------------------
		ExpansibleRow row0; row0 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT+20);
	    row0.add(bboxLabel_x);
	    add(row0);
	    // --------------------
		ExpansibleRow row1; row1 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT+20);
	    row1.add(bboxText_x);
	    add(row1);
	    // ---------------------
	    ExpansibleRow row2; row2 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT+20);
	    row2.add(bboxLabel_y); row2.add(bboxText_y);
	    add(row2);
	    // ---------------------
	    ExpansibleRow row3; row3 = new ExpansibleRow(DialogConstants.CHOICE_FRIENDLY_HEIGHT+20);
	    row3.add(bboxText_y);
	    add(row3);
		// --------------
		ExpansibleRow row5 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		addExpansible(row5);
		// --------------
		ExpansibleRow row6 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row6.add(okButton); row6.add(cancelButton);
		add(row6);
		// --------------
		addEmptyRowIfPhoneME(this);
	}

	public void mousePressed(MouseEvent e) {
		Object src = e.getSource();
		
		if (src == okButton) {
			hide();
			toc.getMapControl().repaint();
			drawFinishFlag();
		}

		if (src == cancelButton) {
			hide();
		}
	}

	private void drawFinishFlag() {
		double lon = 0;
		double lat = 0;
		
		//Get the fields X and Y
		String longitude = bboxText_x.getText();
		String latitude = bboxText_y.getText();
		try{
			lon = Double.parseDouble(longitude);
			lat = Double.parseDouble(latitude);
		}catch (NumberFormatException e1){
			logger.error("Error parsing the double coordinate values from X and/or Y text fields in CoordinatesPointDialog.");
		}
		double[] coords = ConversionCoords.wgs842projection(lon,lat,map.getProjection());
		
		if (actual_flag!=null){
			actual_flag.setXY(coords[0],coords[1]);
			Point2D geoPoint = new Point2D.Double(coords[0],coords[1]);
			actual_track.setGeoPoint(geoPoint);
			actual_track.setGpsPoint(geoPoint);
		}
		else{
			//Generate the new symbol icon and its position
			flagIcon = new MapSymbolIcon(IconResourceReader.GPS_FINISH_ICON);
				
			//Geographic position
			flagIcon.setXY(coords[0], coords[1]);
		
			//Inserting the symbol into the drawn list
			map.getMapContext().getAcetateLayer().addElement(flagIcon, null, "GPS_FINISH_FLAG_ICON");
			logger.info("New symbol icon inserted on acetate layer");
			
			//Generate the new symbol icon and its position
			flagTrackSupport = new MapFinishTrackSupport(coords[0],coords[1],map);
				
			//Inserting the symbol track into the drawn list
			map.getMapContext().getAcetateLayer().addElement(flagTrackSupport.getSymbol(), null, "GPS_FINISH_FLAG_TRACK");
			logger.info("New symbol track inserted on acetate layer");
		}
	}

	public void mouseEntered(MouseEvent e) { }
	public void mouseExited(MouseEvent e) { }
	public void mouseClicked(MouseEvent e) { }
	public void mouseReleased(MouseEvent e) { }
	public void itemStateChanged(ItemEvent e) { }

}
