/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Scrollbar;
import java.awt.TextField;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.ItemEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.sql.Types;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;

import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.fmap.legend.IntervalLegend;
import es.prodevelop.gvsig.mobile.fmap.legend.SingleSymbolLegend;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ColorChooserPanel;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleWindow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;

/**
 * This dialog manages the layer's symbology (for vector layers only). Depending
 * on the type of the geometries, it shows, fill color, border color, stroke
 * width and a checkbox for filled/not filled.
 * 
 * Color are chosen with the <code>ColorChooserPanel</code> and
 * <code>ColorChooser</code>
 * 
 * @see es.prodevelop.gvsig.mobile.gui.dialog.control.ColorChooserPanel
 * @see es.prodevelop.gvsig.mobile.gui.dialog.control.ColorChooser
 * 
 * @author jldominguez
 * 
 */
public class LayerSymbologyDialog extends ExpansibleWindow implements
		MouseListener, AdjustmentListener {

	private static Logger logger = Logger.getLogger(LayerSymbologyDialog.class);
	private FLyrVect layer;
	private LayersManagerDialog parent;
	private Frame mFrame;
	private TocButton okButton, cancelButton;
	private int shapeType = -1;

	private Label titleLabel, pointColorLabel, pointSizeLabel, lineColorLabel,
			lineWidthLabel, polygonFilledLabel, polygonLineColorLabel,
			polygonLineWidthLabel, polygonFillColorLabel, initColorLabel,
			endColorLabel, fieldsLabel, intervalsLabel, minValueLabel,
			maxValueLabel;

	private ColorChooserPanel pointColorChooser, lineColorChooser,
			polygonLineColorChooser, polygonFillColorChooser, initColorChooser,
			endColorChooser;

	private Checkbox fillCB;

	private Choice fieldsChoice;

	private TextField intervalsTB, minValueTB, maxValueTB;

	private Label sizeLabel = null;
	private Scrollbar sizeScroll = null;

	private FSymbol symbol = null;

	private Vector fields = null;

	private CheckboxGroup symbolsCBG;
	private Checkbox singleSymbol;
	private Checkbox rangeSymbol;

	private IntervalLegend intervalLegend;
	private double minValue = 0, maxValue = 0;
	private String[] columns;
	private int w = 1;
	private Color c;

	/**
	 * Constructor.
	 * 
	 * @param lyr
	 *            The layer whose symbology has to viewed/modified
	 * @param p
	 *            parent dialog
	 * @param mf
	 *            main frame
	 */
	public LayerSymbologyDialog(FLyrVect lyr, LayersManagerDialog p, Frame mf) {

		super(mf);
		try {
			if (lyr != null)
				fields = getNumericFields(lyr.getSource());
		} catch (DriverException e) {
			logger.debug("DriverException: " + e.getStackTrace());
		} catch (FMapDriverException e) {
			logger.debug("FMapDriverException: " + e.getStackTrace());
		}

		mFrame = mf;
		parent = p;
		layer = lyr;
		try {
			shapeType = layer.getShapeType();
		} catch (FMapDriverException e) {
			shapeType = FShape.POLYGON;
		}

		symbol = ((SingleSymbolLegend) layer.getLegend()).getDefaultSymbol();
		intervalLegend = layer.getIntervalLegend();

		int w = parent.getWidth();
		setBounds(parent.getX(), parent.getY(), w, parent.getHeight());
		// setModal(true);
		init();
	}

	private Vector getNumericFields(ReadableVectorial rv)
			throws DriverException, FMapDriverException {

		SelectableDataSource ds = rv.getRecordset();
		final int size = ds.getFieldCount();

		Vector fields = new Vector();
		this.columns = new String[size];
		int j = 0;

		for (int i = 0; i < size; i++) {
			try {
				int type = ds.getFieldType(i);

				if ((type == Types.BIGINT) || (type == Types.DECIMAL)
						|| (type == Types.DOUBLE) || (type == Types.FLOAT)
						|| (type == Types.INTEGER) || (type == Types.NUMERIC)
						|| (type == Types.REAL) || (type == Types.TINYINT)) {
					String fieldName = ds.getFieldName(i);
					this.columns[j] = fieldName;
					fieldName = fieldName.substring(
							fieldName.lastIndexOf(':') + 1, fieldName.length());
					fields.add(fieldName);
					j++;
				} else if (type == Types.VARCHAR) {
					String fieldName = ds.getFieldName(i);					
					 Value v = ds.getFieldValue(
					 i,
					 ds.getFieldIndexByName(
					 ds.getFieldName(i)));
					 try {
						 double follow = new Double(v.toString()).doubleValue();
						 this.columns[j] = fieldName;
							fieldName = fieldName.substring(
									fieldName.lastIndexOf(':') + 1, fieldName.length());
							fields.add(fieldName);
							j++;
					 } catch (Exception e){
						 
					 }					 
				}
			} catch (Exception e) {
				logger.debug(e.getStackTrace());
			}
		}

		return fields;
	}

	private void setEnabledComponents(boolean enable) {
		// sizeScroll.setEnabled(enable);

		// if (pointSizeLabel != null)
		// pointSizeLabel.setEnabled(enable);

		if (polygonFilledLabel != null)
			polygonFilledLabel.setEnabled(enable);

		if (polygonLineWidthLabel != null)
			polygonLineWidthLabel.setEnabled(enable);

		if (lineWidthLabel != null)
			lineWidthLabel.setEnabled(enable);

		// if (sizeLabel != null)
		// sizeLabel.setEnabled(enable);

		if (pointColorChooser != null)
			pointColorChooser.setEnabled(enable);

		if (pointColorLabel != null)
			pointColorLabel.setEnabled(enable);

		if (lineColorChooser != null)
			lineColorChooser.setEnabled(enable);

		if (lineColorLabel != null)
			lineColorLabel.setEnabled(enable);

		if (polygonLineColorChooser != null)
			polygonLineColorChooser.setEnabled(enable);

		if (polygonLineColorLabel != null)
			polygonLineColorLabel.setEnabled(enable);

		if (polygonFillColorChooser != null)
			polygonFillColorChooser.setEnabled(enable);

		if (polygonFillColorLabel != null)
			polygonFillColorLabel.setEnabled(enable);

		if (fillCB != null)
			fillCB.setEnabled(enable);

		if (initColorChooser != null)
			initColorChooser.setEnabled(!enable);

		if (initColorLabel != null)
			initColorLabel.setEnabled(!enable);

		if (endColorChooser != null)
			endColorChooser.setEnabled(!enable);

		if (endColorLabel != null)
			endColorLabel.setEnabled(!enable);

		if (this.intervalsTB != null)
			intervalsTB.setEnabled(!enable);

		if (this.fieldsChoice != null) {
			fieldsChoice.setEnabled(!enable);
		}

		if (fieldsLabel != null)
			fieldsLabel.setEnabled(!enable);

		if (intervalsLabel != null)
			intervalsLabel.setEnabled(!enable);

		if (minValueLabel != null)
			minValueLabel.setEnabled(!enable);

		if (maxValueLabel != null)
			maxValueLabel.setEnabled(!enable);

		if (minValueTB != null)
			minValueTB.setEnabled(!enable);

		if (maxValueTB != null)
			maxValueTB.setEnabled(!enable);
	}

	private void init() {

		cancelButton = new TocButton(
				DialogConstants.DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME, 0,
				true, false);
		cancelButton.addMouseListener(this);
		okButton = new TocButton(
				DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, 1,
				true, false);
		okButton.addMouseListener(this);

		// --------- get ------------

		// -------- set -----------
		titleLabel = new Label(Messages.getText("LayerSymbologyDialog.0")); //$NON-NLS-1$
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		titleLabel.setAlignment(1);
		titleLabel.setSize(DialogConstants.LABELS_WIDTH,
				DialogConstants.CONTROL_HEIGHT);

		// --------------
		ExpansibleRow row0 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row0.addExtensible(titleLabel);
		add(row0);
		// --------------------

		symbolsCBG = new CheckboxGroup();
		singleSymbol = new Checkbox(
				Messages.getText("Smbolo simple: "), symbolsCBG, true); //$NON-NLS-1$
		singleSymbol.setSize(DialogConstants.LABELS_WIDTH,
				DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		singleSymbol.setFont(DialogConstants.DIALOG_SMALL_FONT);

		singleSymbol.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					setEnabledComponents(true);
				} else {
					setEnabledComponents(false);
				}
			}
		});

		rangeSymbol = new Checkbox(
				Messages.getText("Rango de valores: "), symbolsCBG, false); //$NON-NLS-1$
		rangeSymbol.setSize(2 * DialogConstants.LABELS_WIDTH,
				DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		rangeSymbol.setFont(DialogConstants.DIALOG_SMALL_FONT);

		rangeSymbol.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					setEnabledComponents(false);
				} else {
					setEnabledComponents(true);
				}
			}
		});

		if (shapeType == FShape.POINT) {

			pointSizeLabel = new Label(Messages
					.getText("LayerSymbologyDialog.1")); //$NON-NLS-1$
			pointSizeLabel.setSize(DialogConstants.LABELS_WIDTH,
					DialogConstants.TABLE_LABEL_HEIGHT);
			pointSizeLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);

			int s = symbol.getStrokeWidth();
			if (layer.IsUsedIntervalLegend()) {
				s = layer.getIntervalLegend().getDefaultSymbol()
						.getStrokeWidth();
			}

			sizeLabel = new Label();
			sizeLabel.setSize(DialogConstants.LABELS_WIDTH / 2,
					DialogConstants.TABLE_LABEL_HEIGHT);
			sizeLabel.setText("" + s); //$NON-NLS-1$
			sizeLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);

			sizeScroll = new Scrollbar(Scrollbar.HORIZONTAL, s, 1, 1, 10);
			sizeScroll.addAdjustmentListener(this);
			sizeScroll.setSize(DialogConstants.LABELS_WIDTH,
					DialogConstants.TABLE_LABEL_HEIGHT);

			pointColorLabel = new Label(Messages
					.getText("LayerSymbologyDialog.3")); //$NON-NLS-1$
			pointColorLabel.setSize(DialogConstants.LABELS_WIDTH,
					DialogConstants.TABLE_LABEL_HEIGHT);
			pointColorLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
			pointColorChooser = new ColorChooserPanel(symbol.getBorderColor(),
					mFrame);

			// --------------
			ExpansibleRow row1 = new ExpansibleRow(
					DialogConstants.TABLE_LABEL_HEIGHT);
			row1.add(pointSizeLabel);
			row1.add(sizeLabel);
			row1.addExtensible(sizeScroll);
			add(row1);

			ExpansibleRow row20 = new ExpansibleRow(
					DialogConstants.CONTROL_HEIGHT);
			row20.addExtensible(singleSymbol);
			add(row20);
			// --------------
			ExpansibleRow row2 = new ExpansibleRow(
					DialogConstants.TABLE_LABEL_HEIGHT);
			row2.add(pointColorLabel);
			row2.add(pointColorChooser);
			add(row2);

		} else {
			if (shapeType == FShape.LINE) {

				lineWidthLabel = new Label(Messages
						.getText("LayerSymbologyDialog.4")); //$NON-NLS-1$
				lineWidthLabel.setSize(DialogConstants.LABELS_WIDTH,
						DialogConstants.TABLE_LABEL_HEIGHT);
				lineWidthLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);

				sizeLabel = new Label();
				sizeLabel.setSize(DialogConstants.LABELS_WIDTH / 2,
						DialogConstants.TABLE_LABEL_HEIGHT);
				sizeLabel.setText("" + symbol.getStrokeWidth()); //$NON-NLS-1$
				sizeLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);

				sizeScroll = new Scrollbar(Scrollbar.HORIZONTAL, symbol
						.getStrokeWidth(), 1, 1, 10);
				sizeScroll.addAdjustmentListener(this);
				sizeScroll.setSize(DialogConstants.LABELS_WIDTH,
						DialogConstants.TABLE_LABEL_HEIGHT);

				lineColorLabel = new Label(Messages
						.getText("LayerSymbologyDialog.6")); //$NON-NLS-1$
				lineColorLabel.setSize(DialogConstants.LABELS_WIDTH,
						DialogConstants.TABLE_LABEL_HEIGHT);
				lineColorLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
				lineColorChooser = new ColorChooserPanel(symbol
						.getBorderColor(), mFrame);

				ExpansibleRow row20 = new ExpansibleRow(
						DialogConstants.CONTROL_HEIGHT);
				row20.addExtensible(singleSymbol);
				add(row20);

				// --------------
				ExpansibleRow row3 = new ExpansibleRow(
						DialogConstants.TABLE_LABEL_HEIGHT);
				row3.add(lineWidthLabel);
				row3.add(sizeLabel);
				row3.addExtensible(sizeScroll);
				add(row3);
				// --------------
				ExpansibleRow row4 = new ExpansibleRow(
						DialogConstants.TABLE_LABEL_HEIGHT);
				row4.add(lineColorLabel);
				row4.add(lineColorChooser);
				add(row4);

			} else {

				polygonLineWidthLabel = new Label(Messages
						.getText("LayerSymbologyDialog.7")); //$NON-NLS-1$
				polygonLineWidthLabel.setSize(DialogConstants.LABELS_WIDTH,
						DialogConstants.TABLE_LABEL_HEIGHT);
				polygonLineWidthLabel
						.setFont(DialogConstants.DIALOG_SMALL_FONT);

				sizeLabel = new Label();
				sizeLabel.setSize(DialogConstants.LABELS_WIDTH / 3,
						DialogConstants.TABLE_LABEL_HEIGHT);
				sizeLabel.setText("" + symbol.getStrokeWidth()); //$NON-NLS-1$
				sizeLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);

				sizeScroll = new Scrollbar(Scrollbar.HORIZONTAL, symbol
						.getStrokeWidth(), 1, 1, 10);
				sizeScroll.addAdjustmentListener(this);
				sizeScroll.setSize(DialogConstants.LABELS_WIDTH,
						DialogConstants.TABLE_LABEL_HEIGHT);

				polygonLineColorLabel = new Label(Messages
						.getText("LayerSymbologyDialog.9")); //$NON-NLS-1$
				polygonLineColorLabel.setSize(
						DialogConstants.LABELS_WIDTH * 6 / 3,
						DialogConstants.TABLE_LABEL_HEIGHT);
				polygonLineColorLabel
						.setFont(DialogConstants.DIALOG_SMALL_FONT);
				polygonLineColorChooser = new ColorChooserPanel(symbol
						.getBorderColor(), mFrame);

				polygonFilledLabel = new Label(Messages
						.getText("LayerSymbologyDialog.10")); //$NON-NLS-1$
				polygonFilledLabel.setSize(
						DialogConstants.LABELS_WIDTH * 6 / 3,
						DialogConstants.TABLE_LABEL_HEIGHT);
				polygonFilledLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
				fillCB = new Checkbox();
				fillCB.setState(symbol.isFilled());
				fillCB.setSize(DialogConstants.SMALL_CONTROL_HEIGHT,
						DialogConstants.TABLE_LABEL_HEIGHT);

				polygonFillColorLabel = new Label(Messages
						.getText("LayerSymbologyDialog.11")); //$NON-NLS-1$
				polygonFillColorLabel.setSize(
						DialogConstants.LABELS_WIDTH * 6 / 3,
						DialogConstants.TABLE_LABEL_HEIGHT);
				polygonFillColorLabel
						.setFont(DialogConstants.DIALOG_SMALL_FONT);
				polygonFillColorChooser = new ColorChooserPanel(symbol
						.getFillColor(), mFrame);

				ExpansibleRow row20 = new ExpansibleRow(
						DialogConstants.CONTROL_HEIGHT);
				row20.addExtensible(singleSymbol);
				add(row20);

				// --------------
				ExpansibleRow row5 = new ExpansibleRow(
						DialogConstants.TABLE_LABEL_HEIGHT);
				row5.add(polygonLineWidthLabel);
				row5.add(sizeLabel);
				row5.addExtensible(sizeScroll);
				add(row5);
				// --------------
				ExpansibleRow row6 = new ExpansibleRow(
						DialogConstants.TABLE_LABEL_HEIGHT);
				row6.add(polygonLineColorLabel);
				row6.add(polygonLineColorChooser);
				add(row6);
				// --------------
				ExpansibleRow row7 = new ExpansibleRow(
						DialogConstants.TABLE_LABEL_HEIGHT);
				row7.add(polygonFilledLabel);
				row7.add(fillCB);
				add(row7);
				// --------------
				ExpansibleRow row8 = new ExpansibleRow(
						DialogConstants.TABLE_LABEL_HEIGHT);
				row8.add(polygonFillColorLabel);
				row8.add(polygonFillColorChooser);
				add(row8);
			}
		}

		ExpansibleRow row30 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row30.addExtensible(rangeSymbol);
		add(row30);

		fieldsChoice = new Choice();
		fieldsChoice.setSize(DialogConstants.LABELS_WIDTH,
				DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		fieldsChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);
		if (fields != null) {
			final int size = fields.size();

			for (int i = 0; i < size; i++) {
				fieldsChoice.add((String) fields.elementAt(i));
			}
		}
		fieldsChoice.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				try {
					getMinMaxValue(columns[fieldsChoice.getSelectedIndex()]);
				} catch (FMapDriverException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (DriverException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});

		if (intervalLegend != null) {
			String fieldName = intervalLegend.getFieldName();
			if (fieldName != null) {
				fieldName = fieldName.substring(fieldName.lastIndexOf(':') + 1,
						fieldName.length());
				fieldsChoice.select(fieldName);
			}
		}

		ExpansibleRow row11 = new ExpansibleRow(
				DialogConstants.TABLE_LABEL_HEIGHT);
		fieldsLabel = new Label(Messages.getText("Campo nmerico: ")); //$NON-NLS-1$
		fieldsLabel.setSize(DialogConstants.LABELS_WIDTH * 6 / 3,
				DialogConstants.TABLE_LABEL_HEIGHT);
		fieldsLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		row11.add(fieldsLabel);
		row11.add(fieldsChoice);
		add(row11);

		ExpansibleRow row21 = new ExpansibleRow(
				DialogConstants.TABLE_LABEL_HEIGHT);
		minValueLabel = new Label(Messages.getText("Valor mnimo: ")); //$NON-NLS-1$
		minValueLabel.setSize(DialogConstants.LABELS_WIDTH * 6 / 3,
				DialogConstants.TABLE_LABEL_HEIGHT);
		minValueLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		minValueTB = new TextField("0.0");
		minValueTB.setSize(DialogConstants.LABELS_WIDTH,
				DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		minValueTB.setFont(DialogConstants.DIALOG_SMALL_FONT);
		maxValueLabel = new Label(Messages.getText("Valor mximo: ")); //$NON-NLS-1$
		maxValueLabel.setSize(DialogConstants.LABELS_WIDTH * 6 / 3,
				DialogConstants.TABLE_LABEL_HEIGHT);
		maxValueLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		maxValueTB = new TextField("0.0");
		maxValueTB.setSize(DialogConstants.LABELS_WIDTH,
				DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		maxValueTB.setFont(DialogConstants.DIALOG_SMALL_FONT);

		row21.add(minValueLabel);
		row21.add(minValueTB);

		add(row21);

		// Light version
		// row21.setVisible(false);

		ExpansibleRow row22 = new ExpansibleRow(
				DialogConstants.TABLE_LABEL_HEIGHT);
		row22.add(maxValueLabel);
		row22.add(maxValueTB);
		add(row22);

		// Light version
		// row22.setVisible(false);

		ExpansibleRow row14 = new ExpansibleRow(
				DialogConstants.TABLE_LABEL_HEIGHT);
		intervalsLabel = new Label(Messages.getText("Nmero de intervalos: ")); //$NON-NLS-1$
		intervalsLabel.setSize(DialogConstants.LABELS_WIDTH * 6 / 3,
				DialogConstants.TABLE_LABEL_HEIGHT);
		intervalsLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		row14.add(intervalsLabel);
		if (intervalLegend == null)
			intervalsTB = new TextField("5");
		else
			intervalsTB = new TextField(String.valueOf(intervalLegend
					.getIntervals()));
		intervalsTB.setSize(DialogConstants.LABELS_WIDTH,
				DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		intervalsTB.setFont(DialogConstants.DIALOG_SMALL_FONT);
		row14.add(intervalsTB);
		add(row14);

		// Light version
		// row14.setVisible(false);

		initColorLabel = new Label(Messages.getText("Color inicial")); //$NON-NLS-1$
		initColorLabel.setSize(DialogConstants.LABELS_WIDTH * 6 / 3,
				DialogConstants.TABLE_LABEL_HEIGHT);
		initColorLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		if (intervalLegend == null)
			initColorChooser = new ColorChooserPanel(Color.blue, mFrame);
		else
			initColorChooser = new ColorChooserPanel(intervalLegend
					.getMinColor(), mFrame);

		ExpansibleRow row12 = new ExpansibleRow(
				DialogConstants.TABLE_LABEL_HEIGHT);
		row12.add(initColorLabel);
		row12.add(initColorChooser);
		add(row12);

		// Light version
		// row12.setVisible(false);

		endColorLabel = new Label(Messages.getText("Color final")); //$NON-NLS-1$
		endColorLabel.setSize(DialogConstants.LABELS_WIDTH * 6 / 3,
				DialogConstants.TABLE_LABEL_HEIGHT);
		endColorLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		if (intervalLegend == null)
			endColorChooser = new ColorChooserPanel(Color.red, mFrame);
		else
			endColorChooser = new ColorChooserPanel(intervalLegend
					.getMaxColor(), mFrame);

		ExpansibleRow row13 = new ExpansibleRow(
				DialogConstants.TABLE_LABEL_HEIGHT);
		row13.add(endColorLabel);
		row13.add(endColorChooser);
		add(row13);

		// Light version
		// row13.setVisible(false);

		ExpansibleRow row9 = new ExpansibleRow(
				DialogConstants.TABLE_LABEL_HEIGHT);
		row9.add(new Label("")); //$NON-NLS-1$
		addExpansible(row9);

		// ----------------------------------
		ExpansibleRow row10 = new ExpansibleRow(
				DialogConstants.TABLE_LABEL_HEIGHT);
		row10.add(okButton);
		row10.add(cancelButton);
		add(row10);

		// --------------
		addEmptyRowIfPhoneME(this);

		this.singleSymbol.setState(!layer.IsUsedIntervalLegend());
		this.rangeSymbol.setState(layer.IsUsedIntervalLegend());
		this.setEnabledComponents(!layer.IsUsedIntervalLegend());

		try {
			// if(fieldsChoice.getItemCount() > 0)
			// fieldsChoice.select(0);
			if (intervalLegend != null) {
				minValue = intervalLegend.getMinValue();
				this.minValueTB.setText(String.valueOf(minValue));

				maxValue = intervalLegend.getMaxValue();
				this.maxValueTB.setText(String.valueOf(maxValue));
			} else if (fieldsChoice.getSelectedIndex() != -1)
				this.getMinMaxValue(columns[fieldsChoice.getSelectedIndex()]);
		} catch (FMapDriverException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (DriverException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		if (fields.size() <= 0) {
			this.rangeSymbol.setEnabled(false);
		}

	}

	private void getMinMaxValue(String column) throws FMapDriverException,
			DriverException {
		final int size = layer.getSource().getShapeCount();
		double min = 0, max = 0;

		if (size < 5) {
			this.intervalsTB.setText(String.valueOf(size));
		}

		for (int i = 0; i < size; i++) {
			Value v = layer.getSource().getRecordset().getFieldValue(
					i,
					layer.getSource().getRecordset()
							.getFieldIndexByName(column));
			if (v != null) {
				String t = v.toString();
				double val = 0;
				if (t.compareTo("") != 0) {
					try {
						val = new Double(t).doubleValue();

						// System.out.println(val);
						if (i == 0) {
							min = val;
							max = val;
						} else {
							if (val < min)
								min = val;
							if (val > max)
								max = val;
						}
					} catch (Exception e) {

					}
				}
			}
		}
		this.minValue = min;
		this.maxValue = max;

		this.minValueTB.setText(String.valueOf(minValue));
		this.maxValueTB.setText(String.valueOf(maxValue));
	}

	private void setUserDataToLayer() {
		if (singleSymbol.getState()) {
			layer.setUseInterval(false);
			if (shapeType == FShape.POINT) {

				c = pointColorChooser.getColor();
				w = sizeScroll.getValue();
				symbol.setBorderColor(pointColorChooser.getColor());
				symbol.setFillColor(pointColorChooser.getColor());
				symbol.setStrokeWidth(sizeScroll.getValue());

			} else {
				if (shapeType == FShape.LINE) {

					c = lineColorChooser.getColor();
					w = sizeScroll.getValue();
					symbol.setBorderColor(lineColorChooser.getColor());
					symbol.setStrokeWidth(sizeScroll.getValue());

				} else {
					c = polygonLineColorChooser.getColor();
					w = sizeScroll.getValue();

					symbol.setBorderColor(polygonLineColorChooser.getColor());
					symbol.setFillColor(polygonFillColorChooser.getColor());
					symbol.setFilled(fillCB.getState());
					symbol.setStrokeWidth(sizeScroll.getValue());

				}
			}
		} else {
			if (shapeType == FShape.POINT) {

				c = pointColorChooser.getColor();
				w = sizeScroll.getValue();	
				symbol.setBorderColor(c);				
				symbol.setStrokeWidth(w);

			} else {
				if (shapeType == FShape.LINE) {

					c = lineColorChooser.getColor();
					w = sizeScroll.getValue();	
					symbol.setBorderColor(c);
					symbol.setStrokeWidth(w);

				} else {
					c = polygonLineColorChooser.getColor();
					w = sizeScroll.getValue();
					symbol.setBorderColor(c);					
					symbol.setStrokeWidth(w);
				}
			}
			fillIntervalColors();
			layer.setUseInterval(true);

		}

		((SingleSymbolLegend) layer.getLegend()).setDefaultSymbol(symbol);
	}

	public void mousePressed(MouseEvent arg0) {
		Object src = arg0.getSource();

		if (src == okButton) {
			setUserDataToLayer();
			parent.updateLayer(layer);
			try {
				hide();
			} catch (NullPointerException npe) {
				logger
						.warn("Dialog hidden: A mysterious NP exception was caught: "
								+ npe.getMessage());
			}
		}

		if (src == cancelButton) {
			try {
				hide();
			} catch (NullPointerException npe) {
				logger
						.warn("Dialog hidden: A mysterious NP exception was caught: "
								+ npe.getMessage());
			}
		}
	}

	public void mouseReleased(MouseEvent arg0) {
	}

	public void mouseEntered(MouseEvent arg0) {
	}

	public void mouseExited(MouseEvent arg0) {
	}

	public void mouseClicked(MouseEvent arg0) {
	}

	public void adjustmentValueChanged(AdjustmentEvent e) {

		Object src = e.getSource();

		if (src == sizeScroll) {
			sizeLabel.setText("" + sizeScroll.getValue()); //$NON-NLS-1$
		}

	}

	private void fillIntervalColors() {
		int intervals = -1;

		try {
			intervals = Integer.valueOf(intervalsTB.getText().trim())
					.intValue();
		} catch (Exception e) {
			logger.debug(e.getStackTrace());
		}

		if (intervals < 0) {
			// Mostrar dilogo con error
		}

		FSymbol s = symbol.createRandomSymbol(true);
		s.setStrokeWidth(sizeScroll.getValue());
		if (intervalLegend == null) {
			intervalLegend = new IntervalLegend(s, initColorChooser.getColor(),
					endColorChooser.getColor(), intervals);			
		} else {
			intervalLegend.setDefaultSymbol(s);
			intervalLegend.setMinColor(initColorChooser.getColor());
			intervalLegend.setMaxColor(endColorChooser.getColor());
			intervalLegend.setIntervals(intervals);
			intervalLegend.calculateIntervalColors();
			intervalLegend.getDefaultSymbol().setStrokeWidth(
					this.sizeScroll.getValue());			
		}
		intervalLegend.setStrokeColor(c);
		intervalLegend.setStrokeWidth(w);

		double min = minValue;
		double max = maxValue;
		try {
			min = new Double(this.minValueTB.getText()).doubleValue();
			max = new Double(this.maxValueTB.getText()).doubleValue();
			minValue = min;
			maxValue = max;
		} catch (NumberFormatException ex) {
			ex.printStackTrace();
		}

		intervalLegend.setMinValue(min);
		intervalLegend.setMaxValue(max);
		intervalLegend.setFieldName(columns[this.fieldsChoice
				.getSelectedIndex()]);
		layer.setIntervalLegend(intervalLegend);

		// int initBlue, initGreen, initRed,
		// endBlue, endGreen, endRed,
		// intervalBlue, intervalGreen, intervalRed,
		// stepBlue, stepGreen, stepRed;
		//
		// initBlue = initColorChooser.getColor().getBlue();
		// initGreen = initColorChooser.getColor().getGreen();
		// initRed = initColorChooser.getColor().getRed();
		//
		// endBlue = endColorChooser.getColor().getBlue();
		// endGreen = endColorChooser.getColor().getGreen();
		// endRed = endColorChooser.getColor().getRed();
		//
		// intervalBlue = endBlue - initBlue;
		// intervalGreen = endGreen - initGreen;
		// intervalRed = endRed - initRed;
		//		
		// stepBlue = intervalBlue/intervals;
		// stepGreen = intervalGreen/intervals;
		// stepRed = intervalRed/intervals;
		//		
		// intervalColors = new HashMap(intervals+1);
		// for( int i = 0; i<=intervals; i++) {
		// Color c = new Color(initBlue, initGreen, initRed);
		// intervalColors.put(String.valueOf(i), c);
		// initBlue += stepBlue;
		// initGreen += stepGreen;
		// initRed += stepRed;
		// }
	}
}