/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.Component;
import java.awt.Frame;
import java.awt.Label;
import java.awt.TextArea;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.tool.DialogButton;

/**
 * This class is used to show short dialog messages. It is used with static methods, so
 * the constructor is private.
 * 
 * 
 */
public class MessageDialog extends ExpansibleDialog implements MouseListener {
	
	private static Logger logger = Logger.getLogger(MessageDialog.class);
	
	private int w = 0;
	private int h = 0;
	private int l = 0;
	

	private Label leftLabel;
	private TextArea msgTextArea;
	private DialogButton icon;
	private String msg = Messages.getText("MessageDialog.0"); //$NON-NLS-1$

	/**
	 * Private constructor, only used by static methods.
	 * 
	 * @param mf parent frame
	 * @param _msg Short text to be shown
	 * @param tit Dialog title
	 */
	private MessageDialog(Frame mf, String _msg, String tit) {
		super(mf);
		
		setUndecorated(false);
		msg = _msg;
		setTitle(tit);

		init();
		setCenteredBounds(mf);
	}
	
	/**
	 * Private constructor, only used by static methods. Includes width, height and
	 * width of the left margin.
	 * 
	 * @param mf parent frame
	 * @param _msg Short text to be shown
	 * @param tit Dialog title
	 * @param _w width of the dialog in pixels
	 * @param _h height of the dialog in pixels
	 * @param _l left margin (distance between text and left border)
	 */
	private MessageDialog(Frame mf, String _msg, String tit, int _w, int _h, int _l) {
		super(mf);
		
		w = _w;
		h = _h;
		l = _l;
		
		setUndecorated(false);
		setModal(true);
		msg = _msg;
		setTitle(tit);

		init();
		setCenteredBounds(mf);
	}
	
	
	private void setCenteredBounds(Component mf) {
		
		int x = mf.getX() + ((mf.getWidth() - w) / 2);
		int y = mf.getY() + ((mf.getHeight() - h) / 2);
		setBounds(x, y, w, h);
	}

	private void init() {
		
		icon = new DialogButton("error_icon.png", "error_icon.png", this); //$NON-NLS-1$ //$NON-NLS-2$
		icon.setSize(GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH, GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH);
		msgTextArea = new TextArea(msg);
		msgTextArea.setBackground(getBackground());
		msgTextArea.setFont(DialogConstants.DIALOG_SMALL_FONT);
		msgTextArea.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		msgTextArea.setEditable(false);
		
		leftLabel = new Label("");  //$NON-NLS-1$
		leftLabel.setSize(l, DialogConstants.CONTROL_HEIGHT);

		// ------------------------
		add(new ExpansibleRow(DialogConstants.CONTROL_HEIGHT));
		// --------------
		ExpansibleRow row00 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row00.add(leftLabel); row00.add(icon); row00.addExtensible(msgTextArea);
		addExpansible(row00);
		// --------------
		add(new ExpansibleRow(DialogConstants.CONTROL_HEIGHT));
	}


	public void mousePressed(MouseEvent arg0) {
		Object src = arg0.getSource();
		
		if (src == icon) {
			try {
				hide();
			} catch (NullPointerException npe) {
				logger.warn("MessageDlg. A mysterious NP exception was caught: " + npe.getMessage()); //$NON-NLS-1$
			}
		}
	}

	public void mouseReleased(MouseEvent arg0) { }
	public void mouseEntered(MouseEvent arg0) { }
	public void mouseExited(MouseEvent arg0) { }
	public void mouseClicked(MouseEvent arg0) { }

	/**
	 * This is the method truly used by other classes. I t simply instantiates 
	 * a <code>MessageDialog</code> and makes it modal and visible
	 * @param tit
	 * @param msg
	 * @param parentFrame
	 * @param w
	 * @param h
	 */
	public static void _showModalErrorMessageDialog(
			String tit,
			String msg,
			Frame parentFrame) {
		
		int w = Math.min(parentFrame.getWidth(), parentFrame.getHeight()) * 3 / 4;
		MessageDialog dlg = new MessageDialog(
				parentFrame, msg, tit, w, w,
				2 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
		dlg.setModal(true);
		dlg.setVisible(true);
	}

}
