/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.Choice;
import java.awt.Color;
import java.awt.FileDialog;
import java.awt.Label;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.Date;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.geodetic.utils.conversion.GeoUtils;
import es.prodevelop.gvsig.mobile.fmap.proj.CRSFactory;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ColorChooserPanel;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;
import es.prodevelop.gvsig.mobile.gui.tab.ExpansibleScrolledPanel;
import es.prodevelop.gvsig.mobile.gui.tab.TabbedDialog;
import es.prodevelop.gvsig.mobile.project.Project;
import es.prodevelop.gvsig.mobile.project.ProjectSerializer;

/**
 * 
 * This dialog shows and edits the current project properties. It's a tabbed dialog
 * with two tabs: 'Basic' and 'View' which show basic project properties (name, path,
 * description, creation/moification dates) and other carthographic properties
 * (extent, SRS)
 * 
 * @see es.prodevelop.gvsig.mobile.project.Project
 * @see es.prodevelop.gvsig.mobile.project.ProjectSerializer
 * 
 * @author jldominguez
 *
 */

public class ProjectPropertiesDialog extends TabbedDialog {
	
	private static Logger logger = Logger.getLogger(ProjectPropertiesDialog.class);
    
    // frame component needed to open further dialogs
    private MainWindow parent = null;
    private Project project;
    private MapControl theCanvas;
    private boolean fromCloseAll = false;
    
    // ----------------------------------------------------------------
    // ---------------------- BASIC -------------------------
    // 1st row
    private Label nameLabel;
    private TextField nameText;
    // 2nd row
    private Label pathLabel;
    private TextField pathText;
    private TocButton pathButton;
    // 3rd row
    private Label descLabel;
    private TextArea descArea;
    // 4th row
    private Label creationLabel;
    private TextField creationField;
    private Label updateLabel;
    private TextField updateField;
    // ---------------------- BASIC -------------------------
    // ----------------------------------------------------------------
    

    // ----------------------------------------------------------------
    // ---------------------- COORDINATES -------------------------
    private Label srsLabel;
    private Choice srsChoice;

    private Label bboxLabel;

    private Label bboxLabel_x;
    private TextField bboxText_xmin, bboxText_xmax;

    private Label bboxLabel_y;
    private TextField bboxText_ymin, bboxText_ymax;

    private Label bgLabel;
    private ColorChooserPanel bgComponent;
    private Color initialColor;
    // ---------------------- COORDINATES -------------------------
    // ----------------------------------------------------------------
    

    /**
     * The constructor needs a parent frame and the project itself.
     * 
     * @param prj The project whose properties have to be viewed/modified.
     * @param parent_frame parent frame, used for graphical purposes
     */
    public ProjectPropertiesDialog(MapControl canv, MainWindow parent_frame, boolean from_close_all) {
    	
        super(parent_frame, Messages.getText("ProjectPropertiesDialog.0")); //$NON-NLS-1$
        theCanvas = canv;
        project = canv.project;
        fromCloseAll = from_close_all;
        setBounds(
                parent_frame.getX(),
                parent_frame.getY(),
                parent_frame.getWidth(),
                parent_frame.getHeight());

        // Set parent frame which is needed to open further dialogs
        parent = parent_frame;

        init();
    }
    
   
    private void init() {
    	
    	int coord_text_w =
    		(parent.getWidth() - DialogConstants.LABELS_WIDTH - 4 * DialogConstants.XSPACE - 8) / 2;
        
        // --------- create controls: ----------------
        srsLabel = new Label(Messages.getText("ProjectPropertiesDialog.1")); //$NON-NLS-1$
        srsLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        srsLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        srsChoice = getSrsChoice();
        srsChoice.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        srsChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);

        bboxLabel = new Label(Messages.getText("ProjectPropertiesDialog.2")); //$NON-NLS-1$
        bboxLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
        bboxLabel.setAlignment(Label.CENTER);
        bboxLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        
        
        
        bboxLabel_x = new Label(Messages.getText("ProjectPropertiesDialog.3")); //$NON-NLS-1$
        bboxLabel_x.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        bboxLabel_x.setFont(DialogConstants.MENU_FONT);
        bboxText_xmin = new TextField();
        bboxText_xmin.setSize(coord_text_w, DialogConstants.TABLE_LABEL_HEIGHT);
        bboxText_xmin.setEnabled(false);
        bboxText_xmin.setFont(DialogConstants.DIALOG_SMALL_FONT);
        bboxText_xmax = new TextField();
        bboxText_xmax.setSize(coord_text_w, DialogConstants.TABLE_LABEL_HEIGHT);
        bboxText_xmax.setEnabled(false);
        bboxText_xmax.setFont(DialogConstants.DIALOG_SMALL_FONT);

        bboxLabel_y = new Label(Messages.getText("ProjectPropertiesDialog.4")); //$NON-NLS-1$
        bboxLabel_y.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        bboxLabel_y.setFont(DialogConstants.MENU_FONT);
        bboxText_ymin = new TextField();
        bboxText_ymin.setSize(coord_text_w, DialogConstants.TABLE_LABEL_HEIGHT);
        bboxText_ymin.setEnabled(false);
        bboxText_ymin.setFont(DialogConstants.DIALOG_SMALL_FONT);
        bboxText_ymax = new TextField();
        bboxText_ymax.setSize(coord_text_w, DialogConstants.TABLE_LABEL_HEIGHT);
        bboxText_ymax.setEnabled(false);
        bboxText_ymax.setFont(DialogConstants.DIALOG_SMALL_FONT);

        bgLabel = new Label(Messages.getText("ProjectPropertiesDialog.5")); //$NON-NLS-1$
        bgLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        bgLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        bgComponent = new ColorChooserPanel(Color.BLACK, parent);
        
        //----------------------------------------------------------------//
        
        nameLabel = new Label(Messages.getText("ProjectPropertiesDialog.6")); //$NON-NLS-1$
        nameLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        nameLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        nameText = new TextField();
        nameText.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        nameText.setFont(DialogConstants.DIALOG_SMALL_FONT);
        
        pathLabel = new Label(Messages.getText("ProjectPropertiesDialog.7")); //$NON-NLS-1$
        pathLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        pathLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        pathText = new TextField();
        pathText.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        pathText.setFont(DialogConstants.DIALOG_SMALL_FONT);
        pathButton = new TocButton("folder_transp.png",true);
        pathButton.addMouseListener(this);

        descLabel = new Label(Messages.getText("ProjectPropertiesDialog.9")); //$NON-NLS-1$
        descLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        descLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        descArea = new TextArea();
        descArea.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        descArea.setFont(DialogConstants.DIALOG_SMALL_FONT);

        creationLabel = new Label(Messages.getText("ProjectPropertiesDialog.10")); //$NON-NLS-1$
        creationLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        creationLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        creationField = new TextField();
        creationField.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        creationField.setEnabled(false);
        creationField.setFont(DialogConstants.DIALOG_SMALL_FONT);
        
        updateLabel = new Label(Messages.getText("ProjectPropertiesDialog.11")); //$NON-NLS-1$
        updateLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        updateLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
        updateField = new TextField();
        updateField.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
        updateField.setEnabled(false);
        updateField.setFont(DialogConstants.DIALOG_SMALL_FONT);
        // -------------------------------------------------------
        // -------------------------------------------------------
        // -------------------------------------------------------
        
        
        // ------------ tab 1 --------------
        ExpansibleScrolledPanel ep1 = new ExpansibleScrolledPanel();
        // ---------------------
        ExpansibleRow row1_1; row1_1 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        row1_1.add(nameLabel); row1_1.addExtensible(nameText);
        ep1.add(row1_1);
        // ---------------------
        ExpansibleRow row1_5; row1_5 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        row1_5.add(pathLabel);
        row1_5.addExtensible(pathText);
        row1_5.add(pathButton);
        ep1.add(row1_5);
        // ---------------------
        ExpansibleRow row1_6; row1_6 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        row1_6.add(descLabel); row1_6.addExtensible(descArea);
        ep1.addExpansible(row1_6);
        // ---------------------
        ExpansibleRow row1_3; row1_3 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        row1_3.add(creationLabel); row1_3.addExtensible(creationField);
        ep1.add(row1_3);
        // ---------------------
        ExpansibleRow row1_4; row1_4 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        row1_4.add(updateLabel); row1_4.addExtensible(updateField);
        ep1.add(row1_4);
        // ---------------------
        // --------------------
        addTab(Messages.getText("ProjectPropertiesDialog.12"), ep1); //$NON-NLS-1$
        
        // -------------------------------------------------------
        // -------------------------------------------------------
        // -------------------------------------------------------

        // ------------ tab 2 --------------
        ExpansibleScrolledPanel ep2 = new ExpansibleScrolledPanel();
        // ---------------------
        ExpansibleRow row2_1; row2_1 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        row2_1.add(srsLabel); row2_1.addExtensible(srsChoice);
        ep2.add(row2_1);
        // ---------------------
        ExpansibleRow row2_6; row2_6 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        row2_6.add(bgLabel); row2_6.add(bgComponent);
        ep2.add(row2_6);
        // ---------------------
        ExpansibleRow row2_2; row2_2 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        row2_2.addExtensible(bboxLabel); 
        ep2.add(row2_2);
        // ---------------------
        ExpansibleRow row2_3; row2_3 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        row2_3.add(bboxLabel_x); row2_3.add(bboxText_xmin);
        row2_3.addExtensible(bboxText_xmax);
        ep2.add(row2_3);
        // ---------------------
        ExpansibleRow row2_4; row2_4 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        row2_4.add(bboxLabel_y); row2_4.add(bboxText_ymin);
        row2_4.addExtensible(bboxText_ymax);
        ep2.add(row2_4);
        // ---------------------
        ExpansibleRow row2_7; row2_7 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
        ep2.addExpansible(row2_7);
        // ---------------------
        addTab(Messages.getText("ProjectPropertiesDialog.13"), ep2); //$NON-NLS-1$
        // ------------------ tab 1 -------------------------------

        
        if (project != null) {
            nameText.setText(project.getName());
            pathText.setText(project.getPath());
            descArea.setText(project.getDescription());
            try {
            	creationField.setText(Utils.dateToString(project.getCreationDate()));
            } catch (Throwable t){
            	logger.debug("Exception while parsing creation date: " + project.getCreationDate());
            }
            
            try {
            	updateField.setText(Utils.dateToString(project.getLastModified()));
            } catch (Throwable t){
            	logger.debug("Exception while parsing upd. date: " + project.getLastModified());
            }
            // tab 2 
            // srsChoice...
            // bbox xy min max
            bgComponent.setColor(
            		(project.getBgColor() == null) ? Color.MAGENTA : project.getBgColor()
            		);
            // save initial color to decide whether to repaint or not:
            initialColor = bgComponent.getColor();
            
            Rectangle2D r2 = project.getMapContext().getViewPort().getExtent();
            bboxText_xmin.setText("" + r2.getMinX()); //$NON-NLS-1$
            bboxText_xmax.setText("" + r2.getMaxX()); //$NON-NLS-1$
            bboxText_ymin.setText("" + r2.getMinY()); //$NON-NLS-1$
            bboxText_ymax.setText("" + r2.getMaxY()); //$NON-NLS-1$
            getSrsChoice().select(project.getProjection().getAbrev());
        }
        // setting values:
        // tab 1 
        
    }
    
	private void updateProject(){
		logger.debug("Entering updateProject"); //$NON-NLS-1$
        project.setProjection(CRSFactory.getCRS(srsChoice.getItem(srsChoice.getSelectedIndex())));
        logger.debug("Entering updateProject, project.srs = " + project.getProjection().getAbrev()); //$NON-NLS-1$
        project.setBgColor(bgComponent.getColor(), false);
        logger.debug("Project Name: " + nameText.getText()); //$NON-NLS-1$
        project.setPath(pathText.getText());
        project.setName(nameText.getText());
        project.setDescription(descArea.getText(), false);
        project.setLastModified(new Date(), false);        
         
	}

    private Choice getSrsChoice() {
    	
    	if (srsChoice == null) {
    	
    		srsChoice = new Choice();
    		addKnownCodes(srsChoice);
    		// srsChoice.add("EPSG:23030");
    		// srsChoice.add("EPSG:4326");
    		
    	}
		return srsChoice;
	}

	private void addKnownCodes(Choice ch) {
		Iterator iter = GeoUtils.CRSIterator();
		while (iter.hasNext()) ch.add((String) iter.next());
	}

	public void mousePressed(MouseEvent e) {
		// logger.debug("Entering MousePressed"); //$NON-NLS-1$
		
		Object src = e.getSource();
		// logger.debug(src.getClass().getName());
		
		if (src == pathButton) {
			
			FileDialog dlg = new FileDialog(parent, Messages.getText("ProjectPropertiesDialog.22"), FileDialog.LOAD); //$NON-NLS-1$
			dlg.setModal(true);
			dlg.show();

			String getFile = dlg.getFile();
			if (getFile == null) return;
			
			String _dir = dlg.getDirectory();
			String last_ch = _dir.substring(_dir.length() - 1);
			if (last_ch.compareTo(File.separator) == 0) {
				_dir = _dir.substring(0, _dir.length() - 1);
			}
			pathText.setText(_dir);
		} else if (src == okButton){
			// logger.debug("ProjectPropertiesDialog, okButton pressed"); //$NON-NLS-1$
			updateProject();
			theCanvas.project.setLastModified(new Date(), true);
			ProjectSerializer.storeProject(theCanvas.project);
			theCanvas.project.setNew(false);
			if (bgColorHasChanged()) {
				theCanvas.repaint();
			}
			
			if (fromCloseAll) {
				
				try {
					hide();
				} catch (NullPointerException npe) {
					logger.warn("Tabbed dialog. A mysterious NP exception was caught: " + npe.getMessage());
				}
				parent.beforeClosingAndClose();
			}
		}
		super.mousePressed(e);
		
	}
	
	private boolean bgColorHasChanged() {
		
		Color user_color = bgComponent.getColor();
		if (user_color.getRed() != initialColor.getRed()) return true; 
		if (user_color.getGreen() != initialColor.getGreen()) return true; 
		if (user_color.getBlue() != initialColor.getBlue()) return true; 
		return false;
	}

	public void mouseClicked(MouseEvent arg0) { }
	public void mouseReleased(MouseEvent arg0) { }
	public void mouseEntered(MouseEvent arg0) { }
	public void mouseExited(MouseEvent arg0) {
		// System.out.print(this.getClass().getName());
		// System.out.println(arg0);
	}
}
