/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.Choice;
import java.awt.Frame;
import java.awt.Label;
import java.awt.TextField;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.sql.Types;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import com.hardcode.gdbms.engine.values.BooleanValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.FBitSet;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleWindow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.LayersTable;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;

/**^
 * This dialog is used to perform searches by attribute value in vector layers.
 * The user chooses a field, an operation and a second operator. The available
 * operations are: =, <>, >, >=, <, <=, contains, begins with and ends with.
 * 
 * Comparison is resolved through the methods of the <code>Value</code> interface and the
 * classes implementing it.
 * 
 * When the user clicks on "OK", the features  whose value returns <code>true</code>
 * will show as selected.
 * 
 * @see com.hardcode.gdbms.engine.values.Value
 * @see es.prodevelop.gvsig.mobile.fmap.layer.SelectionSupport
 * 
 * 
 * @author jldominguez
 *
 */
public class SearchDialog extends ExpansibleWindow implements MouseListener, ItemListener {
	
	private static Logger logger = Logger.getLogger(SearchDialog.class);
	public static final String OP_EQ_STRING = Messages.getText("SearchDialog.0"); //$NON-NLS-1$
	public static final String OP_GT_STRING = Messages.getText("SearchDialog.1"); //$NON-NLS-1$
	public static final String OP_LT_STRING = Messages.getText("SearchDialog.2"); //$NON-NLS-1$
	public static final String OP_GET_STRING = Messages.getText("SearchDialog.3"); //$NON-NLS-1$
	public static final String OP_LET_STRING = Messages.getText("SearchDialog.4"); //$NON-NLS-1$
	public static final String OP_NEQ_STRING = Messages.getText("SearchDialog.5"); //$NON-NLS-1$
	public static final String OP_CONTAINS_STRING = Messages.getText("SearchDialog.6"); //$NON-NLS-1$
	public static final String OP_BEGINS_WITH_STRING = Messages.getText("SearchDialog.7"); //$NON-NLS-1$
	public static final String OP_ENDS_WITH_STRING = Messages.getText("SearchDialog.8"); //$NON-NLS-1$
	
	private LayersTable toc = null;
	
	private Label titleLabel = null;
	private Choice lyrChoice = null;
	private Label lyrLabel;
	private Choice attrChoice = null;
	private Label attrLabel;
	private Choice opChoice = null;
	private Label opLabel;
	private TextField valField = null;
	private Label valLabel;
	
	private TocButton okButton, cancelButton;

	/**
	 * Constructor. Needs the TOC to list all the available vector layers.
	 * 
	 * @param _toc the current toc, where the dialog will search for vector layers.
	 * @param p parent frame, needed for graphical purposes
	 */
	public SearchDialog(LayersTable _toc, Frame p) {
		super(p);
		toc = _toc;
		setBounds(p.getX(), p.getY(), p.getWidth(), p.getHeight());
		init();
		setVisible(true);
	}
	
	private void init() {
		
		cancelButton = new TocButton(DialogConstants.DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME, 0, true, false);
		cancelButton.addMouseListener(this);
		okButton = new TocButton(DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, "toc_ok_dis_small.png", 1, true, false); //$NON-NLS-1$
		okButton.addMouseListener(this);
		
		titleLabel = new Label(Messages.getText("SearchDialog.10")); //$NON-NLS-1$
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		titleLabel.setAlignment(1);

		lyrLabel = new Label(Messages.getText("SearchDialog.11")); //$NON-NLS-1$
		lyrLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		attrLabel = new Label(Messages.getText("SearchDialog.12")); //$NON-NLS-1$
		attrLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		opLabel = new Label(Messages.getText("SearchDialog.13")); //$NON-NLS-1$
		opLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		valLabel = new Label(Messages.getText("SearchDialog.14")); //$NON-NLS-1$
		valLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);

		lyrChoice = getLyrChoice(toc, okButton);
		lyrChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);
		attrChoice = getAttrChoice();
		attrChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);
		opChoice = getOpChoice();
		opChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);
		valField = getValTextField();
		valField.setFont(DialogConstants.DIALOG_SMALL_FONT);
		
		titleLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		
		lyrLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		lyrChoice.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		attrLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		attrChoice.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		opLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		opChoice.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		valLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		valField.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		
		// -------------------
		
		// --------------
		ExpansibleRow row00 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row00.addExtensible(titleLabel);
		add(row00);
		// --------------------
		ExpansibleRow row0 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row0.add(lyrLabel); row0.addExtensible(lyrChoice);
		add(row0);
		// --------------------
		ExpansibleRow row01 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row01.add(attrLabel); row01.addExtensible(attrChoice);
		add(row01);
		// --------------
		ExpansibleRow row1 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row1.add(opLabel); row1.addExtensible(opChoice);
		add(row1);
		// --------------
		ExpansibleRow row2 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row2.add(valLabel); row2.addExtensible(valField);
		add(row2);
		// --------------
		ExpansibleRow row5 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		addExpansible(row5);
		// --------------
		ExpansibleRow row6 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row6.add(okButton); row6.add(cancelButton);
		add(row6);
		// --------------
		addEmptyRowIfPhoneME(this);
	}
	
	private void reloadAttrChoice() {
		
		if (attrChoice == null) {
			attrChoice = getAttrChoice();
		}
		
		attrChoice.removeAll();
		try {
			SelectableDataSource sds = getSDS();
			String[] atts = sds.getFieldNames();
			for (int i=0; i<atts.length; i++) attrChoice.addItem(atts[i]);
		} catch (Exception e) {
			logger.error("While setting attributes in combo."); //$NON-NLS-1$
		}
	}

	private Choice getLyrChoice(LayersTable t, TocButton b) {
		
		FLayer[] ll = t.getLayers();
		Choice resp = new Choice();
		
		for (int i=0; i<ll.length; i++) {
			FLayer lyr = ll[i];
			if (lyr instanceof FLyrVect) {
				resp.addItem(ll[i].getName());
			}
		}
		b.setEnabled(resp.getItemCount() > 0);
		resp.addMouseListener(this);
		resp.addItemListener(this);
		return resp;
	}

	private TextField getValTextField() {
		return new TextField();
	}

	private Choice getOpChoice() {
		Choice resp = new Choice();
		resp.addItem(OP_EQ_STRING);
		resp.addItem(OP_GT_STRING);
		resp.addItem(OP_GET_STRING);
		resp.addItem(OP_LT_STRING);
		resp.addItem(OP_LET_STRING);
		resp.addItem(OP_NEQ_STRING);
		
		resp.addItem(OP_CONTAINS_STRING);
		resp.addItem(OP_BEGINS_WITH_STRING);
		resp.addItem(OP_ENDS_WITH_STRING);
		return resp;
	}

	private Choice getAttrChoice() {
		Choice resp = new Choice();
		

		try {
			SelectableDataSource sds = getSDS();
			String[] atts = sds.getFieldNames();
			for (int i=0; i<atts.length; i++) resp.addItem(atts[i]);
		} catch (Exception e) {
			logger.error("While setting attributes in combo."); //$NON-NLS-1$
		}
		return resp;
	}

	private SelectableDataSource getSDS() throws FMapDriverException {
		FLyrVect layer = getSelectedLayer();
		ReadableVectorial rv = layer.getSource();
		return rv.getRecordset();
	}

	private void executeSearch() {
		
		if (getSelectedLayer() == null) return;
		
		try {
			FBitSet fbs = new FBitSet(); 
			SelectableDataSource sds = getSDS();
			Value userv, v;
			
			int cnt = (int) sds.getRowCount();
			if (cnt < 1) return;
			
			int att_ind = getAttributeIndex(sds.getFieldNames(), attrChoice.getSelectedItem());
			if (att_ind == -1) {
				logger.error("Attribute not found: " + attrChoice.getSelectedItem()); //$NON-NLS-1$
				return;
			}

			boolean is_numeric = isNumericType(sds.getFieldType(att_ind));
			
			if (is_numeric) {
				
				double usr_v = 0.0;
				try {
					usr_v = Double.parseDouble(valField.getText());
				} catch (Exception parse_exc) {
					logger.error("Not a valid numeric value: " + valField.getText() + " (0 assumed)"); //$NON-NLS-1$ //$NON-NLS-2$
				}
				userv = ValueFactory.createValue(usr_v);
			} else {
				userv = ValueFactory.createValue(valField.getText());
			}
			
			if (opChoice.getSelectedItem().compareTo(OP_GT_STRING) == 0) {
				for (int i=0; i<cnt; i++) {
					v = sds.getFieldValue(i, att_ind);
					if (((BooleanValue) v.greater(userv)).getValue()) fbs.set(i);
				}
				
			} else {
				if (opChoice.getSelectedItem().compareTo(OP_LT_STRING) == 0) {
					for (int i=0; i<cnt; i++) {
						v = sds.getFieldValue(i, att_ind);
						if (((BooleanValue) v.less(userv)).getValue()) fbs.set(i);
					}

				} else {
					if (opChoice.getSelectedItem().compareTo(OP_EQ_STRING) == 0) {
						for (int i=0; i<cnt; i++) {
							v = sds.getFieldValue(i, att_ind);
							if (((BooleanValue) v.equals(userv)).getValue()) fbs.set(i);
						}

					} else {
						if (opChoice.getSelectedItem().compareTo(OP_GET_STRING) == 0) {
							for (int i=0; i<cnt; i++) {
								v = sds.getFieldValue(i, att_ind);
								if (((BooleanValue) v.greaterEqual(userv)).getValue()) fbs.set(i);
							}

						} else {
							if (opChoice.getSelectedItem().compareTo(OP_LET_STRING) == 0) {
								for (int i=0; i<cnt; i++) {
									v = sds.getFieldValue(i, att_ind);
									if (((BooleanValue) v.lessEqual(userv)).getValue()) fbs.set(i);
								}

							} else {
								if (opChoice.getSelectedItem().compareTo(OP_NEQ_STRING) == 0) {
									for (int i=0; i<cnt; i++) {
										v = sds.getFieldValue(i, att_ind);
										if (((BooleanValue) v.notEquals(userv)).getValue()) fbs.set(i);
									}
								} else {
									if (opChoice.getSelectedItem().compareTo(OP_CONTAINS_STRING) == 0) {
										for (int i=0; i<cnt; i++) {
											v = sds.getFieldValue(i, att_ind);
											if (contains(v, userv)) fbs.set(i);
										}
									} else {
										if (opChoice.getSelectedItem().compareTo(OP_BEGINS_WITH_STRING) == 0) {
											for (int i=0; i<cnt; i++) {
												v = sds.getFieldValue(i, att_ind);
												if (beginsWith(v, userv)) fbs.set(i);
											}
										} else {
											if (opChoice.getSelectedItem().compareTo(OP_ENDS_WITH_STRING) == 0) {
												for (int i=0; i<cnt; i++) {
													v = sds.getFieldValue(i, att_ind);
													if (endsWith(v, userv)) fbs.set(i);
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
			
			if (!fbs.isEmpty()) {
				// toc.getMapControl().setProcessing(true);
				sds.setSelection(fbs);
			}
			
		} catch (Exception ex) {
			logger.error("SEARCH DIALOG: " + ex.getMessage()); //$NON-NLS-1$
		}
	}
	
	private boolean contains(Value v_big, Value v_small) {
		String v_big_lower = v_big.toString().toLowerCase();
		String v_small_lower = v_small.toString().toLowerCase();
		return v_big_lower.indexOf(v_small_lower) != -1;
	}
	
	private boolean beginsWith(Value v_big, Value v_small) {
		String v_big_lower = v_big.toString().toLowerCase();
		String v_small_lower = v_small.toString().toLowerCase();
		return v_big_lower.indexOf(v_small_lower) == 0;
	}

	private boolean endsWith(Value v_big, Value v_small) {
		String str_big = v_big.toString().toLowerCase();
		String str_sma = v_small.toString().toLowerCase();
		int l = str_big.length() - str_sma.length();
		if (l < 0) return false;
		return str_big.indexOf(str_sma) == l;
	}

	
	private boolean isNumericType(int t) {
		
		if ((t == Types.BIGINT)
				|| (t == Types.BINARY)
			    || (t == Types.BIT)
			    || (t == Types.DECIMAL)
			    || (t == Types.DOUBLE)
			    || (t == Types.FLOAT)
			    || (t == Types.INTEGER)
		){
			return true;
		}
		return false;
	}

	private int getAttributeIndex(String[] ff, String f) {
		for (int i=0; i<ff.length; i++) {
			if (ff[i].compareToIgnoreCase(f) == 0) return i; 
		}
		return -1;
	}

	private FLyrVect getSelectedLayer() {
		String lyrname = lyrChoice.getSelectedItem();
		
		if (lyrname == null) return null;

		FLayer[] ll = toc.getLayers();
		for (int i=0; i<ll.length; i++) {
			FLayer lyr = ll[i];
			if (lyr instanceof FLyrVect) {
				if (lyr.getName().compareToIgnoreCase(lyrname) == 0) {
					return (FLyrVect) lyr;
				}
			}
		}
		return null;
	}

	public void mousePressed(MouseEvent e) {
		
		Object src = e.getSource();
		
		if (src == okButton) {
			try {
				hide();
			} catch (NullPointerException npe) {
				logger.warn("Dialog hidden: A mysterious NP exception was caught: " + npe.getMessage());
			}
			toc.getMapControl().repaint();
			executeSearch();
		}

		if (src == cancelButton) {
			try {
				hide();
			} catch (NullPointerException npe) {
				logger.warn("Dialog hidden: A mysterious NP exception was caught: " + npe.getMessage());
			}
		}
	}

	public void mouseEntered(MouseEvent e) { }
	public void mouseExited(MouseEvent e) { }
	public void mouseClicked(MouseEvent e) { }
	public void mouseReleased(MouseEvent e) { }

	public void itemStateChanged(ItemEvent e) {
		
		Object src = e.getSource();
		
		if (src == lyrChoice) {
			reloadAttrChoice();
		}

	}

}
