/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.Color;
import java.awt.Frame;
import java.awt.Label;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.common.ProcessUtils;

/**
 * This class is used to show a hyperlink that starts the web browser when the
 * user clicks on it. It is used when an info-by-point operation returns a XML/HTML
 * file that contains a <code>a</code> element with a <code>href</code> attribute
 *  
 *  @see es.prodevelop.gvsig.mobile.gui.dialog.InfoAttributesDialog
 *  
 * @author jldominguez
 *
 */
public class UrlLabel extends Label implements MouseListener {
	
	private static Logger logger = Logger.getLogger(UrlLabel.class);	
	private String targetURL = null;
	private Frame parent = null;
	
	/**
	 * Constructor. Needs the text to be showsn and the URL to be opened.
	 * 
	 * @param caption text to be shown
	 * @param url URL that will be opened when the user clicks on this label.
	 * @param p parent frame, needed for graphical purposes
	 */
	public UrlLabel(String caption, String url, Frame p) {
		super(caption);
		
		parent = p;
		setForeground(Color.BLUE);
		targetURL = url;
		addMouseListener(this);
	}

	public void mouseClicked(MouseEvent e) { }
	public void mouseEntered(MouseEvent e) { }
	public void mouseExited(MouseEvent e) { }
	public void mouseReleased(MouseEvent e) { }

	public void mousePressed(MouseEvent e) {
		
		if (targetURL == null) {
			logger.error("Dest url is NULL !!"); //$NON-NLS-1$
			return;
		}
		
		try {
			logger.debug("Opening url: " + targetURL); //$NON-NLS-1$
			ProcessUtils.startOrForeground(
					"YOU SHOULD NOT BE SEEING THIS", //$NON-NLS-1$
					"iexplore.exe", //$NON-NLS-1$
					targetURL);
		} catch (Exception ex) {
			logger.error("While accerssing url: " + ex.getMessage()); //$NON-NLS-1$
			AcceptCancelDialog.informUser(
                    Messages.getText("UrlLabel.7"), //$NON-NLS-1$
                    ex.getMessage(),
					parent);
		} catch (Throwable th) {
			logger.error("While accerssing url: " + th.getMessage()); //$NON-NLS-1$
			
			AcceptCancelDialog.informUser(
                    Messages.getText("UrlLabel.9"), //$NON-NLS-1$
                    th.getMessage(),
					parent);
		}
	}

}