/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog.control;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.gui.tab.ExpansibleScrolledPanel;

/**
 * 
 * This new dialog resizes automatically its components to fit the screen,
 * both horizontally and vertically.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tab.ExpansibleScrolledPanel
 * 
 * @author jcarras
 *
 */
public class ExpansibleDialog extends Dialog implements ActionListener {
	
	private static Logger logger = Logger.getLogger(ExpansibleDialog.class);
	private ExpansibleScrolledPanel panel;
	
	public ExpansibleDialog(Frame p){
		super(p);
		initialize();
	}
	public ExpansibleDialog(Dialog p){
		super(p);
		initialize();
	}
	private void initialize(){
		int x=0;
		int y=0;
		if (Utils.USING_PDA) {
			setUndecorated(true);
		} else {
			y=20;
		}
		setLayout(null);
		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				expand();
			}
		});
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				close();
			}
		});
		panel = new ExpansibleScrolledPanel(this.getWidth()-x,this.getHeight()-y);
		panel.setBounds(x, y, this.getWidth()-x, this.getHeight()-y);
		super.add(panel);
	}
	
	public void close() {
		try {
			dispose();
		} catch (Exception ex) {
			logger.error("A mysterious exception was caught: " + ex.getMessage());
		}
	}
	
	public void expand(){
		panel.setSize(this.getWidth()-panel.getBounds().x, this.getHeight()-panel.getBounds().y);
		panel.expand();
	}
	public Component add(Component com){
		//com.setLocation(DialogConstants.XSPACE, currY);
		//currY += DialogConstants.YSPACE + com.getHeight();
		//return super.add(com);
		return panel.add(com);
	}
	
	public void addExpansible(ExpansibleRow row){
		//extCom = row;
		panel.addExpansible(row);
	}
	
	private int getMaxHeight(){
		return panel.getMaxHeight();
	}
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		//System.out.print(e.toString());
	}


	
}
