/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tab;

import java.awt.Graphics;
import java.awt.Label;
import java.awt.event.MouseListener;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;

/**
 * This class represenst one tab label in a <code>TabbedDialog</code>.
 * It simply listens to mouse events and notifies that the card layout must be
 * updated.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tab.TabbedPanel
 * @see es.prodevelop.gvsig.mobile.gui.tab.TabbedDialog
 * @see es.prodevelop.gvsig.mobile.gui.tab.TabbedPanelTopPanel
 * 
 * @author jldominguez
 *
 */
public class TabLabel extends Label {
	
	private static Logger logger = Logger.getLogger(TabLabel.class);
	private boolean selected = false;
	
	/**
	 * Sets this label as selected
	 * 
	 * @param b
	 */
	public void setSelected(boolean b) {
		selected = b;
	}
	
	/**
	 * Gets whether this label is currently selected.
	 * @return whether this label is currently selected.
	 */
	public boolean isSelected() {
		return selected;
	}
	
	/**
	 * Constructor. 
	 * @param t the string to be shown 
	 * @param ml the mouselistener to be notified of mouse events
	 */
	public TabLabel(String t, MouseListener ml) {
		super(t);
		setFont(DialogConstants.DIALOG_SMALL_FONT);
		setAlignment(Label.CENTER);
		addMouseListener(ml);
		int w =
			getFontMetrics(DialogConstants.DIALOG_SMALL_FONT).stringWidth(t) +
			10 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
		
		setSize(w, DialogConstants.TABLE_LABEL_HEIGHT);
	}
	
	public void paint(Graphics g) {
		
		super.paint(g);
		
		g.drawRect(0, 0, getWidth() - 1, getHeight() - 1);			
		if (selected) {
			g.setColor(getBackground());
			g.drawLine(1, getHeight() - 2, getWidth() - 2, getHeight() - 2);
			g.drawLine(1, getHeight() - 1, getWidth() - 2, getHeight() - 1);
			g.drawLine(1, getHeight()    , getWidth() - 2, getHeight()    );
		}

	}

}
