/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import java.awt.Container;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.MouseListener;

import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;

/**
 * Abstract class for gvSIG Mobile buttons. All types of buttons 
 * inheritate from GenericButton
 * 
 * @author jldominguez
 * @author mmontesinos
 * 
 * @see TBWidget
 *
 */
public class GenericButton extends Container{


	protected Image icon;
	protected Image selIcon;
	protected boolean selected = false;
	
	/**
	 * Constructs a new instance of this class given the icons <code>String</code>s
	 * and the listener of the events for this new instance
	 * @param _icon Image to be loaded as <i>normal</i> image 
	 * @param _selicon Image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param ml Listener of Mouse Events, which must be a
	 * 				 <code>MouseListener</code>
	 */
	public GenericButton(String _icon, String _selicon, MouseListener ml) {

		icon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, _icon);
		selIcon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, _selicon);
		addMouseListener(ml);
	}
	
	/**
	 * Constructs a new instance of this class given the icons <code>Image</code>s
	 * and the listener of the events for this new instance
	 * @param _icon <code>Image</code> to be loaded as <i>normal</i> image 
	 * @param _selicon <code>Image</code> to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param ml Listener of Mouse Events, which must be a
	 * 				 <code>MouseListener</code>
	 */
	public GenericButton(Image _icon, Image _selicon, MouseListener ml) {
		icon = _icon;
		selIcon = _selicon;
		addMouseListener(ml);
		
	}

	/**
	 * Sets the widget to <i>selected/unselected</i> status
	 * @param _selected <code>true</code>: Sets the widget to <i>selected</i> status
	 * 		  <code>false</code>: Sets the widget to <i>unselected</i> status
	 */
	public void setSelected(boolean _selected) {
		selected = _selected;
	}
	
	/**
	 * @return Returns the selected.
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * @return Returns the button's <i>normal</i> Icon  (<code>Image</code>).
	 */
	protected Image getIcon() {
		return icon;
	}

	/**
	 * @return Returns the <i>selected</i> Icon (<code>Image</code>).
	 */
	protected Image getSelectedIcon() {
		return selIcon;
	}

	/**
	 * Forces the paint of the object. This method should be called when the 
	 * content of the GenericButton needs to be painted.
	 * This method overrides the paint method of the <code>Component</code> parent
	 */
	public void paint(Graphics g) {
		if (icon != null) g.drawImage(icon, 0, 0, null);
	}
}