/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import es.prodevelop.gvsig.mobile.common.GvsigMobileException;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;

/**
 * Interface for specifying the common issues of a buttons container, e.g.
 * a <code>ToolBar</code> or a <code>TBComboButton</code>
 * 
 * @author mmontesinos
 *
 */
public interface IToolContainer {
	/**
	 * Method used for adding a <code>TBButton</code> to a <code>IToolContainer</code>
	 * @param button to be added
	 * @param index position in the array of buttons
	 * @throws GvsigMobileException
	 */
	public void addButton(TBButton button, int index) throws GvsigMobileException;
	/**
	 * Gets the array of <code>TBButton</code>s that the object contain
	 * @return an array of code>TBButton</code>s
	 */
	public TBButton [] getButtons();
	/**
	 * Gets the total number of buttons in the container 
	 * @return total number of buttons (buttons array length)
	 */
	public int getButtonCount();
	/**
	 * Gets the MainWindow the <code>IToolContainer</code> is attached to
	 * @return the MainWindow of the object
	 */
	public MainWindow getWindow();
	/**
	 * Sets the index of the active tool in the container.
	 * Use only if an active tool makes sense in the container.
	 * For instance it's necessary in a combobutton list but not in a toolbar
	 * @param index of the new tool to be set to active in the container
	 */
	public void setActiveTool(int index);
	/**
	 * Forces repaint of the container after changes have been made
	 *
	 */
	public void repaint();
	
}
