/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;

/**
 * 
 * This class is used to show the startup logo when the application starts.
 * 
 * @author jcarras
 *
 */
public class SplashWidget extends Component{
	
	protected Image icon;
	protected int width;
	protected int height;
	private static Logger logger = Logger.getLogger(SplashWidget.class);
	
	private boolean withText = false;
	private String message = "";
	private Font messageFont;
	private Color fColor, bColor;
	
	/**
	 * Constructor
	 * 
	 * @param icon image icon (for example, a 200 x 200 image)
	 * @param width the width of the image
	 * @param height the height of the image
	 */
	public SplashWidget(Image icon, int _width, int _height){
		this.icon=icon;
		width = _width;
		height = _height;
		setBounds(0, 0,width,height);
	}
	
	public SplashWidget(
			String msg, 
			Color fground, Color bground,
			int canvasw) {
		
		message = msg;
		withText = true;
		messageFont = DialogConstants.DIALOG_BOLD_FONT;
		fColor = fground;
		bColor = bground;
		width = canvasw;
		height = 3 * GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT;
		setPos(0, 0);
	}
	
	
	/**
	 * Resets image position
	 * 
	 * @param x new x coordinate
	 * @param y new y coordinate
	 */
	public void setPos(int x, int y){
		setBounds(x,y,width,height);
	}
	
	public void paint(Graphics g) {
		
		if (withText) {
			
			int top = 5 + GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT + messageFont.getSize();
			int textw = g.getFontMetrics(messageFont).stringWidth(message);
			int left = width - textw - 5;
			g.setFont(messageFont);
			g.setColor(bColor);
			g.drawString(message,
					left + 1, // GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR,
					top + 1); // GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR);
			g.setColor(fColor);
			g.drawString(message, left, top);
			
		} else {
			if (icon != null && isVisible()) {
				g.drawImage(icon, getX(), getY(), null);
				// logger.debug("Widget painted");
			}
		}
	}
	
}
