/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import java.awt.Graphics;
import java.awt.event.MouseListener;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;

/**
 * Class which shows a drop down arrow to allow the user to click
 * on it and get a drop-down menu with all the buttons included in the
 * combo button list plus a short text description
 * @author mmontesinos
 *
 */
public class TBDropDownButton extends TBWidget {
	
	protected static Logger logger = Logger.getLogger(TBComboButton.class);

	/**
	 * Class constructor. It needs an owner and a  <code>MouseListener</code> as parameters
	 * @param owner which must be <code>IToolContainer</code> interface compliant
	 * 				(<code>TBComboButton</code>)
	 * @param parent which must be a <code>MouseListener</code>
	 */
	public TBDropDownButton(IToolContainer owner, MouseListener parent) {
		//Parent constructor given two images and a MouseListener
		super(owner, GvsigMobileGUIConstants.DROP_DOWN_ICON, GvsigMobileGUIConstants.DROP_UP_ICON, parent);
		//Set location
		setBounds(GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH, 0, GvsigMobileGUIConstants.ARROW_WIDTH, GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT);
		//addMouseListener(parent);
	}
	
	/**
	 * Override method of <code>paint</code> method which forces the paint of
	 * the component with status control
	*/
	public void paint(Graphics g) {
		if (selected) {
			if (selIcon != null) g.drawImage(selIcon, GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH, 0, null);
		} else {
			if (icon != null) {
				if (g.drawImage(icon, GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH, 0, null) == false){
					//Error;
					logger.error("Error trying to load the drop-down button " + this.toString() + " at the tool bar");
				}
			}
		}
	}
	
	/**
	 * Changes the <i>selected/unselected</i> state of the button.
	 * If it's <i>selected</i>, the new status will be <i>selected</i>
	 * and vice versa 
	 *
	 */
	public void switchButton() {
		selected = !selected;
	}


}