/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import java.awt.Graphics;
import java.awt.event.MouseListener;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;

/**
 * Help button placed at the end of a Tool Bar which shows a help menu with a 
 * list of the buttons placed in the toolbar
 * 
 * @author jldominguez
 * @author mmontesinos
 *
 */
public class TBHelpButton extends TBWidget {
	
	/**
	 * Creates an instance of <code>TBHelpButton</code> given the owner 
	 * (<code>ToolBar</code>), the icons and the<code>MouseListener</code>
	 * @param _owner	<code>ToolBar</code> to which the help button is attached
	 * @param _icon		<i>Normal</i> image of the button
	 * @param _selicon	<i>Selected</i> (help shown) image of the button
	 * @param ml		<code>MouseListener</code> of this button
	 */
	public TBHelpButton(ToolBar _owner, String _icon, String _selicon, MouseListener ml) {
		super(_owner, _icon, _selicon, ml);
		setVisible(false);
		setBounds(GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_WIDTH + GvsigMobileGUIConstants.HELP_ICON_WIDTH, 0,
				GvsigMobileGUIConstants.HELP_ICON_WIDTH, GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT);
	}
		
	/**
	 * Forces the paint of the object. This method should be called when the 
	 * content of the TBHelpButton needs to be painted.
	 * This method overrides the paint method of the <code>TBWidget</code>
	 * superclass parent by painting a different image if the <code>selected</code>
	 * is set to <code>true</code> or not
	 */
	public void paint(Graphics g) {
		if (selected) {
			if (selIcon != null) g.drawImage(selIcon, 0, 0, null);
		} else {
			if (icon != null) g.drawImage(icon, 0, 0, null);
		}
	}
	
	/**
	 * Changes the <i>selected/unselected</i> state of the button.
	 * If it's <i>selected</i>, the new status will be <i>selected</i>
	 * and vice versa 
	 *
	 */
	public void switchButton() {
		selected = !selected;
	}


}
