/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Image;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileException;
import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;


/**
 * A TBMenu is a rectangle which is shown to get mor
 * Class that represents a drop-down menu in a toolbar. It is displayed 
 * after a <code>TBDropDownButton</code> in a <code>TBComboButton</code> is
 * clicked 
 * @author mmontesinos
 *
 */
public class TBMenu extends Container implements IToolContainer {

	private int x = 32;
	private int y = GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT;
	private int width = GvsigMobileGUIConstants.MENU_WIDTH;
	private int height = GvsigMobileGUIConstants.MENU_HEIGHT;
	protected MainWindow mainWindow = null;
	protected IToolContainer owner;
	private TBButton[] buttons = new TBButton[0];
	protected static Logger logger = Logger.getLogger(TBMenu.class);
	
	public static Image BACKGROUND_STRIP = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "background.png");
	
	/**
	 * Creates an instance of <code>TBMenu</code> in a Window from a position
	 * (usually the bottom-left corner of a combo button after checking that
	 * there's enough space in the screen) 
	 * @param _owner 	ToolBar widget which displays the menu. It must
	 * 					accomplish <code>IToolContainer</code> interface and
	 * 					contain the array of <code>TBButton</code>s 
	 * 					to be displayed in the menu 
	 * @param xTop 		X Coordinate of the left border of the menu
	 * @param yTop		Y Coordinate of the top border of the menu
	 */
	public TBMenu(IToolContainer _owner, int xTop, int yTop) {
		
		TBButton[] _buttons;
		
		mainWindow = _owner.getWindow();
		MapControl _mc = mainWindow.getMapControl();
		_buttons = _owner.getButtons();
		owner = _owner;
		
		/*
		 * Let's calculate position
		 * If there's enough space the top-left corner of the menu must be
		 * the bottom-left corner of the Combo Button (the whole TBComboButton
		 * not the drop-down button)
		 * If not, then it will be aligned to the right of the screen
		 */
		if ((mainWindow.getWidth() - xTop - GvsigMobileGUIConstants.MENU_WIDTH) >= 0) {//space enough
			x = xTop;
			y = yTop;
		} else{ //not enough space
			x = mainWindow.getWidth() - GvsigMobileGUIConstants.MENU_WIDTH;
			y = yTop;
		}
		height = _buttons.length * GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT + 5; 
		
		this.setBounds(x, y, width, height);
		_mc.add(this);
		
		addButtons (_buttons);
		addMouseListener(mainWindow);
		
		/*
		 * Let's asign this menu as the active menu in the mainWindow so that we can
		 * close it when clicked outside the menu
		 */ 
		mainWindow.dropMenu = this;
		//this.paintComponents(this.getGraphics());
		
	}
	
	/*
	 * Adds all the buttons in the <code>buttons</code> array attribute to the menu
	 */
	private void addButtons(TBButton[] _buttons) {
		for (int i = 0; i < _buttons.length; i++){
			try {
				addButton(_buttons[i], i);
			} catch (GvsigMobileException e){
				logger.error(e.getMessage());
				// e.printStackTrace();
			}
		}
	}

	/**
	 * Adds a new item to the menu. The items must be instances of 
	 * <code>TBButton</code>
	 * @param button Button to be added to the menu
	 *
	 */
	public void addButton(TBButton button, int index) throws GvsigMobileException {
		int newLength = 0;
		
		if (index < 0){
			throw new GvsigMobileException(GvsigMobileGUIConstants.ERROR_INDEX_UNDER_0);
		} else if (index > buttons.length){
			throw new GvsigMobileException(GvsigMobileGUIConstants.ERROR_TB_BUTTON_GAPS);
		} 
		
		newLength = buttons.length + 1;
		TBButton [] newButtons = new TBButton [newLength];
		
		if (buttons.length == 0){ //First button to be added
			buttons = newButtons;
		} else{
			if (index > 0){ //Left-side of the array need to be copied
				System.arraycopy (buttons, 0, newButtons, 0, index);
			}
			if (index < newLength - 1){ //Rigth-side of the array need to be copied
				System.arraycopy (buttons, index, newButtons, index + 1, newLength - index - 1);
			}
			buttons = newButtons;
		}
	
		/*
		 * We have a new buttons array with an empty space for the new
		 * TBButton to be created
		 */
		
		buttons[index] = button.buttonClone();
		buttons[index].setVisible(true);

		add(buttons[index]);
		//mainWindow.getMapControl().add(buttons[index]);
		//buttons[index].setBounds(this.x + 2, this.y + 2 + index * GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT, GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH, GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT);
		buttons[index].setBounds(2, 2 + index * GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT, GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH, GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT);
		
		logger.debug("MENU BUTTON BOUNDS: " + buttons[index].getBounds().toString());
		//buttons[index].addMouseListener(mainWindow);
	
	}
	
	/**
	 * Returns one tool in the list of tools displayed in the menu,
	 * given its index
	 * @param index index number of the tool in the array of buttons following
	 * 				the order number specified in the creation of the menu 
	 * @return		the tool in the index number
	 */
	public TBButton getTool(int index)  throws GvsigMobileException {
		
		if (index < 0){
			throw new GvsigMobileException(GvsigMobileGUIConstants.ERROR_INDEX_UNDER_0);
		} else if (index > (buttons.length - 1)){
			throw new GvsigMobileException(GvsigMobileGUIConstants.ERROR_INDEX_OVERFLOW);
		} else{
			return buttons[index];
		}
				
	}
	
	/**
	 * Closes the <code>TBMenu</code> and removes all its components
	 *
	 */
	public void close(){
		for (int i = 0; i < buttons.length; i++){
			mainWindow.getMapControl().remove(buttons[i]);
			buttons[i].setVisible(false);
		}
		removeAll();
		buttons = null;
		mainWindow.getMapControl().remove(this);
		setVisible(false);
		mainWindow.dropMenu = null;
		System.gc();
	}

	/**
	 * Returns the number of buttons existing in the menu.
	 * 
	 * @return the number of buttons
	 */
	public int getButtonCount(){
		return buttons.length;
	}
	
	/**
	 * Returns the array of buttons of type <code>TBButton</code> that
	 * the menu contains
	 * @return array of buttons of type <code>TBButton</code>
	 */
	public TBButton [] getButtons() {
		return buttons;
	}


	/**
	 * Returns the MainWindow of the menu
	 * @return  MainWindow where the menu has been placed
	 */
	public MainWindow getWindow() {
		return this.owner.getWindow();
	}

	/**
	 * @return Returns the owner, which is <code>IToolContainer</code> compliant
	 */
	public IToolContainer getOwner() {
		return owner;
	}

	/**
	 * stub method. Not used
	 */
	public void setActiveTool(int index) {
		// Not used in this class	
	}
	
	/**
	 * Override method of <code>paint</code> method which forces the paint of
	 * the component
	*/
	public void paint(Graphics g) {
		if (this.isVisible()) {
			//OJO
			super.paint(g);
			
			//Drawing of border
			g.setColor(GvsigMobileGUIConstants.MENU_BORDER_COLOR);
			g.drawRect(0, 0, width - 2, height - 2);
			
			//Drawing of border shadow
			g.setColor(GvsigMobileGUIConstants.MENU_BORDER_SHADOW_COLOR);
			g.drawLine(1, height - 1, width - 1, height - 1);
			g.drawLine(width - 1, 1, width - 1, height - 1);
			
			//Filling of empty spaces
			g.setColor(GvsigMobileGUIConstants.MENU_BORDER_COLOR);
			g.drawLine(1, 1, width - 3, 1); //Top gap above buttons
			g.drawLine(1, 2, 1, height - 3); //Left gap before buttons
			g.drawLine(1, height - 3, width - 3, height - 3); //Bottom gap below buttons
			
			// g.fillRect(GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH + 2, 2, width - GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH - 3, height - 5); //Rigth gap after buttons
			
			//Buttons help text writing
			g.setFont(DialogConstants.DIALOG_SMALL_FONT);
			g.setColor(Color.WHITE);
			//Let's draw the help text of every button
			//Possible enhance: use decorator pattern
			for (int i = 0; i < buttons.length; i++){
				g.drawImage(
						BACKGROUND_STRIP,
						GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH + 2,
						buttons[i].getY(),
						null);

				g.drawString(buttons[i].getHelpText(),
						GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH + 5,
						17 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR + i * GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT);
			}
		}
	}

}
