/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import java.awt.Image;
import java.awt.event.MouseListener;

import org.apache.log4j.Logger;

/**
 * Abstract class for toolbar elements. All types of buttons in a toolbar
 * inheritate from TBWidget
 * @author mmontesinos
 * 
 * @see TBButton
 * @see ToolBarSelectedGroupButton
 * @see ToolBarHelpButton
 *
 */
public abstract class TBWidget extends GenericButton {

	protected static Logger logger = Logger.getLogger(TBWidget.class);
	protected IToolContainer owner = null;
	
	/**
	 * Constructs a new instance of this class given the owner where it should
	 * be placed (which must accomplish a <code>IToolContainer</code> interface), the icons and the
	 * listener of the events for this new instance
	 * @param _owner Parent of the TBWidget (<code>IToolContainer</code> compliant,
	 * 				 e.g. a <code>ToolBar</code>) where the button is to be added
	 * @param _icon Name of the image to be loaded as <i>normal</i> image 
	 * @param _selicon Name of the image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param ml Listener of Mouse Events, which must be a
	 * 				 <code>MouseListener</code>
	 */
	public TBWidget(IToolContainer _owner, String _icon, String _selicon, MouseListener ml) {
		super(_icon, _selicon, ml);
		owner = _owner;
	}

	/**
	 * Constructs a new instance of this class given the icons and the
	 * listener of the events for this new instance
	 * @param _icon Name of the image to be loaded as <i>normal</i> image 
	 * @param _selicon Name of the image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param ml Listener of Mouse Events, which must be a
	 * 				 <code>MouseListener</code>
	 */
	public TBWidget(String _icon, String _selicon, MouseListener ml) {
		super(_icon, _selicon, ml);
	}

	/**
	 * Constructs a new instance of this class given the owner where it should
	 * be placed (which must accomplish a <code>IToolContainer</code> interface), the icons <code>Image</code>s
	 * and the listener of the events for this new instance
	 * @param _owner Parent of the TBWidget (<code>IToolContainer</code> compliant,
	 * 				 e.g. a <code>ToolBar</code>) where the button is to be added
	 * @param _icon Image to be loaded as <i>normal</i> image 
	 * @param _selicon Image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param ml Listener of Mouse Events, which must be a
	 * 				 <code>MouseListener</code>
	 */
	public TBWidget(IToolContainer _owner, Image _icon, Image _selicon, MouseListener ml) {
		super(_icon, _selicon, ml);
		owner = _owner;
		
	}

	/**
	 * Gets the TBWidget's owner (<code>IToolContainer</code> compliant,
	 * 				 e.g. a <code>ToolBar</code>).
	 * 
	 * @return Owner(parent) of the widget. <code>IToolContainer</code> the 
	 * 		   TBWidget belongs to
	 */
	public IToolContainer getOwner() {
		return owner;
	}

	/**
	 * Set the control's owner
	 * @param owner <code>IToolContainer</code>
	 */
	public void setOwner(IToolContainer owner) {
		this.owner = owner;
	}
	
}