/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import java.awt.Graphics;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileException;
import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;

/**
 * Element of the GUI which represents a container of buttons displayed for
 * allowing the user to access specific actions.
 * <p>A <code>ToolBar</code> contains:</p>
 * <ul>
 * 		<li>A <code>ToolBarGroupButton</code> representing the Tool Bar</li>
 * 		<li>An array of <code>TBButton</code> (the buttons offered in the Tool Bar)</li>
 * 		<li>A <code>ToolBarHelpButton</code> for displaying help about buttons</li>
 * </ul>
 * 
 * Note: This class is not intended to be subclassed
 * 
 * @author jldominguez
 * @author mmontesinos
 *
 */
public class ToolBar implements IToolContainer{
	

	private static Logger logger = Logger.getLogger(ToolBar.class);
	
	private ToolBarGroupButton groupButton;
	private TBButton [] buttons = new TBButton[0];
	private TBHelpButton helpButton = null;
	protected MainWindow mainWindow = null;
	private TBMenu menu = null;
	
	private int[] emptyX = null;
	private int maxButtonCount = 0;


	/*
	 * The maximum number of buttons that can be added to a toolbar.
	 * It should be managed better, for allowing different button
	 * sizes, different toolbar sizes, different configurations of
	 * the screen (horizontal, vertical, ...)
	 * Or even to handle oversizing with scrolling buttons
	 * Partial fix 7/23/08: Screen size is taken into account
	 */
	//public static final int MAX_BUTTON_COUNT = 7;
	// private TBEmptyButton emptyButton = null;
	
	/**
	 * Creates a new instance of a <code>ToolBar</code> given the window
	 * to be included in, a <code>ToolBarGroupButton</code> representing
	 * the new Tool Bar, the index order of this new ToolBar among all
	 * existing ToolBars, and the name of this toolbar
	 * @param _mw MainWindow of the app
	 * @param gButton A <code>ToolBarGroupButton</code> representing the Tool Bar
	 * @param _index Index (order) number of the toolbar
	 */
	public ToolBar(MainWindow _mw,	ToolBarGroupButton gButton,	int _index){
		
		MapControl _mc = _mw.getMapControl();
		mainWindow = _mw;
		groupButton = gButton;
		//index = _index;
		
		//Calculate the max number of buttons that can be allocated in the tool bar 
		maxButtonCount = Math.round((mainWindow.getWidth() - GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_WIDTH - 
				GvsigMobileGUIConstants.HELP_ICON_WIDTH) / GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH);

		//addTool(firstTool);
		
		// --- group button
		gButton.setOwner(this);
		_mc.add(gButton);
		
		// --- empty + help
		// emptyButton = new TBEmptyButton(this, "menutoolbar_space8_black.png", "menutoolbar_space8_black.png", _mw);
		
		helpButton = new TBHelpButton(this, 
					"menutoolbar_dropdown_black.png",
					"menutoolbar_dropup_orange.png", _mw);
		
		// _mc.add(emptyButton);
		_mc.add(helpButton);
	}
	
	/**
	 * Add empty button images. This is called from the MapControl paint method.
	 * 
	 * @param g
	 */
	public void paintEmptyButtons(Graphics g) {
		
		for (int i=0; i<emptyX.length; i++) {
			g.drawImage(TBEmptyButton.EMPTY_BUTTON_IMAGE, emptyX[i], 0, null);
		}
	}
	
	/**
	 * Adds a button in the toolbar. The button must be of type
	 * <code>TBButton</code>
	 * 
	 * @param button <code>TBButton</code> to be added to the ToolBar
	 * @param index number of the position that the new button will have in the
	 * 		  array of buttons that the <code>ToolBar</code> contains.
	 * 		  It must be an <code>int</code> between 0 and the actual size of
	 * 		  the array.
	 * @throws GvsigMobileException
	 */
	public void addButton (TBButton button, int index) throws GvsigMobileException{
	
		int newLength = 0;
		
		//TODO Check the max size of the toolbar to see if the button has place
		if (index < 0){
			throw new GvsigMobileException(GvsigMobileGUIConstants.ERROR_INDEX_UNDER_0);
		} else if (index > maxButtonCount){
			throw new GvsigMobileException(GvsigMobileGUIConstants.ERROR_TB_BUTTON_CAPACITY);
			//TODO Centralize Error texts
		} else if (index > buttons.length){
			throw new GvsigMobileException(GvsigMobileGUIConstants.ERROR_TB_BUTTON_GAPS);
		} 
		
		newLength = buttons.length + 1;
		TBButton [] newButtons = new TBButton [newLength];
		
		if (buttons.length == 0){ //First button to be added
			buttons = newButtons;
		} else{
			if (index > 0){ //Left-side of the array need to be copied
				System.arraycopy (buttons, 0, newButtons, 0, index);
			}
			if (index < newLength - 1){ //Rigth-side of the array need to be copied
				System.arraycopy (buttons, index, newButtons, index + 1, newLength - index - 1);
			}
			buttons = newButtons;
		}
		
		/*
		 * We have a new buttons array with an empty space for the new
		 * TBButton to be created
		 */
		buttons[index] = button;
		mainWindow.getMapControl().add(buttons[index]);
	}
	
	/**
	 * Returns the number of buttons existing in the toolbar.
	 * 
	 * @return the number of buttons
	 */
	public int getButtonCount(){
		return buttons.length;
	}
	
	/**
	 * Returns the array of visible buttons of type <code>TBButton</code> that
	 * the ToolBar contains.
	 * For a <code>TBActionButton</code> or <code>TBSelectableButton</code>
	 * it returns that button.
	 * For a <code>TBComboButton</code> it returns the active button in the list
	 * @return array of buttons of type <code>TBButton</code>
	 */
	public TBButton [] getButtons() {
		TBButton[] returnButtons;
		
		returnButtons = new TBButton[buttons.length];
		
		/*
		 * Let's go through all the buttons to set the button to be returned
		 */
		for (int i = 0; i < buttons.length; i++){
			if ((buttons[i] instanceof TBActionButton)
					|| (buttons[i] instanceof TBSelectableButton)
					|| (buttons[i] instanceof TBSwitcherButton)){
				//Direct assignment
				returnButtons[i] = buttons[i];
			} else if (buttons[i] instanceof TBComboButton){
				//Return only active button
				returnButtons[i] = ((TBComboButton) buttons[i]).getActiveTool();
			} else{
				returnButtons[i] = null;
			}
		}
		return returnButtons;
	}
	
	/**
	 * Returns the array of all buttons of type <code>TBButton</code> that
	 * the ToolBar contains, including the ones in a combo list
	 * For a <code>TBActionButton</code> or <code>TBSelectableButton</code>
	 * it returns that button.
	 * For a <code>TBComboButton</code> it returns the active button in the list
	 * @return array of buttons of type <code>TBButton</code>
	 */
	public TBButton [] getAllButtons() {
		TBButton[] returnButtons;
		TBButton[] tmpButtons;
		TBComboButton combo;
		int numButtons;
		int extraButtons = 0;
		
		returnButtons = new TBButton[buttons.length];
		
		/*
		 * Let's go through all the buttons to set the button to be returned
		 */
		for (int i = 0; i < buttons.length; i++){
			if ((buttons[i] instanceof TBActionButton) || (buttons[i] instanceof TBSelectableButton)){
				//Direct assignment
				returnButtons[i + extraButtons] = buttons[i];
			} else if (buttons[i] instanceof TBComboButton){
				combo = ((TBComboButton) buttons[i]);
				//Let's redimension the final array
				numButtons = combo.getButtonCount();
				tmpButtons = new TBButton[buttons.length + numButtons - 1];
				if (i >0){
					System.arraycopy(returnButtons, 0, tmpButtons, 0, i);
				}
				System.arraycopy(combo.getButtons(), 0, tmpButtons, i, numButtons);
				returnButtons = tmpButtons;
				extraButtons += numButtons - 1;
				//Return all the buttons in the combo list
				//returnButtons[i] = ((TBComboButton) buttons[i]).getActiveTool();
			} else{
				returnButtons[i + extraButtons] = null;
			}
		}
		return returnButtons;
	}
	/**
	 * Calculates the X position of a <code>TBButton</code> in the screen,
	 * given the order (number) of the button for which we want to know the
	 * X position and the size of the Window
	 * @param position Order number of the button in the <code>buttons</code> 
	 * 		  array of the ToolBar (display order from left to right)
	 * @param screenWidth Screen width in pixels of the <code>MapControl</code>
	 * @return X position of the left pixel of the button
	 */
	public int getToolXOrigin(int position, int screenWidth) {
		
		int x = 0;
		int sought = position;
		
		if (position >= buttons.length){
			sought = buttons.length;
		}
		for (int i = 0; i < sought; i++){
			x += buttons[i].getWidth();
		}
		return GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_WIDTH + x;
	}
	
	/**
	 * Calculates the Y position of a <code>TBButton</code> in the screen,
	 * given the order (number) of the button for which we want to know the
	 * Y position and the size of the Window
	 * @param position Order number of the button in the <code>buttons</code> 
	 * 		  array of the ToolBar (display order from left to right)
	 * @param screenWidth Screen width in pixels of the Window
	 * @return Y position of the top left pixel of the button
	 */
	public static int getToolYOrigin(int position, int screenWidth) {
		// int n_per_row = Math.max(1, screenWidth / GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH);
		// int y = 0;
		// y = position / n_per_row;
		return 0; // y * GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT;
	}
	
	/**
	 * Gets the group button associated to the toolbar
	 * @return the <code>ToolBarGroupButton</code> of the toolbar
	 */
	public ToolBarGroupButton getGroupButton() {
		return groupButton;
	}
	
	/**
	 * Reconfigures the toolbar after adding new buttons.
	 * <p>It should be called after all the buttons in a <code>ToolBar</code>
	 * have been added
	 * @param width of the screen
	 */
	public void reorder(int width) {
		
		if (buttons.length == 0) return;
		
		for (int i = 0; i < buttons.length; i++) {
			
			int org_x = getToolXOrigin(i, width);
			int org_y = getToolYOrigin(i, width);
			
			/*
			 * Set position and size (bounds) of every button in the toolbar
			 * getWidth is used in order to allow different widths
			 * (e.g. combo buttons)
			 */
			buttons[i].setBounds(
					org_x, org_y,
					buttons[i].getWidth(),
					buttons[i].getHeight());
		}
		
		TBButton last_b = buttons[buttons.length - 1];
		int last_x_available = last_b.getX() + last_b.getWidth();
		
		if ((last_x_available + GvsigMobileGUIConstants.HELP_ICON_WIDTH) > width) {
			logger.warn("Too many buttons and/or combo buttons: ");
			logger.warn("--> last_x_available + GvsigMobileGUIConstants.HELP_ICON_WIDTH = " + (last_x_available + GvsigMobileGUIConstants.HELP_ICON_WIDTH));
			logger.warn("--> width = " + width);
		}
		
		int empty_count = Math.max(
				0,
				(width - (last_x_available + GvsigMobileGUIConstants.HELP_ICON_WIDTH)) / GvsigMobileGUIConstants.HELP_ICON_WIDTH);
		
		
		emptyX = new int[empty_count];
		for (int i=0; i<empty_count; i++) {
			emptyX[i] = last_x_available + i * GvsigMobileGUIConstants.HELP_ICON_WIDTH;
		}
		// logger.debug("Setting help location: " + width+" - "+GvsigMobileGUIConstants.HELP_ICON_WIDTH+", 0");
		int posx = width - GvsigMobileGUIConstants.HELP_ICON_WIDTH;
		helpButton.setLocation(posx, 0);
		//helpButton.setLocation(1242, 0);
		//helpButton.setLocation(1242, 0);
		//helpButton.setLocation(300, 300);

		// int n_per_row = Math.max(1, (width-ToolBarGroupButton.ICON_WIDTH) / GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH);
		/*
		emptyButton.setLocation(
				ToolBarGroupButton.ICON_WIDTH + n_per_row * GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH, 0);
				*/
	}

	/**
	 * Gets the empty button of the toolbar
	 * @return <code>TBEmptyButton</code> of the <code>ToolBar</code>
	 */
	/*
	public TBEmptyButton getEmptyButton() {
		return emptyButton;
	}
	*/

	/**
	 * Gets the help button of the toolbar
	 * @return <code>TBHelpButton</code> of the <code>ToolBar</code>
	 */
	public TBHelpButton getHelpButton() {
		return helpButton;
	}

	/**
	 * Sets all buttons in the <code>ToolBar</code> to Visible or Invisible
	 *
	 * @param visibility Visibility end state:<br/>
	 * 					 <code>true</code>: Set all buttons to visible<br/>
	 * 					 <code>false</code>: Set all buttons to invisible
	 */
	public void setButtonsVisible(boolean visibility){
		// logger.debug("Toolbar: Setting all widgets to " + visibility);
		//First the GroupButton
		this.groupButton.setVisible(visibility);
		//Next the ToolBarButtons
		for (int i = 0; i < buttons.length; i++) {
			if (buttons[i].getParent() == null) {
				mainWindow.getMapControl().add(buttons[i]);
			}
			buttons[i].setVisible(visibility);
		}
		//And finally the help button
		this.helpButton.setVisible(visibility);
		// logger.debug("Toolbar.HelpButton: " + this.helpButton.isVisible());
		
	}
	
	/**
	 * Unselects all buttons in the <code>ToolBar</code>
	 * It doesn't change other widgets but <code>TBButton</code>'s
	 *
	 */
	public void unselectAllButtons(){
		// logger.debug("Toolbar: Unselecting all buttons");
		for (int i = 0; i < buttons.length; i++) {
			buttons[i].setSelected(false);
		}
	}

	/**
	 * Returns the MainWindow of the toolbar
	 * @return MainWindow where the toolbar has been placed
	 */
	public MainWindow getWindow() {
		
		return mainWindow;
	}

	/**
	 * Stub method. Not used
	 */
	public void setActiveTool(int index) {
		// Not used in this class	
	}

	/**
	 * Stub method. Not used
	 */
	public void repaint() {
		// Not used in this class	
	}
	
	/**
	 * Create a Drop-down menu associated to the ToolBar with
	 * the buttons visible in the <code>ToolBar</code> as the items list
	 *
	 */
	public void createMenu(){
		menu = new TBMenu(this, helpButton.getX(), helpButton.getHeight());
		mainWindow.dropMenu = menu;
		
	}

	/**
	 * @return Returns the menu.
	 */
	public TBMenu getMenu() {
		return menu;
	}
	
	/**
	 * Removes the menu displayed associated to the help button of the tool bar
	 *
	 */
	public void removeMenu(){
		ToolBar myToolBar;
		
		menu.close();
		menu = null;
		//Let's remove all buttons from the toolbar and re-add them
		//this.removeAll();
		reorder(mainWindow.getMapControl().getWidth());
		//for (int i = 0; i < buttons.length; i++){
		//	this.add(buttons[i]);
		//	buttons[i].setVisible(false);
		//}
		//Set selected button properly
		//buttons[index].setVisible(true);
		//buttons[index].setBounds(0, 0, GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH, GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT);
		
		//Re-add drop-down button
		//this.add(dropDown);
	}
	
	public boolean isSelected() {
		return groupButton.isSelected();
	}

	/**
	 * @return the maxButtonCount
	 */
	public int getMaxButtonCount() {
		return maxButtonCount;
	}



}
