/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;

/**
 * This class represents one of the buttons that show on the left side of the main window.
 * When the user clicks on one of them, a new tool bar shows at the top.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.ToolBar
 * 
 * @author jldominguez
 *
 */
public class ToolBarGroupButton extends Component {
	
	protected Image icon, selIcon;
	private boolean selected = false;
	private ToolBar owner = null;
	private int index = 0;

	/**
	 * Constructor.
	 * 
	 * @param _icon the file path to the icon image file (unselected)
	 * @param _selicon the file path to the icon image file  (selected)
	 * @param _mw the parent main window
	 * @param _i the 0-based index of this icon
	 */
	public ToolBarGroupButton(String _icon, String _selicon, MainWindow _mw, int _i) {

		index = _i;
		
		icon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, _icon);
		selIcon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, _selicon);
		addMouseListener(_mw);
		setBounds(0, index * GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_HEIGHT, GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_WIDTH, GvsigMobileGUIConstants.DEFAULT_TB_GROUP_BUTTON_HEIGHT);
	}
	
	public int getIndex() {
		return index;
	}
	
	public void setSelected(boolean b) {
		selected = b;
	}
	
	public boolean isSelected() {
		return selected;
	}
	
	/**
	 * This method will paint the icon (selected or unselected)
	 * 
	 * @param g the object on which we have to paint
	 */
	public void paint(Graphics g) {
		
		if (selected) {
			if (selIcon != null) {
				g.drawImage(selIcon, 0, 0, null);
			}
		} else {
			if (icon != null) {
				g.drawImage(icon, 0, 0, null);
			}
		}
	}
	
	/**
	 * @return the owner of this button (the tool bar)
	 */
	public ToolBar getOwner() {
		return owner;
	}

	/**
	 * Sets the owner of this button (a tool bar)
	 * @param tb the owner of this button
	 */
	public void setOwner(ToolBar tb) {
		owner = tb;
	}
}
