/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool.behavior;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;
import es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.PointToolListener;
import es.prodevelop.gvsig.mobile.gui.tool.util.PanRefreshTimerTask;

/**
 * This behavior class notifies point events only,
 * si it's useful for point-oriented
 * tools (centter on point, info, etc.)
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.PointToolListener
 * 
 * @author jldominguez
 *
 */
public class PointBehavior extends Behavior {
	
	private static Logger logger = Logger.getLogger(PointBehavior.class);
	private PointToolListener listener = null;
	
	/**
	 * Gets the current listener, which will be notified when point events occur.
	 * @return
	 */
	public PointToolListener getListener() {
		return listener;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param l the listener, which will be notified when point events occur.
	 */
	public PointBehavior(PointToolListener l) {
		listener = l;
	}

	/**
	 * Overriden method, in case the listener needs to set another cursor while
	 * its tool is active.
	 * 
	 * @return the cursor to be used when the associated tool is active.
	 */
	public Cursor getCursor() {
		return listener.getCursor();
	}

	/**
	 * This behavior class creates a relevant event for point events and
	 * notifies the current listener.
	 * 
	 * @param e the mouse event generated by the user on the map
	 */
	public void mousePressed(MouseEvent e) {
		
		logger.debug("GUI event: mouse pressed...");
		logger.debug("GUI event: mouse pressed...e.getX() = " + e.getX());
		logger.debug("GUI event: mouse pressed...e.getY() = " + e.getY());

		PointEvent pe = new PointEvent(new Point(e.getX(), e.getY()), e);

		listener.point(pe);
	}
	
	/**
	 * This behavior class creates a relevant event for point events and
	 * notifies the current listener.
	 * 
	 * @param e the mouse event generated by the user on the map
	 */
	public void mouseReleased(MouseEvent e) {
		
		logger.debug("GUI event: mouse released...");
		
		PointEvent pe = new PointEvent(new Point(e.getX(), e.getY()), e);

		listener.point(pe);
	}
	
	/**
	 * This behavior class creates a relevant event for point events and
	 * notifies the current listener.
	 * 
	 * @param e the mouse event generated by the user on the map
	 */
	public void mouseDragged(MouseEvent e) {
		
		logger.debug("GUI event: mouse dragged...");
		logger.debug("GUI event: mouse dragged... e = " + e);
		logger.debug("GUI event: mouse dragged... e.getX() = " + e.getX());
		logger.debug("GUI event: mouse dragged... e.getY() = " + e.getY());

		PointEvent pe = new PointEvent(new Point(e.getX(), e.getY()), e);
			
		listener.point(pe);
	}

}
