/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool.listener;

import java.awt.Cursor;
import java.awt.event.MouseEvent;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;
import es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.PointToolListener;

/**
 * This listener is associated with the center-on-point tool. It will simply tell the
 * <code>MapControl</code> to recenter on the point.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.behavior.PointBehavior
 * 
 * @author jldominguez
 *
 */
public class CenterOnPointListener implements PointToolListener {
	
	private static Logger logger = Logger.getLogger(CenterOnPointListener.class);
	private MapControl mapCtrl;

	/**
	 * This method will be called when the user clicks on the user with this tool
	 * being the active tool.
	 * 
	 * @param event the point event that indicates a recenter request
	 */
	public void point(PointEvent event) {
		
		if (event.getEvent().getID() != MouseEvent.MOUSE_PRESSED) {
			return;
		}
		
		mapCtrl.recenter(event.getPoint().x,event.getPoint().y);
		
	}

	/**
	 * Gets the cursor that should be used while this tool is active.
	 * 
	 * @return the cursor that should be used while this tool is active.
	 */
	public Cursor getCursor() {
		return Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
	}

	/**
	 * This method asks the listener to cancel the current drawing process
	 * (currently unsued)
	 * 
	 * @return whether it was possible to cancel the drawing
	 */
	public boolean cancelDrawing() {
		return false;
	}

	/**
	 * Sets the <code>MapControl</code> object that will be updated by
	 * this listener.
	 */
	public void setMapControl(MapControl mc) {
		mapCtrl = mc;
	}


}
