/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool.listener;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

import org.apache.log4j.Logger;

import es.prodevelop.geodetic.utils.conversion.GeoUtils;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.gui.bar.StatusBar;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;
import es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.PointToolListener;

/**
 * This listener is associated with the coordinates tool. It will simply
 * write the pixel map coordinates on the status bar.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.behavior.PointBehavior
 * 
 * @author jldominguez
 *
 */
public class PointCoordinatesListener implements PointToolListener {
	
	private static Logger logger = Logger.getLogger(PointCoordinatesListener.class);
	private MapControl mapCtrl;

	/**
	 * This method will be called when the user clicks on the map and the coordinates tool
	 * is active.
	 * 
	 * It will simply write the pixel map coordinates on the status bar using the
	 * current extent and map size to compute the result.
	 * 
	 * @param event the point event that indicates the point of interest
	 */
	public void point(PointEvent event) {
		int id = event.getEvent().getID();
		
		if (id == MouseEvent.MOUSE_PRESSED) {
			
			Point2D mp = mapCtrl.getViewPort().toMapPoint(
					event.getPoint().x, event.getPoint().y);

			String str = getCoordinatesString(
					mp.getX(), mp.getY(),
					mapCtrl.project.getProjection());
			
			mapCtrl.setStatusBarText(StatusBar.MEASURES_BAR, str);
		}
	}
	
	private String getCoordinatesString(double x, double y, IProjection p) {
		
		String units = p.getUnitsAbbrev();
		int decimals = 2;

		if (GeoUtils.isLatLonProjection(p.getAbrev())) {
			decimals = 7;
		}
		
		return Utils.getFormattedDouble(x, decimals, false)
		+ " " + units + ", "
		+ Utils.getFormattedDouble(y, decimals, false)
		+ " " + units;
		
	}

	/**
	 * Gets the cursor that should be used while this tool is active.
	 * 
	 * @return the cursor that should be used while this tool is active.
	 */
	public Cursor getCursor() {
		return null;
	}

	/**
	 * This method asks the listener to cancel the current drawing process
	 * (currently unsued)
	 * 
	 * @return whether it was possible to cancel the drawing
	 */
	public boolean cancelDrawing() {
		return false;
	}

	/**
	 * Sets the <code>MapControl</code> object that will be updated by
	 * this listener.
	 */
	public void setMapControl(MapControl mc) {
		mapCtrl = mc;
	}


}
