/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces;

import java.awt.Cursor;

import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;

/**
 * Tool listener are the ones which perform the operations on the map (MapCOntrol).
 * Behavior classes gather information from the java event system and
 * notify the listeners when something relevant happens. Whether an event is relevant
 * or not depends also on the type of tool.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.behavior.Behavior
 * 
 * @author jldominguez
 *
 */
public interface ToolListener {
	
	/**
	 * Allows the tool listener to set a new cursor while the associated tool
	 * is active.
	 * 
	 * @return the cursor icon to be used when this tool is active.
	 */
	public Cursor getCursor();
	
	/**
	 * Allows the tool listener to cancel the current drawing process. Currently unsued.
	 * 
	 * @return this method will return <code>true</code> when the drawing has been
	 * cancelled (cancel request accepted)
	 * 
	 */
	public boolean cancelDrawing();
	
	/**
	 * Sets the <code>MapControl</code> object. Listeners will have to do some kind
	 * of request to the map, depending on the tool.
	 * 
	 * @param mc the <code>MapControl</code> object to be used by this listener.
	 */
	public void setMapControl(MapControl mc);
	
}