/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool.maplistener;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.tool.TBButton;
import es.prodevelop.gvsig.mobile.gui.tool.TBSelectableButton;
import es.prodevelop.gvsig.mobile.gui.tool.behavior.Behavior;
import es.prodevelop.gvsig.mobile.gui.tool.listener.InfoPointListener;

/**
 * This is the mouse listener used by the <code>MapControl</code>
 * to perform actions depending on the tool that is active in each moment.
 * 
 * It keeps a changing reference to the current behavior which in turn is linked
 * to the current tool listener.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.ToolListener
 * @see es.prodevelop.gvsig.mobile.gui.tool.behavior.Behavior
 * @see es.prodevelop.gvsig.mobile.gui.canvas.MapControl
 * @see es.prodevelop.gvsig.mobile.gui.tool.TBButton
 *  
 * @author jldominguez
 *
 */
public class MapToolListener implements MouseListener, MouseMotionListener {
	
	private static Logger logger = Logger.getLogger(InfoPointListener.class);
	
	private Behavior toolBehavior = null;
	private TBButton currentTool = null;
	private MapControl mc = null;
	
	/**
	 * Constructor.
	 * 
	 * @param _mc the <code>MapControl</code> object associated with this listener.
	 */
	public MapToolListener(MapControl _mc) {
		mc = _mc;
	}
	
	/**
	 * Returns the active tool.
	 * <p>Only tools of type <code>TBSelectableTool</code> (subclass of 
	 * <code>TBButton</code>) registers listeners, so only this
	 * objects will be real active tools 
	 * 
	 * @return the current tool (must be of type <code>TBButton</code>)
	 */
	public TBButton getCurrentTool(){
		return currentTool;
	}
	
	/**
	 * Sets the current tool. 
	 * 
	 * @param tb the new tool to be set as current tool
	 * @param clearStatusBar whether the status bar has to be cleared when this tool
	 * becomes the active tool.
	 */
	public void setTool(TBButton tb, boolean clearStatusBar) {
		
		currentTool = tb;
		//Remove all Non Permanent Bars
		mc.getParentFrame().statusBar.removeNonPermanentBars();
		
		if (tb == null) {
			toolBehavior = null;
			return;
		} else {
			if (tb instanceof TBSelectableButton){
				toolBehavior = ((TBSelectableButton) tb).getBehavior();
			}
			
			if (tb.getStatusBarId() != -1) {
				mc.getParentFrame().statusBar.addBar(tb.getStatusBarId());
			}
		}
	}

	/**
	 * All mouse events are delivered to the behavior, who will process it
	 * depending on the functionality of the current tool.
	 * 
	 *  @param arg0 the mouose event that has to be managed by the behavir current
	 */
	public void mouseClicked(MouseEvent arg0) {
		
		if (toolBehavior == null) return;
		toolBehavior.mouseClicked(arg0);
	}

	/**
	 * All mouse events are delivered to the behavior, who will process it
	 * depending on the functionality of the current tool.
	 * 
	 *  @param arg0 the mouose event that has to be managed by the behavir current
	 */
	public void mousePressed(MouseEvent arg0) {
		
		if (toolBehavior == null) return;
		toolBehavior.mousePressed(arg0);
	}

	/**
	 * All mouse events are delivered to the behavior, who will process it
	 * depending on the functionality of the current tool.
	 * 
	 *  @param arg0 the mouose event that has to be managed by the behavir current
	 */
	public void mouseReleased(MouseEvent arg0) {
		
		if (toolBehavior == null) return;
		toolBehavior.mouseReleased(arg0);
	}

	public void mouseEntered(MouseEvent arg0) {
	}

	public void mouseExited(MouseEvent arg0) {
	}

	/**
	 * All mouse events are delivered to the behavior, who will process it
	 * depending on the functionality of the current tool.
	 * 
	 *  @param arg0 the mouose event that has to be managed by the behavir current
	 */
	public void mouseDragged(MouseEvent arg0) {
		
		if (toolBehavior == null) return;
		toolBehavior.mouseDragged(arg0);
	}

	/**
	 * All mouse events are delivered to the behavior, who will process it
	 * depending on the functionality of the current tool.
	 * 
	 *  @param arg0 the mouose event that has to be managed by the behavir current
	 */
	public void mouseMoved(MouseEvent arg0) {
		
		if (toolBehavior == null) return;
		toolBehavior.mouseMoved(arg0);
	}

}
