/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */


package org.gvsig.forms.view;
import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Types;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.HashMap;

import org.apache.log4j.Logger;
import org.gvsig.forms.model.FeatureTemp;

import com.hardcode.gdbms.engine.values.DateValue;
import com.hardcode.gdbms.engine.values.DoubleValue;
import com.hardcode.gdbms.engine.values.FloatValue;
import com.hardcode.gdbms.engine.values.IntValue;
import com.hardcode.gdbms.engine.values.LongValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.string.StringUtilities;

/**
 * This form is created automatically specifically for a feature
 * Using the information of the feature attribute types and the
 * thinlet template used for generic forms and custom forms
 *  
 * @author jcarras
 *
 */
public class GenericForm extends Dialog implements FeatureForm, WindowListener, ComponentListener{
	
	private static Logger logger = Logger.getLogger(GenericForm.class);
	private transient Image doublebuffer;
	private String template = ResourceReader.getResourceFile("forms","template.xml").getAbsolutePath();
	private boolean opened = false;
	protected FeatureTemp originalFeature;
	protected Frame parent_frame;
	
	protected ThinletForm frm = new ThinletForm(this); 
	
	
	
	private void setSize(int x, int y, int w, int h){
		logger.debug("Setting sizr: " + x + ", "+ y + ", "+ w + ", "+ h);
		pack();
		setBounds(x,y,w,h);
		//26 pixels is the size from the accept/cancel buttons 
		//frm.setBounds(x,y-26,w,h);
		Insets i = this.getInsets();
		frm.setBounds(i.left,i.top,w-i.left-i.right,h-i.top-i.bottom);
		
		//frm.setBounds(0,0,w,h);
		//frm.doLayout();
	}
	
	private void setSize(){
		setSize(
                parent_frame.getX(),
                parent_frame.getY(),
                parent_frame.getWidth(),
                parent_frame.getHeight());

//		int width=240;
//		int height=280;
//		Insets is = getInsets();
//		width += is.left + is.right;
//		height += is.top + is.bottom;
//		Dimension ss = getToolkit().getScreenSize();
//		width = Math.min(width, ss.width);
//		height = Math.min(height, ss.height);
//		setBounds((ss.width - width) / 2, (ss.height - height) / 2, width, height); 
		//frm.setBounds((ss.width - width) / 2, (ss.height - height) / 2, width, height);
		
	}
	
	public GenericForm(Frame parent){
		super(parent,true);
		parent_frame=parent;
		setLayout( null );
		if (Utils.USING_PDA) {
			setUndecorated(true);
		} 
		setSize();
	}
	
	
	
	protected void addCustomFormLines(StringBuffer buff){
		if (originalFeature==null) return;
		buff.append("\n<panel weightx=\"1\" columns=\"2\" gap=\"2\">\n");
		Value[] vals = originalFeature.getValues();
		String[] names = originalFeature.getFieldNames();
		for (int i=0; i<names.length; i++){ 
			//TODO: Adapt every field appearance to the value type 
			buff.append("<label text=\"" + names[i].toString() + "\"/>");
			buff.append("<textfield weightx=\"1\" property=\"featureAttribute=" + names[i].toString() + "\" />\n");
		}
		buff.append("</panel>\n");
		//System.out.print(buff);
	}
	
	protected InputStream getMixedTemplate(){
		StringBuffer result = new StringBuffer();
		try{
			BufferedReader tin = new BufferedReader(
					new FileReader(template)
					);
			String line;
			while ( (line=tin.readLine()) != null ) {
				if (line.trim().equals("<label name=\"content\" text=\"Content\"/>")){
					addCustomFormLines(result);
				} else {
					result.append(line);
				}
            }
			
			
			
		} catch (FileNotFoundException e){
			e.printStackTrace();
		} catch (IOException e1){
			e1.printStackTrace();
		}
		
		//TODO: please improve me
		byte[] bArray = result.toString().getBytes();
		ByteArrayInputStream bais = new ByteArrayInputStream(bArray);
		
		return bais;
	}

	public void cancelEdition() {
		//Sets the original values in every field and sets fields to not enabled
		// TODO Auto-generated method stub
	}

	public void close() {
		//Terminates the form
		this.opened=false;
		this.frm.destroy();
		dispose();
	}



	public void hide() {
		//Hides the form to the user
		//setVisible(false);
		this.opened=false;
		super.hide();
	}

	public boolean isCanceled() {
		//True if user closed the form without having clicked save
		return frm.cancelled;
	}

	public boolean isModified() {
		//True if user has modified any value
		return frm.modified;
	}

	/**
	 * @param component
	 * @param value
	 * @param type
	 */
	protected void fillField(Object component, Value value, int type){
		Object valType = frm.getProperty(component, "valueType");
		Object defaultValue = frm.getProperty(component, "defaultValue");
		String fieldType = frm.getClass(component);
		////System.out.println(value.getClass().getName() + " " + type);
		
		if (value==null || value.toString()==null){
			if (!"checkbox".equals(fieldType))
				frm.setString(component, "text", "");
			return;
		}
		
		if ("checkbox".equals(fieldType)){
			Object chkValue = frm.getProperty(component, "chkValue");
			
			if (value!=null && chkValue!=null && chkValue.toString().trim().equals(value.toString().trim())){
				frm.setBoolean(component,"selected",true);
				return;
			}
			frm.setBoolean(component,"selected",false);
			return;
		}
		
		if ("combobox".equals(fieldType)){
			Object[] options = frm.getItems(component);
			for (int i =0; i<options.length;i++){
				if (value.toString().trim().equals(frm.getString(options[i], "name").trim())){
					frm.setInteger(component,"selected",i);
					frm.setString(component, "text", frm.getString(options[i], "text"));
					return;
				}
			}
			frm.setInteger(component,"selected",-1);
			frm.setString(component, "text", "");
			return;
		}
		
		//If the field is telling us the format we rely on it
		Object valueType = frm.getProperty(component, "valueType");
		if ("INTEGER".equals(valueType)){
			try{
				int val = (int)(Double.parseDouble(value.toString()));
				frm.setString(component, "text", "" + val);
				return;
			} catch (Exception e){ e.printStackTrace(); }
		}
		if ("NUMERIC".equals(valueType)){
			try{
				double val = Double.parseDouble(value.toString());
				frm.setString(component, "text", "" + val);
				return;
			} catch (Exception e){ e.printStackTrace(); }
		}
		if ("DATE".equals(valueType)){
			try{
				Date val = new Date(value.toString());
				frm.setString(component, "text", "" + val);
				return;
			} catch (Exception e){ e.printStackTrace(); }
		}
//		if ("TIME".equals(valueType)){
//			try{
//				String date_time= "00/00/0000" + (String)frm.getString(component, "text");
//				Date value = new Date(Date.parse(date_time));
//				return ValueFactory.createValue(value);
//			} catch (Exception e){ e.printStackTrace(); }
//		}
		
		//If the field is not telling us the format,
		// we rely on the feature attribute type
		if (value instanceof IntValue || value instanceof LongValue)
			try{
				int val = (int)(Double.parseDouble(value.toString()));
				frm.setString(component, "text", "" + val);
				return;
			} catch (Exception e){ e.printStackTrace(); }
			
		if (value instanceof DoubleValue || value instanceof FloatValue ){
			try{
				double val = Double.parseDouble(value.toString());
					DecimalFormat frmt = new DecimalFormat("#########.##");
					frm.setString(component, "text", frmt.format(val));			
				return;
			} catch (Exception e){ e.printStackTrace(); }
		}
		
		if (value instanceof DateValue){
			try{
				Date val = new Date(value.toString());
				frm.setString(component, "text", "" + val);
				return;
			} catch (Exception e){ e.printStackTrace(); }
		}
		
		if (value!=null)
			frm.setString(component, "text", value.toString().trim());
		else 
			frm.setString(component, "text", String.valueOf(value));
	}
	protected void fillFields(){
		if (originalFeature==null)return;
		//Do the binding
		HashMap comps = getBindedComponents();
		String[] names = originalFeature.getFieldNames();
		Value[] vals = originalFeature.getValues();
		int[] types = originalFeature.getFieldTypes();
		for (int i=0;i<names.length;i++){
			if (comps.containsKey(names[i])){
				Object comp = comps.get(names[i]);
				fillField(comp, vals[i], types[i]);
				frm.setBoolean(comp, "enabled", false);
			}
		}
	}
	
	private void debugThinlet(Object component, int nTabs){
		for (int i=0;i<frm.getCount(component);i++){
			for (int t=0;t<nTabs;t++)
				System.out.print("\t");
			Object c2 = frm.getItem(component, i);
			System.out.println(frm.getClass(c2));
			debugThinlet(c2, nTabs+1);
		}
	}
	public void setFeature(FeatureTemp feature) {
		//Fills all the binded components with the values in the feature
		originalFeature = feature;
		try{
			
			frm.add(frm.parse(getMixedTemplate()));
			//debugThinlet(frm.getDesktop(),0);
			add(frm, BorderLayout.CENTER);
			//pack();
			frm.requestFocus();
			addWindowListener(this);
			parent_frame.addComponentListener(this);
			//System.out.println("ADDED. there are " + this.getComponentCount() + " components");
			frm.setIcon(
					frm.find("btnsave"),
					"icon",
					ResourceReader.getResourceImage(
							ResourceReader.IMG_SKIN_PATH, 
							"ok_2_new.png"));
			frm.setIcon(
					frm.find("btncancel"),
					"icon",
					ResourceReader.getResourceImage(
							ResourceReader.IMG_SKIN_PATH, 
							"cancel_2_new.png"));
			//pack();

		} catch (IOException e){
			e.printStackTrace();
		}
		fillFields();
	}
	
	
	private Value createValue(String val, int type){
		try{
			if (val==null || "".equals(val)) 
				return ValueFactory.createNullValue();
			
			if ((type == Types.INTEGER) || (type == Types.SMALLINT)) {
				int value = 0;
				try{
					value = Integer.parseInt(val);
				} catch (Exception e) {
					logger.debug("Ignored bad input string: " + val);
				}
				return ValueFactory.createValue(value);
			}
			
			if ((type == Types.DOUBLE) || (type == Types.FLOAT)) {
				double value = 0;
				try{
					value = Double.parseDouble(StringUtilities.replace(val, ",", "."));
				} catch (Exception e){ e.printStackTrace(); }
				return ValueFactory.createValue(value);
			}
			
			if ((type == Types.BIGINT) || (type == Types.TIMESTAMP)) {
				long value = 0;
				try{
					value = Long.parseLong(val);
				} catch (Exception e){ e.printStackTrace(); }
				return ValueFactory.createValue(value);
			}
			
			if (type == Types.DATE){
				Date value = new Date();
				try{
					value = Utils.dateFromString(val);
				} catch (Exception e){ e.printStackTrace(); }
				return ValueFactory.createValue(value);
			}
	
			return ValueFactory.createValue(val);
		} catch (Exception e){
			e.printStackTrace();
			return ValueFactory.createNullValue();
		}
	}
	
	public Value getValue(Object component, int type){
		
		String fieldType = frm.getClass(component);
		String value = (String)frm.getString(component, "text");
		
		if ("checkbox".equals(fieldType)){
			Object chkValue = frm.getProperty(component, "chkValue");
			Object nonChkValue = frm.getProperty(component, "nonChkValue");
			
			if (frm.getBoolean(component,"selected")){
				value = (String)chkValue;
			}else{
				value = (String)nonChkValue;

			}
		}
		
		if ("combobox".equals(fieldType)){
			//TODO: support text writting on combos
			Object option = frm.getSelectedItem(component);
			if (option==null){
				value = null;
			} else{
				value = (String)frm.getString(option, "name");
			}
		}
		
	
		return createValue(value, type);
		
//		Object valueType = frm.getProperty(component, "valueType");
//		if (valueType!=null){
//			if ("INTEGER".equals(valueType)){
//				try{
//					int value = Integer.parseInt((String)frm.getString(component, "text"));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//	
//			if ("NUMERIC".equals(valueType)){
//				try{
//					double value = Double.parseDouble((String)frm.getString(component, "text"));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//	
//			if ("DATE".equals(valueType)){
//				try{
//					Date value = new Date(Date.parse((String)frm.getString(component, "text")));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//	
//			if ("TIME".equals(valueType)){
//				try{
//					String date_time= "00/00/0000" + (String)frm.getString(component, "text");
//					Date value = new Date(Date.parse(date_time));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//		} else {
//
//			if (type == Types.INTEGER){
//				try{
//					int value = Integer.parseInt((String)frm.getString(component, "text"));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//			
//			if (type == Types.SMALLINT ){
//				try{
//					byte value = ((String)frm.getString(component, "text")).getBytes()[0];
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//			
//			if (type == Types.DOUBLE){
//				try{
//					double value = Double.parseDouble((String)frm.getString(component, "text").replace(',', '.'));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//			if (type == Types.FLOAT){
//				try{
//					float value = Float.parseFloat((String)frm.getString(component, "text").replace(',', '.'));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//			if (type == Types.BIGINT){
//				try{
//					long value = Long.parseLong((String)frm.getString(component, "text"));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//			
//			if (type == Types.SMALLINT){
//				try{
//					short value = Short.parseShort((String)frm.getString(component, "text"));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//	
//			if (type == Types.TIMESTAMP){
//				try{
//					double value = Double.parseDouble((String)frm.getString(component, "text"));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//	
//			if (type == Types.DATE){
//				try{
//					Date value = new Date(Date.parse((String)frm.getString(component, "text")));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//	
//			if (type == Types.TIME){
//				try{
//					String date_time= "00/00/0000" + (String)frm.getString(component, "text");
//					Date value = new Date(Date.parse(date_time));
//					return ValueFactory.createValue(value);
//				} catch (Exception e){ e.printStackTrace(); }
//			}
//		}
//		String value = (String)frm.getString(component, "text");
//		return ValueFactory.createValue(value);
	}
	
	public FeatureTemp getFeature() {
		//Returns the feature with the new values updated by the user
		FeatureTemp fea = (FeatureTemp)originalFeature.clone();
		HashMap comps = getBindedComponents();
		String[] names = fea.getFieldNames();
		int[] types = fea.getFieldTypes();
		for (int i=0;i<names.length;i++){
			if (comps.containsKey(names[i])){
				try{
					fea.getValues()[i]=getValue(comps.get(names[i]),types[i]);
				} catch (Exception e){e.printStackTrace();}
			}
		}
		return fea;
	}

	public void setVisible(boolean value){
		//super.setVisible(true);
		
		//System.out.println("SETTING VISIBLE");

		this.opened=value;

		super.setVisible(value);
	}

	public void startEdition() {
		//Puts the fields with modifiable=true to enabled
			if (originalFeature==null)return;
			frm.setBoolean(frm.find("btnsave"), "enabled", true);
			HashMap comps = getBindedComponents();
			String[] names = originalFeature.getFieldNames();
			for (int i=0;i<names.length;i++){
				if (comps.containsKey(names[i])){
					Object comp = comps.get(names[i]);
					String mod = (String)this.frm.getProperty(comp,"modifiable");
					if (mod==null || "TRUE".equalsIgnoreCase(mod)){
						frm.setBoolean(comp, "enabled", true);
					}
				}
			}
		
	}

	public boolean isValid() {
		//Checks the restrictions of fields and alerts user if there are incorrections
		// TODO Auto-generated method stub
		return true;
	}
	public boolean isOpen() {
		return opened;
	}	
	

	
	public HashMap getBindedComponents(){
		HashMap result = new HashMap();
		getBindedComponents(result, frm.getDesktop());
		return result;
	}
	
	private void getBindedComponents(HashMap result, Object component){
		Object[] comps = this.frm.getItems(component);
		for (int i=0;i<comps.length;i++){
			Object comp = comps[i];
			Object att = frm.getProperty(comp, "featureAttribute");
			if (att!=null){
				result.put(att,comp);
			}
			getBindedComponents(result,comp);
		}
	}
	
	public void windowClosing(WindowEvent e) {
		opened=false;
		setVisible(false);
		
	}
	public void windowOpened(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	
	/**
	 * Call the paint method to redraw this component without painting a
	 * background rectangle
	 */
	public void update(Graphics g) {
		paint(g);
	}

	/**
	 * Create a double buffer if needed,
	 * the <i>thinlet</i> component paints the content
	 */
	public void paint(Graphics g) { 
		if (doublebuffer == null) {
			Dimension d = parent_frame.getSize();
			//Dimension d = getSize();
			doublebuffer = createImage(d.width, d.height);
		}
		Graphics dg = doublebuffer.getGraphics();
		dg.setClip(g.getClipBounds());
		super.paint(dg);
		dg.dispose();
		g.drawImage(doublebuffer,0,0,this);
	}
	
	/**
	 * Clear the double buffer image (because the frame has been resized),
	 * the overriden method lays out its components
	 * (centers the <i>thinlet</i> component)
	 */
	public void doLayout() {
		if (doublebuffer != null) {
			doublebuffer.flush();
			doublebuffer = null;
		}
//		frm.doLayout();
		super.doLayout();
//		frm.doLayout();
//		super.doLayout();
	}

	public void componentHidden(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void componentMoved(ComponentEvent e) {
		//Parent frame moved
		setSize();
	}

	public void componentResized(ComponentEvent e) {
		//parent frame resized
		setSize(
				e.getComponent().getX(),
				e.getComponent().getY(),
				e.getComponent().getWidth(),
				e.getComponent().getHeight()
				);
	}

	public void componentShown(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}

}
