/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap.core;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.io.Serializable;

import es.prodevelop.gvsig.mobile.fmap.proj.ICoordTrans;

/**
 * Aade el tipo de la shape a la interfaz java.awt.Shape
 */
public interface FShape extends Shape, Serializable {
	public final static int NULL = 0;
	public final static int POINT = 1;
	public final static int LINE = 2;
	public final static int POLYGON = 3;
	public final static int TEXT = 4;
	public final static int MULTI = 5;
	public final static int MULTIPOINT = 6;
	public final static int MULTIPOLYGON = 7; 
	public final static int CIRCLE = 8;
	public final static int ARC = 9;
	public final static int ELLIPSE= 10;
	public final static int Z= 16;
	/**
	 * Obtiene el tipo de shape que contiene. Puede ser una de las constantes
	 * POINT, LINE o POLYGON
	 *
	 * @return int
	 */
	public int getShapeType();

	/**
	 * Clona FShape.
	 *
	 * @return FShape clonado.
	 */
	public FShape cloneFShape();

	/**
	 * Reprojecta el FShape.
	 *
	 * @param ct Coordenadas de transformacin.
	 */
	public void reProject(ICoordTrans ct);
	/**
	 * Returns the handlers they utilized to stretch the geometries.
	 *
	 * @return Handlers
	 */
	public Handler[] getStretchingHandlers();
	/**
	 * Returns the hanlers they utilized to select the geometries.
	 *
	 * @return Handlers
	 */
	public Handler[] getSelectHandlers();
	
	public void transform(AffineTransform at);
	
	public boolean intersects(Rectangle2D r);
	
    /**
     * Returns a high precision and more accurate bounding box of
     * the <code>Shape</code> than the <code>getBounds</code> method.
     * Note that there is no guarantee that the returned 
     * {@link Rectangle2D} is the smallest bounding box that encloses 
     * the <code>Shape</code>, only that the <code>Shape</code> lies 
     * entirely within the indicated <code>Rectangle2D</code>.  The 
     * bounding box returned by this method is usually tighter than that 
     * returned by the <code>getBounds</code> method and never fails due 
     * to overflow problems since the return value can be an instance of 
     * the <code>Rectangle2D</code> that uses double precision values to 
     * store the dimensions.
     * @return an instance of <code>Rectangle2D</code> that is a
     *                 high-precision bounding box of the <code>Shape</code>.
     * @see #getBounds
     */
    public Rectangle2D getBounds2D();
	
	
}
