package es.prodevelop.gvsig.mobile.fmap.core.acetate;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */


import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

import es.prodevelop.geodetic.utils.conversion.ConversionCoords;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;

/**
 * This class contains the symbol icons used on the map with geographic info.
 * 
 * @author Carlos Snchez Perin (csanchez@prodevelop.es)
 */
public class MapFinishTrack implements IAcetateElement{
	
	private Point2D geoPoint = null;
	private Point2D gpsPoint = null;
	private boolean visible = true;
	/**
	 * Constructor
	 * @param i the icon image to be used. Preferable, a square image
	 * with odd size: w = h = 2n + 1 (n is a natural number).
	 * 
	 */
	public MapFinishTrack(double longitude_dest,double latitude_dest, IProjection projection){
		double[] coords=null;
		//To add selector of SRS
		if(projection!=null){
			coords=ConversionCoords.wgs842projection(longitude_dest, latitude_dest, projection);
		}else{
			coords=new double[2];
			coords[0]=longitude_dest;
			coords[1]=latitude_dest;
		}
		setGeoPoint(new Point2D.Double(coords[0],coords[1]));
		setGpsPoint(new Point2D.Double(coords[0],coords[1]));
	}
		
	public void draw(Graphics2D g, AffineTransform at) {	
		if (visible){
			Point2D aux = new Point2D.Double(0, 0);
			Point2D auxGPS = new Point2D.Double(0, 0);
			at.transform(getGeoPoint(), aux);
			at.transform(getGpsPoint(), auxGPS);
			g.drawLine((int)Math.round(aux.getX()),(int)Math.round(aux.getY()),(int)Math.round(auxGPS.getX()),(int)Math.round(auxGPS.getY()));
		}
	}
  
	/**
	 * @param geoPoint the geoPoint to set
	 */
	public void setGeoPoint(Point2D geoPoint) {
		this.geoPoint = geoPoint;
	}
	
	/**
	 * @param gpsPoint the gpsPoint to set
	 */
	public void setGpsPoint(Point2D gpsPoint) {
		this.gpsPoint = gpsPoint;
	}
	
	/**
	 * @return the geoPoint
	 */
	public Point2D getGeoPoint() {
		return geoPoint;
	}

	public Point2D getGpsPoint() {
		// TODO Auto-generated method stub
		return gpsPoint;
	}

	public double getRotation() {
		// TODO Auto-generated method stub
		return 0;
	}

	public void setRotation(double r) {
		// TODO Auto-generated method stub
		
	}

	public void setVisible(boolean b) {
		visible=b;
	}	
}
