/*
 * Created on 17-dic-2004
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap.driver.raster;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;

import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;

/**
 * Driver de raster (tal y como los abre CMS.
 * Primera aproximacin al raster. Borrador para revisar.
 *
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class DefaultRasterDriver implements GeorreferencedRasterDriver {
	
	public final static int FILE_TYPE_UKW = -1;
	public final static int FILE_TYPE_ECW = 0;
	public final static int FILE_TYPE_TIF = 1;
	public final static int FILE_TYPE_GIF = 2;
	public final static int FILE_TYPE_JPG = 3;
	
	private int type = FILE_TYPE_UKW;
	private File file = null;
	private String filePath = "";
	private IProjection proj = null;
	int trans = 255;
	
	public String getFilePath() {
		return filePath;
	}

	public String getName() {
		return "gvSIG Image Driver";
	}

	public void open(File f) throws IOException {
		file = f;
	}

	public void close() throws IOException {
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#initialize()
	 */
	public void initialize() throws IOException {
		if (proj != null){
			filePath = file.getCanonicalPath();
			type = getType(filePath);
			
			switch (type) {
			
			case FILE_TYPE_UKW:
				throw new IOException("Tipo desconocido.");
			case FILE_TYPE_ECW:
				break;
			case FILE_TYPE_TIF:
				break;
			case FILE_TYPE_GIF:
				break;
			case FILE_TYPE_JPG:
				break;
			
			}
			
		} else {
			throw new IOException("Proyeccin no asignada");
		}

	}
	
	private int getType(String path) {
		
		int ind_ext = path.lastIndexOf(".");
		if (ind_ext == -1) {
			return FILE_TYPE_UKW;
		}
		String ext = filePath.substring(ind_ext + 1, filePath.length());
		if (ext.compareToIgnoreCase("ecw") == 0) return FILE_TYPE_ECW;
		if (ext.compareToIgnoreCase("tif") == 0) return FILE_TYPE_TIF;
		if (ext.compareToIgnoreCase("tiff") == 0) return FILE_TYPE_TIF;
		if (ext.compareToIgnoreCase("gif") == 0) return FILE_TYPE_GIF;
		if (ext.compareToIgnoreCase("jpg") == 0) return FILE_TYPE_JPG;
		return FILE_TYPE_UKW;
	}
	

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.GeorreferencedRasterDriver#initialize(org.cresques.cts.IProjection)
	 */
	public void initialize(IProjection p) throws IOException {
		proj = p;
	}


	public int getTransparency() {
		return trans;
	}

	public void setTransparency(int t) {
		trans = t;
	}

	public IProjection getProjection() {
		return proj;
	}

	public void setProjection(IProjection p) {
		proj = p;
	}

	public int getNumBands() {
		return 0;
	}


	public int getRasterDataType() {
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.RasterDriver#getData(int, int, int)
	 */
	public Object getData(int x, int y, int band) {
		return null;
	}

	public boolean fileAccepted(File file) {
		
		try {
			filePath = file.getCanonicalPath();
			int ind_ext = filePath.lastIndexOf(".");
			if (ind_ext == -1) {
				return false;
			}
			String ext = filePath.substring(ind_ext + 1, filePath.length());
			return acceptedExtension(ext);
		} catch (IOException e) {
			return false;
		}
	}

	private boolean acceptedExtension(String ext) {
		
		if (ext.compareToIgnoreCase("ecw") == 0) return true;
		if (ext.compareToIgnoreCase("tif") == 0) return true;
		if (ext.compareToIgnoreCase("tiff") == 0) return true;
		if (ext.compareToIgnoreCase("gif") == 0) return true;
		if (ext.compareToIgnoreCase("jpg") == 0) return true;
		return false;
	}

	public int[] getPixel(double wcx, double wcy){
		return null;
	}

	public boolean isGeoreferenced() {
		return true;
	}
	
	public void setAffineTransform(AffineTransform t){ 
	}
	
	public AffineTransform getAffineTransform(){
		return null;
	}

	public Rectangle2D getFullExtent() {
		// TODO Auto-generated method stub
		return null;
	}

	public void draw(Graphics2D g, ViewPort viewPort) throws FMapDriverException {
		// TODO Auto-generated method stub
		
	}
	
}
