/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.driver.raster.osm;

import java.awt.geom.Rectangle2D;
import java.awt.geom.RectangularShape;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Random;

import org.apache.log4j.Logger;

public class OpenStreetMapTile implements Cloneable {
	
	private static Logger logger = Logger.getLogger(OpenStreetMapTile.class);
	
	private int leftInd = 0;
	private int topInd = 0;
	private int depth = 0;
	private Random rnd;
	private int max_tile = 0;
	private int layer = OpenStreetMapUtils.OSM_LAYER_MAPNIK;
	
	private Rectangle2D extent = null; // new Rectangle2D.Doutble(0, 0, 1, 1);
	
	public OpenStreetMapTile(int left, int top, int d, int lyr) {
		leftInd = left;
		topInd = top;
		depth = d;
		layer = lyr;
		
		double tile_w = 2 * OpenStreetMapUtils.SEMI_EQUATOR_LENGTH / Math.pow(2.0, d);
		extent = new Rectangle2D.Double(
				-OpenStreetMapUtils.SEMI_EQUATOR_LENGTH + leftInd * tile_w,
				OpenStreetMapUtils.SEMI_EQUATOR_LENGTH - (topInd + 1) * tile_w,
				tile_w,
				tile_w);
		rnd = new Random(System.currentTimeMillis());
		max_tile = Math.round((float) Math.pow(2.0, depth));
		
	}

	public RectangularShape getExtent() {
		return extent;
	}

	public URL getURL() {

		String u = "http://" + randomLetter() + ".tile";
		
		if (layer == OpenStreetMapUtils.OSM_LAYER_OSMARENDER) {
			u = "http://" + randomLetter() + ".tah";
			u = u + ".openstreetmap.org/Tiles/tile.php/"
			 + depth + "/"
			 + leftInd + "/"
			 + topInd + ".png";
		} else {
			
			if (layer == OpenStreetMapUtils.OSM_LAYER_MAPNIK) {
				u = "http://" + randomLetter() + ".tile";
				u = u + ".openstreetmap.org/"
				 + depth + "/"
				 + leftInd + "/"
				 + topInd + ".png";
			}
		}
		
		
		URL resp = null;
		try {
			resp = new URL(u);
		} catch (MalformedURLException e) {
			logger.error("Bad url: " + u);
		}
		return resp;
	}
	
	public String getCacheFilePath() {
		
		String sub = OpenStreetMapUtils.OSM_LAYER_MAPNIK_SUBFOLDER;
		if (layer == OpenStreetMapUtils.OSM_LAYER_OSMARENDER) {
			sub = OpenStreetMapUtils.OSM_LAYER_OSMARENDER_SUBFOLDER;
		}
		String path = ""
			+ depth + File.separator
			+ leftInd + File.separator
			+ topInd + ".png";
		return  sub + File.separator + path;
	}
	

	private String randomLetter() {
		
		return "b";
		
		/*
		int r = rnd.nextInt(3);
		if (r == 0) return "a";
		if (r == 1) return "b";
		return "c";
		*/
	}
	
	// ------------- movements

	public OpenStreetMapTile getTileToTheRight() {
		return new OpenStreetMapTile((leftInd + 1)% max_tile, topInd, depth, layer);
	}
	
	public Object clone() {
		return new OpenStreetMapTile(leftInd, topInd, depth, layer);
	}

	public OpenStreetMapTile getTileUnder() {
		return new OpenStreetMapTile(leftInd, (topInd + 1) % max_tile, depth, layer);
	}
	

}
