/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.driver.raster.osm;

import java.awt.geom.Point2D;

public class OpenStreetMapUtils {
	
	public static double SEMI_EQUATOR_LENGTH = 180.0;
	public static final boolean TILE_STRATEGY_RESIZE_DOWN = true;
	
	public static final int OSM_LAYER_MAPNIK = 0;
	public static final int OSM_LAYER_OSMARENDER = 1;
	
	public static final String OSM_LAYER_MAPNIK_SUBFOLDER = "mnik";
	public static final String OSM_LAYER_OSMARENDER_SUBFOLDER = "osma";

	public static OpenStreetMapTile getTileFor(Point2D p, int d, int lyr) {
		
		Point2D _p = normalize(p);

		double left = _p.getX() + SEMI_EQUATOR_LENGTH;
		double top = -_p.getY() + SEMI_EQUATOR_LENGTH;
		
		double tile_w = 2 * SEMI_EQUATOR_LENGTH / Math.pow(2.0, d);
		int max_ind = Math.round((float) Math.pow(2.0, d));
		
		int left_ind = ((int) Math.floor(left / tile_w)) % max_ind;
		int top_ind = ((int) Math.floor(top / tile_w)) % max_ind;
		
		OpenStreetMapTile tile = new OpenStreetMapTile(left_ind, top_ind, d, lyr);
		
		return tile;
	}
	
	public static int[] getDepthAndTileSize(
			double world_dist, int pixels, boolean st) {
		
		int[] resp = new int[2];
		
		double dist_per_pixel = world_dist / pixels;
		int d = 0;
		while (dist_per_pixel < (2.0 * SEMI_EQUATOR_LENGTH / 256.0)) {
			d++;
			dist_per_pixel = 2 * dist_per_pixel;
		}
		
		resp[0] = d;
		
		if (!st) {
			resp[0]--;
			if (resp[0] < 0) resp[0] = 0;
		}
		
		dist_per_pixel = 2.0 * SEMI_EQUATOR_LENGTH / dist_per_pixel;
		resp[1] = Math.round((float) dist_per_pixel);
		
		if (!st) {
			resp[1] = 2 * resp[1];
		}

		return resp;
	}
	
	public static Point2D normalize(Point2D p) {
		
		if ((p.getX() >= -SEMI_EQUATOR_LENGTH) && (p.getX() < SEMI_EQUATOR_LENGTH)
				&& (p.getY() > -SEMI_EQUATOR_LENGTH) && (p.getY() <= SEMI_EQUATOR_LENGTH)) {
			return p;
		}

		double tx = 0;
		double ty = 0;
		double x = 0, y = 0;
		if (p.getX() > 0) {
			tx = 1 + Math.floor((p.getX() - SEMI_EQUATOR_LENGTH) / (2 * SEMI_EQUATOR_LENGTH));
			x = p.getX() - (tx * (2 * SEMI_EQUATOR_LENGTH));
		} else {
			tx = -(1 + Math.floor((-p.getX() - SEMI_EQUATOR_LENGTH) / (2 * SEMI_EQUATOR_LENGTH)));
			x = p.getX() - (tx * (2 * SEMI_EQUATOR_LENGTH));
		}
		
		if (p.getY() > 0) {
			ty = 1 + Math.floor((p.getY() - SEMI_EQUATOR_LENGTH) / (2 * SEMI_EQUATOR_LENGTH));
			y = p.getY() - (ty * (2 * SEMI_EQUATOR_LENGTH));
		} else {
			ty = -(1 + Math.floor((-p.getY() - SEMI_EQUATOR_LENGTH) / (2 * SEMI_EQUATOR_LENGTH)));
			y = p.getY() - (ty * (2 * SEMI_EQUATOR_LENGTH));
		}
		
		return new Point2D.Double(x, y);
	}
	
	public static double normalizeOffset(double d) {
		
		if (Math.abs(Math.abs(d) - (2 * OpenStreetMapUtils.SEMI_EQUATOR_LENGTH))
			<
			(0.000001 * OpenStreetMapUtils.SEMI_EQUATOR_LENGTH)) return 0;
		return d;
	}
	
	
	

}
