/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap.driver.vect.shp;

import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.SHPMultiLine;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.SHPMultiPoint;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.SHPPoint;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.SHPPolygon;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.SHPShape;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.ShapefileException;



/**
 * Clase con las constantes que representan los diferentes tipos de Shape y
 * mtodos estticos relativos a los shapes.
 *
 * @author Vicente Caballero Navarro
 */
public class SHP {

    public static final int SHPT_NULL = 		0;
    public static final int SHPT_POINT = 		1;	
    public static final int SHPT_ARC = 			3;	
    public static final int SHPT_POLYGON = 		5;
    public static final int SHPT_MULTIPOINT = 	8;	
    public static final int SHPT_POINTZ = 		11;	
    public static final int SHPT_ARCZ = 		13;
    public static final int SHPT_POLYGONZ = 	15;
    public static final int SHPT_MULTIPOINTZ = 	18;
    public static final int SHPT_POINTM = 		21;
    public static final int SHPT_ARCM = 		23;
    public static final int SHPT_POLYGONM = 	25;
    public static final int SHPT_MULTIPOINTM = 	28;
    public static final int SHPT_MULTIPATCH = 	31;

    /**
     * Crea a partir del tipo de geometra un shape del tipo ms adecuado.
     *
     * @param type Tipo de geometra.
     *
     * @return Geometra ms adecuada.
     *
     * @throws ShapefileException Se lanza cuando es causada por la creacin
     *         del shape.
     */
    public static SHPShape create(int type) throws ShapefileException {
        SHPShape shape;

        switch (type) {
            case 1:
            case 11:
            case 21:
                shape = new SHPPoint(type);

                break;

            case 3:
            case 13:
            case 23:
                shape = new SHPMultiLine(type);

                break;

            case 5:
            case 15:
            case 25:
                shape = new SHPPolygon(type);

                break;

            case 8:
            case 18:
            case 28:
                shape = new SHPMultiPoint(type);

                break;

            default:
                shape = null;
        }

        return shape;
    }
    

    /**
     * Crea un fichero en formato shape con las geometras que se pasan como
     * parmetro en forma de array, un bitset para saber las seleccionadas, un
     * SelectableDataSource para obtener los valores y el fichero a crear.
     *
     * @param fgs Array de geometras.
     * @param bitset Bitset con la seleccin.
     * @param sds SelectableDataSource.
     * @param f Fichero a crear o modificar.
     */
   

    /**
     * Obtiene un fichero shape con las geometras que se pasan como parmetro,
     * sin crear un dbf.
     *
     * @param fgs Array de IGeometries.
     * @param f Fichero que hay que crear.
     */
    
    /**
     * Devuelve un array con dos doubles, el primero representa el mnimo valor
     * y el segundo el mximo de entre los valores que se pasan como parmetro
     * en forma de array.
     *
     * @param zs Valores a comprobar.
     *
     * @return Array de doubles con el valor mnimo y el valor mximo.
     */
    
    public static double[] getZMinMax(double[] zs) {
        if (zs == null) {
            return null;
        }

        double min = Double.MAX_VALUE;
        // double max = Double.MIN_VALUE;
        // corregido tras el mail:
        double max = -0.5 * Double.MAX_VALUE;

        for (int i = 0; i < zs.length; i++) {
            if (zs[i] > max) {
                max = zs[i];
            }

            if (zs[i] < min) {
                min = zs[i];
            }
        }

        return new double[] { min, max };
    }
}
