/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/
package es.prodevelop.gvsig.mobile.fmap.driver.vect.write;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.nio.ByteOrder;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.shp.SHP;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.shp.ShapeFileHeader;
import es.prodevelop.gvsig.mobile.fmap.edition.VectorialEditableAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.symbol.FConstant;

/**
 * This class is used write SHP file to disk
 *
 * @author Vicente Caballero Navarro
 * @author Juan Lucas Dominguez Rubio, jldominguez@prodevelop.es
 */
public class SHPFileWrite {
	
	private SHPShape itemShapeWriter;
	
	private static Logger logger = Logger.getLogger(SHPFileWrite.class);
	
	private int m_offset = 0;
	private int m_pos = 0;
	private int m_type;
	private int m_cnt;
	
	private OrderFileOutputStream shpOutputStream, shxOutputStream; 

	/**
	 * Crea un nuevo SHPFileWrite.
	 *
	 * @param shpChannel DOCUMENT ME!
	 * @param shxChannel DOCUMENT ME!
	 */
	public SHPFileWrite(File shp_out, File shx_out) {
		
		try {
			shp_out.createNewFile();
			shx_out.createNewFile();
			
			shpOutputStream = new OrderFileOutputStream(shp_out);
			shxOutputStream = new OrderFileOutputStream(shx_out);
		} catch (IOException e) {
			logger.error("While createing SHPFileWrite: " + e.getMessage());
		}
	}

	/**
	 * Close the underlying Channels.
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	public void close() throws IOException {
		
		shpOutputStream.flush();
		shpOutputStream.close();
		shxOutputStream.flush();
		shxOutputStream.close();
		
		shpOutputStream = null;
		shxOutputStream = null;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param geometries DOCUMENT ME!
	 * @param type DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 * @throws ShapefileException DOCUMENT ME!
	 */
	public synchronized void write(VectorialEditableAdapter vea, int type)
		throws Exception {
		
		itemShapeWriter = SHP.create(type);
		writeHeaders(vea, type);

		int count = vea.getShapeCount();
		for (int i = 0; i< count; i++) {
			writeGeometry(vea.getShape(i));
		}
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param geometries DOCUMENT ME!
	 * @param type DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 * @throws ShapefileException DOCUMENT ME!
	 */
	public synchronized int write(ReadableVectorial rv, int shptype, int feattype, int esritype)
		throws Exception {
		
		itemShapeWriter = SHP.create(feattype);
		int writtenc = writeHeaders(rv, shptype, feattype, esritype);
		
		if (writtenc == 0) {
			return 0;
		}

		int count = rv.getShapeCount();
		for (int i = 0; i< count; i++) {
			IGeometry ig = rv.getShape(i);
			if (ig.getGeometryType() == shptype) {
				writeGeometry(ig);
			}
		}
		return writtenc;
	}	

	/**
	 * DOCUMENT ME!
	 *
	 * @param geometries DOCUMENT ME!
	 * @param type DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	private synchronized void writeHeaders(VectorialEditableAdapter vea, int type)
		throws Exception {
		
		int fileLength = 100;
		Rectangle2D extent = null;
		int len = vea.getShapeCount();

		for (int i = len - 1; i >= 0; i--) {
			IGeometry fgeometry = vea.getShape(i);
			itemShapeWriter.obtainsPoints(fgeometry);
			int size = itemShapeWriter.getLength(fgeometry) + 8;
			fileLength += size;

			if (extent == null) {
				extent = new Rectangle2D.Double(fgeometry.getBounds2D().getMinX(),
						fgeometry.getBounds2D().getMinY(),
						fgeometry.getBounds2D().getWidth(),
						fgeometry.getBounds2D().getHeight());
			} else {
				extent.add(fgeometry.getBounds2D());
			}
		}
		writeHeaders(extent, type, len, fileLength);
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param geometries DOCUMENT ME!
	 * @param type DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	private synchronized int writeHeaders(ReadableVectorial rv, int shptype, int feattype, int esritype)
		throws Exception {
		
		int fileLength = 100;
		Rectangle2D extent = null;
		int len = rv.getShapeCount();
		int writt_len = 0;

		for (int i = len - 1; i >= 0; i--) {
			IGeometry fgeometry = rv.getShape(i);
			
			if (fgeometry.getGeometryType() == shptype) {
				
				writt_len++;
				
				itemShapeWriter.obtainsPoints(fgeometry);
				int size = itemShapeWriter.getLength(fgeometry) + 8;
				fileLength += size;

				if (extent == null) {
					extent = new Rectangle2D.Double(fgeometry.getBounds2D().getMinX(),
							fgeometry.getBounds2D().getMinY(),
							fgeometry.getBounds2D().getWidth(),
							fgeometry.getBounds2D().getHeight());
				} else {
					extent.add(fgeometry.getBounds2D());
				}
			}
		}
		
		if (writt_len == 0) {
			logger.warn("No features of type " + shptype + " found.");
			return 0;
		}

		
		writeHeaders(extent, esritype, writt_len, fileLength);
		return writt_len;
	}

	/**
	 * Writes shape header (100 bytes)
	 *
	 * @param bounds DOCUMENT ME!
	 * @param type DOCUMENT ME!
	 * @param numberOfGeometries DOCUMENT ME!
	 * @param fileLength DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	public synchronized void writeHeaders(Rectangle2D bounds, int type,
		int numberOfGeometries, int fileLength) throws IOException {

		ShapeFileHeader header = new ShapeFileHeader();

		header.write(shpOutputStream, type, numberOfGeometries, fileLength / 2,
			bounds.getMinX(), bounds.getMinY(), bounds.getMaxX(),
			bounds.getMaxY(), 0, 0, 0, 0);

		header.write(shxOutputStream, type, numberOfGeometries,
			50 + (4 * numberOfGeometries), bounds.getMinX(), bounds.getMinY(),
			bounds.getMaxX(), bounds.getMaxY(), 0, 0, 0, 0);

		m_offset = 50;
		m_type = type;
		m_cnt = 0;
	}
	
	public synchronized void writeIGeometry(IGeometry g) throws IOException, ShapefileException
	{
		int shapeType = getShapeType(g.getGeometryType());
		itemShapeWriter = SHP.create(shapeType);
		writeGeometry(g);
	}



	/**
	 * Writes a single Geometry.
	 *
	 * @param g
	 * @return the position of buffer (after the last geometry, it will allow you to
	 * write the file size in the header.
	 * @throws IOException
	 */
	public synchronized void writeGeometry(IGeometry g) throws IOException {
		
		itemShapeWriter.obtainsPoints(g);
		int length = itemShapeWriter.getLength(g);

		length /= 2;

		shpOutputStream.order(ByteOrder.BIG_ENDIAN);
		shpOutputStream.putInt(++m_cnt);
		shpOutputStream.putInt(length);
		shpOutputStream.order(ByteOrder.LITTLE_ENDIAN);
		shpOutputStream.putInt(m_type);
		itemShapeWriter.write(shpOutputStream, g);

		// write to the shx
		shxOutputStream.putInt(m_offset);
		shxOutputStream.putInt(length);
		m_offset = m_offset + length + 4;
	}

	/**
	 * Returns a shapeType compatible with shapeFile constants from a gvSIG's IGeometry type
	 * @param geometryType
	 * @return a shapeType compatible with shapeFile constants from a gvSIG's IGeometry type
	 */
	public static int getShapeType(int geometryType) {
		if (geometryType>=FShape.Z){
			switch (geometryType - FShape.Z) {
			case (FShape.POINT):
				return FConstant.SHAPE_TYPE_POINTZ;

			case (FShape.LINE):
				return FConstant.SHAPE_TYPE_POLYLINEZ;

			case FShape.POLYGON:
				return FConstant.SHAPE_TYPE_POLYGONZ;

			case FShape.MULTIPOINT:
				return FConstant.SHAPE_TYPE_MULTIPOINTZ; //TODO falta aclarar cosas aqu.
		}

		}else{
			switch (geometryType) {
				case FShape.POINT:
					return FConstant.SHAPE_TYPE_POINT;

				case FShape.LINE:
				case FShape.ELLIPSE:
				case FShape.CIRCLE:
				case FShape.ARC:
					return FConstant.SHAPE_TYPE_POLYLINE;

				case FShape.POLYGON:
					return FConstant.SHAPE_TYPE_POLYGON;

				case FShape.MULTIPOINT:
					return FConstant.SHAPE_TYPE_MULTIPOINT; //TODO falta aclarar cosas aqu.
			}
		}
			return FConstant.SHAPE_TYPE_NULL;
		}
	
	public static int getSHPShapeType(int type) {
		
		switch (type) {
		
		case FConstant.SHAPE_TYPE_POINT:
		case FConstant.SHAPE_TYPE_POINTZ:
			return SHP.SHPT_POINT;
			
		case FConstant.SHAPE_TYPE_MULTIPOINT:
		case FConstant.SHAPE_TYPE_MULTIPOINTZ:
			return SHP.SHPT_MULTIPOINT;

		case FConstant.SHAPE_TYPE_POLYLINE:
		case FConstant.SHAPE_TYPE_POLYLINEZ:
			return SHP.SHPT_ARC;

		case FConstant.SHAPE_TYPE_POLYGON:
		case FConstant.SHAPE_TYPE_POLYGONZ:
			return SHP.SHPT_POLYGON;

		default:
			logger.error("Not supported fshapetype: " + type);
			return SHP.SHPT_NULL;
		}
	}

}
