/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/
package es.prodevelop.gvsig.mobile.fmap.driver.vect.write;

import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.util.ArrayList;

import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry3D;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.shp.SHP;
import es.prodevelop.gvsig.mobile.fmap.symbol.FConstant;



/**
 * Elemento shape de tipo multipunto.
 *
 * @author Vicente Caballero Navarro
 */
public class SHPMultiPoint implements SHPShape {
	private int m_type;
	private int numpoints;
	private Point2D[] points;
	private double[] zs;

	/**
	 * Crea un nuevo SHPMultiPoint.
	 */
	public SHPMultiPoint() {
		m_type = FConstant.SHAPE_TYPE_MULTIPOINT;
	}

	/**
	 * Crea un nuevo SHPMultiPoint.
	 *
	 * @param type Tipo de multipunto.
	 *
	 * @throws ShapefileException
	 */
	public SHPMultiPoint(int type) throws ShapefileException {
		if ((type != FConstant.SHAPE_TYPE_MULTIPOINT) &&
				(type != FConstant.SHAPE_TYPE_MULTIPOINTM) &&
				(type != FConstant.SHAPE_TYPE_MULTIPOINTZ)) {
			throw new ShapefileException("No es de tipo 8, 18, o 28");
		}

		m_type = type;
	}

	/**
	 * Devuelve el tipo de multipoint en concreto.
	 *
	 * @return Tipo de multipoint.
	 */
	public int getShapeType() {
		return m_type;
	}



	/**
	 * @throws IOException 
	 * @see com.iver.cit.gvsig.fmap.shp.SHPShape#write(ByteBuffer, IGeometry)
	 */
	public void write(OrderFileOutputStream buffer, IGeometry geometry) throws IOException {

		Rectangle2D box = geometry.getBounds2D();
		buffer.putDouble(box.getMinX());
		buffer.putDouble(box.getMinY());
		buffer.putDouble(box.getMaxX());
		buffer.putDouble(box.getMaxY());
		///obtainsPoints(geometry.getGeneralPathXIterator());
		buffer.putInt(numpoints);

		for (int t = 0, tt = numpoints; t < tt; t++) {
			Point2D point = points[t];
			buffer.putDouble(point.getX());
			buffer.putDouble(point.getY());
		}

		  if (m_type == FConstant.SHAPE_TYPE_MULTIPOINTZ) {
		   double[] zExtreame = SHP.getZMinMax(zs);
		   if (Double.isNaN(zExtreame[0])) {
		       buffer.putDouble(0.0);
		       buffer.putDouble(0.0);
		   } else {
		       buffer.putDouble(zExtreame[0]);
		       buffer.putDouble(zExtreame[1]);
		   }
		   for (int t = 0; t < numpoints; t++) {
		       double z = zs[t];
		       if (Double.isNaN(z)) {
		           buffer.putDouble(0.0);
		       } else {
		           buffer.putDouble(z);
		       }
		   }
		   }
		   if ((m_type == FConstant.SHAPE_TYPE_MULTIPOINTM) ||
		           (m_type == FConstant.SHAPE_TYPE_MULTIPOINTZ)) {
		       buffer.putDouble(-10E40);
		       buffer.putDouble(-10E40);
		       for (int t = 0; t < numpoints; t++) {
		           buffer.putDouble(-10E40);
		       }
		   }

	}

	/**
	 * @see com.iver.cit.gvsig.fmap.shp.SHPShape#getLength(com.iver.cit.gvsig.core.BasicShape.FGeometry)
	 */
	public int getLength(IGeometry fgeometry) {
		//FMultiPoint2D mp = (FMultiPoint2D) fgeometry.getShape();
		///obtainsPoints(fgeometry.getGeneralPathXIterator());

		int length;

		if (m_type == FConstant.SHAPE_TYPE_MULTIPOINT) {
			// two doubles per coord (16 * numgeoms) + 40 for header
			length = (numpoints * 16) + 40;
		} else if (m_type == FConstant.SHAPE_TYPE_MULTIPOINTM) {
			// add the additional MMin, MMax for 16, then 8 per measure
			length = (numpoints * 16) + 40 + 16 + (8 * numpoints);
		} else if (m_type == FConstant.SHAPE_TYPE_MULTIPOINTZ) {
			// add the additional ZMin,ZMax, plus 8 per Z
			length = (numpoints * 16) + 40 + 16 + (8 * numpoints);
		} else {
			throw new IllegalStateException("Expected ShapeType of Arc, got " +
				m_type);
		}

		return length;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.shp.write.SHPShape#obtainsPoints(com.iver.cit.gvsig.fmap.core.GeneralPathXIterator)
	 */
	public void obtainsPoints(IGeometry g) {
		if (FConstant.SHAPE_TYPE_MULTIPOINTZ == m_type){
			zs=((IGeometry3D)g).getZs();
		}
		PathIterator theIterator = g.getPathIterator(null); //polyLine.getPathIterator(null, flatness);
		double[] theData = new double[6];
		ArrayList ps=new ArrayList();
		while (!theIterator.isDone()) {
			//while not done
			int theType = theIterator.currentSegment(theData);

			ps.add(new Point2D.Double(theData[0], theData[1]));
			theIterator.next();
		} //end while loop
		points=(Point2D[])ps.toArray(new Point2D.Double[0]);
		numpoints=points.length;
	}
//	public void setFlatness(double flatness) {
//		//this.flatness=flatness;
//	}
}
