/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/
package es.prodevelop.gvsig.mobile.fmap.driver.vect.write.export;

import java.io.File;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.SHPFileWrite;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.util.string.StringUtilities;

/**
 * This class creates SHP and SHX files from a given ReadableVectorial object.
 * Used to export a vector layer
 *
 * @author Juan Lucas Dominguez Rubio, jldominguez@prodevelop.es
 * @author Vicente Caballero Navarro
 */
public class SHPSHXFromReadVect {
	
	private static Logger logger = Logger.getLogger(SHPSHXFromReadVect.class);
	
	private ReadableVectorial theRV = null;
	
	private String shpPath;
	private String shxPath;
	private int temp = 0;
	
	private int shp_type;
	private int feat_type;
	private int esri_type;

	/**
	 * Crea un nuevo SHPSHXFromGeometries.
	 *
	 * @param geometries DOCUMENT ME!
	 * @param file DOCUMENT ME!
	 */
	public SHPSHXFromReadVect(ReadableVectorial rv, File file, int _type) {
		theRV = rv;
		shp_type = _type;
		feat_type = SHPFileWrite.getShapeType(_type);
		esri_type = SHPFileWrite.getSHPShapeType(feat_type);
		setFile(file);
	}

	//private FBitSet bitset = null;
	//private int num = 0;

	/**
	 * DOCUMENT ME!
	 *
	 * @param f DOCUMENT ME!
	 */
	private void setFile(File f) {
		shpPath = f.getAbsolutePath();
		String strFichshx = StringUtilities.replace(f.getAbsolutePath(), ".shp", ".shx");
		shxPath = StringUtilities.replace(strFichshx, ".SHP", ".SHX");
		
	}

	/**
	 * DOCUMENT ME!
	 */

	/*public void setType(int i) {
	   type = i;
	   }
	 */

	/**
	 * Finaliza el visitor.
	 */
	public int create() {
		
		SHPFileWrite filewrite;
		int written_count = 0;
		
		File shp_f = null;
		File shx_f = null;  

		try {
			shp_f = new File(shpPath);  
			shx_f = new File(shxPath);  
			filewrite = new SHPFileWrite(shp_f, shx_f);

			if (theRV.getShapeCount() > 0) {
				written_count = filewrite.write(theRV, shp_type, feat_type, esri_type);
				filewrite.close();
			}
		} catch (Exception e1) {
			logger.error("While writing SHP/SHX: " + e1.getMessage());
			return 0;
		}

		if (written_count == 0) {
			shp_f.delete();
			shx_f.delete();
			logger.warn("No SHP, SHX files written (shape type not found)");
		} else {
			logger.debug("Wrote SHP, SHX (" + written_count + " features)");
		}
		return written_count;
	}


}
