/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap.layer;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.List;

import es.prodevelop.gvsig.mobile.fmap.MapContext;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.proj.ICoordTrans;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;

/**
 * Interfaz que tienen que implementar todas las capas.
 */
public interface FLayer {
	/**
	 * Obtiene una representacin de la coleccin de capas de forma recursiva
	 *
	 * @return XMLEntity.
	 * @throws XMLException
	 */
	//XMLEntity getXMLEntity() throws XMLException;

	/**
	 * Inserta las propiedades del XMLEntity al objeto actual.
	 *
	 * @param xml XMLEntity
	 *
	 * @throws XMLException
	 */
	//void setXMLEntity(XMLEntity xml) throws XMLException;

	/**
	 * Pone la capa actual a activa o inactiva segn el boolean que se pasa
	 * como parmetro.
	 *
	 * @param selected activa.
	 */
	void setActive(boolean selected);

	/**
	 * Devuelve true si la capa esta activa.
	 *
	 * @return activa.
	 */
	boolean isActive();

	/**
	 * Inserta un nombre a la capa.
	 *
	 * @param name nombre.
	 */
	void setName(String name);

	/**
	 * Devuelve el nombre de la capa.
	 *
	 * @return nombre de la capa.
	 */
	String getName();
	
	void setMemo(String memo);
	String getMemo();

	/**
	 * Realiza las operaciones de inicializacin de la capa. El mtodo es
	 * invocado una nica vez durante la vida de la capa y justo antes de
	 * visualizar la capa
	 *
	 * @throws DriverIOException
	 */
	void load() throws FMapDriverException;

	/**
	 * Pone la capa en modo visible o no visible.
	 *
	 * @param visibility visibilidad.
	 */
	void setVisible(boolean visibility);
	void setInfoable(boolean info);
	long getId();

	/**
	 * Devuelve true si la capa es visible.
	 * Es dependiente isAvialable @link isAvialable 
	 *
	 * @return visibilidad.
	 * 
	 * @see isAvialable()
	 * @see setAvialable()
	 * @see visibleRequired()
	 */
	boolean isVisible();
	boolean isInfoable();

	/**
	 * Devuelve el FLayers padre de la capa.
	 *
	 * @return FLayers padre de la capa.
	 */
	/**
	 * Devuelve el FMap al que est aadida la capa o null si la capa no ha
	 * sido aadida a ningn FMap
	 *
	 * @return FMap
	 */
	public MapContext getMapContext();
	public void setMapContext(MapContext m);

	/**
	 * Inserta el FLayers padre de la capa.
	 *
	 * @param root capa padre.
	 */

	/**
	 * Obtiene la extensin completa de la capa
	 *
	 * @return FullExtent.
	 *
	 * @throws FMapDriverException
	 */
	Rectangle2D getFullExtent() throws FMapDriverException;

	/**
	 * Dibuja la capa
	 *
	 * @param g Graphics2D sobre el que dibujar.
	 * @param viewPort Propiedades de la vista.
	 *
	 * @throws FMapDriverException
	 */
	void draw(Graphics2D g, ViewPort viewPort) throws FMapDriverException;

	/**
	 * Inserta las coordenadas de transformacin.
	 *
	 * @param ct Coordenadas de transformacin.
	 */
	void setCoordTrans(ICoordTrans ct);

	/**
	 * Devuelve las coordenadas de transformacin.
	 *
	 * @return Coordenadas de transformacin.
	 */
	ICoordTrans getCoordTrans();

	/**
	 * Aade un listener LayerListener a la lista de listeners.
	 *
	 * @param o Listener.
	 *
	 * @return True si es correcta la insercin del listener.
	 */
	public boolean addLayerListener(LayerListener o);
	public LayerListener[] getLayerListeners();
	/**
	 * Borra de la lista el LayerListener que se pasa como parmetro.
	 *
	 * @param o Listener.
	 *
	 * @return True si es correcto el borrado del listener.
	 */
	public boolean removeLayerListener(LayerListener o);

	public boolean isWithinScale(double scale);


	/**
	 * La capa no se visualiza si est por debajo de esa escala
	 * @return la escala minima de visualizacin
	 */
	public double getMinScale();

	/**
	 * La capa no se visualiza si est por encima de esa escala
	 * @return la escala mxima de visualizacin
	 */
	public double getMaxScale();

	public void setMinScale(double minScale);
	public void setMaxScale(double maxScale);
	
	
	/**
	 * Icono a mostrar en el TOC junto a la capa
	 * @return el icono
	 */
	/**
	 * If the layer appears in the TOC then <b>true</b> is returned,
	 * if <b>false</b> the layer will not be displayed at the TOC
	 * although it remains in the view and in the project
	 */
	boolean isInTOC();
	
	/**
	 * @return true if this layer need a repaint.
	 */
	public boolean isDirty();

	/**
	 * true if this layer need a repaint. By default, all layers will be
	 * set to dirty when the extent changes. But for events like changing
	 * its legend, or editing a layer, we can perform some optimization
	 * in the method prepareDrawing from FMap.
	 * @param dirty
	 */
	public void setDirty(boolean dirty);
	
	/*
	 * This stuff is to save error's info that causes
	 * unavailable status.
	 * */
	/**
	 * Return if the layer is in OK status
	 * (it hasnt got errors)
	 */
	public boolean isOk();
	/**
	 * returns the number of errors that causes layer
	 * unavailable status
	 * @return
	 */
	public int getNumErrors();
	
	/**
	 * return the specified error
	 * @param i
	 * @return
	 */
	public FMapDriverException getError(int i);
	
	/**
	 * add an error cause to describe the layer's wrong status
	 * @param error
	 */
	public void addError(FMapDriverException error);
	
	/**
	 * Returns a list with all layer errors
	 * @return
	 */
	public List getErrors();
	

	/**
	 * @return set layer aviable or not.
	 */
	public void setAvailable(boolean available);
	
	/**
	 * @return true if this layer is aviable.
	 * 
	 * Default value is true.
	 */	
	public boolean isAvailable();
	
	/**
	 * Intenta recuperar una capa ante un posible error.
	 * Si tiene algun problema en la carga, marca
	 * el avialable a false y lanza una excepcion.
	 *
	 * @throws DriverIOException
	 */
	public void reload() throws FMapDriverException;
	
	
	/**
	 * Devuelve true si la capa esta establecida como visible.
	 *
	 * @return visibilidad.
	 */
	boolean visibleRequired();

	/**
	 * Devuelve una cadena con informacion sobre la capa.
	 *
	 * @return visibilidad.
	 */
	public String getInfoString();
	
	/**
	 * @return true if this layer can be put in edition mode and save the
	 * edits in itself.
	 */
	public boolean isWritable();
	
	/**
	 * Useful to associate any object to a layer. For example, you
	 * can attach a network definition to key "network" and
	 * check if a layer has a network loaded if getAssociatedObject("network")
	 * is not null
	 * 
	 * @param key
	 * @return null if key is not found
	 */
	public Object getProperty(Object key);
	
	/**
	 * @param key
	 * @param obj
	 */
	public void setProperty(Object key, Object obj);
	
	public void wakeUp(boolean driver_is_already_open);
	
	public StringBuffer toXML(String name, String proyectPath);

	public void clean();

	public void setEditing(boolean b) throws Exception;

	public boolean isEditing();

	/**
	 * 	Gets Custom Form File from a path asociated with the layer.
	 * @return String	XML File Path Name  
	 */
	public String getCustomForm();

	void setCustomFormActive(boolean state);

	void setCustomForm(String customFormPath);
}
