/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.layer;


import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.MapContext;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialFileDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.shp.DiskShpDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.shp.MemoryShpDriver;
import es.prodevelop.gvsig.mobile.fmap.legend.LegendFactory;
import es.prodevelop.gvsig.mobile.fmap.legend.SingleSymbolLegend;
import es.prodevelop.gvsig.mobile.fmap.legend.VectorialLegend;
//import es.prodevelop.gvsig.mobile.fmap.proj.cts.CRSFactory;
//import es.prodevelop.gvsig.mobile.fmap.proj.cts.IProjection;
import es.prodevelop.gvsig.mobile.fmap.proj.CRSFactory;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
//import es.prodevelop.gvsig.mobile.fmap.util.xml.IPersistance;
//import es.prodevelop.gvsig.mobile.fmap.util.xml.XMLEntity;
//import es.prodevelop.gvsig.mobile.fmap.util.xml.XMLException;

public class FLayerFileVectorial extends FLyrVect {
	
	private static Logger logger = Logger.getLogger(FLayerFileVectorial.class);
	private boolean loaded = false;
	private VectorialDriver vDriver = null;
	private boolean driver_already_open = false;
	
	public static FLayerFileVectorial createLayer(File f, String lyr_name, MapContext mc){
		MemoryShpDriver drv;
		logger.debug(Utils.time() + "MainWindow instancia el driver MEMORY SHP.");
		drv = new MemoryShpDriver();
		
		try {
			logger.debug(Utils.time() + "MainWindow llama open(f) de driver MEMORY SHP.");
			drv.open(f);
			logger.debug(Utils.time() + "MainWindow llama initialize() de driver MEMORY SHP.");
			drv.initialize();
		} catch (IOException e) {
			logger.error("Error while opening file: " + e.getMessage());
			
			logger.debug(Utils.time() + "No hay memoria suficuente: probamos DISK SHP.");
			drv = null; System.gc();
			drv = new DiskShpDriver();

			try {
				logger.debug(Utils.time() + "MainWindow llama open(f) de driver DISK SHP.");
				drv.open(f);
				logger.debug(Utils.time() + "MainWindow llama initialize() de driver DISK SHP.");
				drv.initialize();
			} catch (IOException e1) {
				logger.error("Error while opening file: " + e1.getMessage());
				drv = null; System.gc();
				return null;
			}
		}

		logger.debug(Utils.time() + "MainWindow instancia FLayerFileVectorial.");
		FLayerFileVectorial shplyr = new FLayerFileVectorial();
		shplyr.setMapContext(mc);
		
		shplyr.setVisible(true);
		shplyr.setAvailable(true);
		
		/*if (in_mem) {
			shplyr.setName(lyr_name + " [m]");
		} else {
			shplyr.setName(lyr_name + " [d]");
		}*/
		shplyr.setName(lyr_name);
		

		shplyr.setDriver(drv);
		shplyr.setProjection(mc.getProjection());
		
		logger.debug(Utils.time() + "Vamos a llamar a wakeup.");
		shplyr.wakeUp(true);

		// System.out.println("Layer shp instanciada (no probada)");
		logger.debug(Utils.time() + "MainWindow llama addLayer de mapcontext.");

		return shplyr;
	}
	
	public FLayerFileVectorial() {
		id = System.currentTimeMillis();
	}
	
	/* Esto deberia ir en el FLyrDefault */
	/*
	public void setProjectionByName(String projectionName) throws Exception{		
		IProjection proj = CRSFactory.getCRS(projectionName);
		if (proj == null) {
			throw new Exception("No se ha encontrado la proyeccion: "+ projectionName);
		}
		setProjection(proj);
	}
	*/

	public void setDriver(VectorialDriver vDriver) {
		this.vDriver = vDriver;
	}
	
	public VectorialDriver getDriver() {
		return this.vDriver ;
	}

	public void setDriverByName(String driverName) {
		this.setDriver(
		  (VectorialDriver)LayerFactory.getDM().getDriver(driverName) 
		);
	}

	
	/* Esto deberia ir en FLyrVect */
	private void initializeLegendDefault() {
		setLegend(LegendFactory.createSingleSymbolLegend());
	}
	
	public void setRandomColor() {
		((SingleSymbolLegend) getLegend()).setDefaultSymbol(
				FSymbol.createRandomSymbol(false));
	}

	public void wakeUp(boolean driver_open) {
		
		driver_already_open = driver_open;
		
		if (!loaded) {
			try {
				load();
			} catch (FMapDriverException e) {
			}
		}
		
	}

	
	public void load() throws FMapDriverException {
		if (this.getName() == null || this.getName().length() == 0) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new FMapDriverException("No se ha especificado nombre de capa");
		}
		if (this.vDriver == null) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new FMapDriverException("No se ha especificado driver");			
		}
		if (this.getProjection() == null) {
			this.setAvailable(false);
			//TODO: traducir???
			throw new FMapDriverException("No se ha especificado proyeccion");			
		}
		
		VectorialFileAdapter adapter = null;
		try {
			VectorialFileDriver filedrv = (VectorialFileDriver) vDriver;
			File f = filedrv.getFile();
			adapter = new VectorialFileAdapter(f);
			
			if (driver_already_open) adapter.setReferenceCount(1);

			adapter.setDriver(filedrv);
			setSource(adapter);
		} catch (Exception e) {
			this.setAvailable(false);
			throw new FMapDriverException(e.getMessage());
		}
		
		try {
			initializeLegendDefault();
		} catch (Exception e) {
			setAvailable(false);
			//TODO: traducir???
			throw new FMapDriverException(e.getMessage());
		}
	}
/*
	public void setXMLEntity(XMLEntity xml) throws XMLException {
        IProjection proj = null;
        if (xml.contains("proj")) {
            proj = CRSFactory.getCRS(xml.getStringProperty("proj"));
        }
        else
        {
            proj = this.getMapContext().getViewPort().getProjection();
        }
		this.setName(xml.getName());
		this.setProjection(proj);
		
        String driverName = xml.getStringProperty("other");
        VectorialDriver driver = null;
        try {
            driver = (VectorialDriver) LayerFactory.getDM().getDriver(driverName);
        } catch (DriverLoadException e) {
            // Si no existe ese driver, no pasa nada.
            // Puede que el desarrollador no quiera que
            // aparezca en el cuadro de dilogo y ha metido
            // el jar con sus clases en nuestro directorio lib.
            // Intentamos cargar esa clase "a pelo".
            if (xml.getChild(2).contains("className"))
            {
                String className2 = xml.getChild(2).getStringProperty("className");
                try {
                    driver = (VectorialDriver) Class.forName(className2).newInstance();
                } catch (Exception e1) {
                    throw new XMLException(e1);
                }
            }
        } catch (NullPointerException npe) {
            // Si no existe ese driver, no pasa nada.
            // Puede que el desarrollador no quiera que
            // aparezca en el cuadro de dilogo y ha metido
            // el jar con sus clases en nuestro directorio lib.
            // Intentamos cargar esa clase "a pelo".
            if (xml.getChild(2).contains("className"))
            {
                String className2 = xml.getChild(2).getStringProperty("className");
                try {
                    driver = (VectorialDriver) Class.forName(className2).newInstance();
                } catch (Exception e1) {
                    throw new XMLException(e1);
                }
            }
        }
        if (driver == null) {
        	throw new XMLException(new Exception("Error al cargar el driver"));
        }
        if (driver instanceof IPersistance)
        {
            IPersistance persist = (IPersistance) driver;
            persist.setXMLEntity(xml.getChild(2));
        }
        this.setDriver(driver);
        super.setXMLEntity(xml);
	}*/
	
	public StringBuffer toXML(String name, String projectPath) {
		StringBuffer buff = new StringBuffer();
		buff.append("<" + name +">\n");
		
			buff.append("<type>");
			buff.append(this.getClass().getName());
			buff.append("</type>\n");

			if (vDriver instanceof VectorialFileDriver){
				VectorialFileDriver drv = (VectorialFileDriver)vDriver;
				buff.append("<path>");
				String absPath = drv.getFilePath();
				String relPath = Utils.relPath(projectPath, absPath);
				if (relPath == null) relPath = absPath;
				buff.append(relPath);
				buff.append("</path>\n");
			}
		
		buff.append("</" + name +">\n");
		return buff;
	}
	
	public String getFilePath() {
		
		if (vDriver instanceof VectorialFileDriver){
			VectorialFileDriver drv = (VectorialFileDriver)vDriver;
			return drv.getFilePath();
		}
		return "[None]";
		
	}
	
	public void clean() {
		logger.debug("Cleaning layer: " + getName());
		
		vDriver = null;
		setLegend(null);
		getSource().setDriver(null);
		try {
			getRecordset().setSelectionSupport(null);
		} catch (Exception e) {
			logger.error("While cleaning layer: " + e.getMessage());
		}
		setRecordset(null);
		System.gc();
	}
	
}
