/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.layer;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.MapContext;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.RasterAdapter;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.RasterDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.RasterFileAdapter;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.RasterOperations;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.ecw.EcwRasterDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.memory.wkformat.WKFRasterDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.BadWMSResponseException;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.WMSLayerDescription;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.WMSProtocolHandler;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.WmsRasterDriver;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.cancel.Cancellable;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;

public class FLayerGenericRaster extends FLyrDefault implements RasterOperations {
	
	private static Logger logger = Logger.getLogger(FLayerGenericRaster.class);
	private RasterAdapter adapter = null;
	
	public static FLayerGenericRaster createWMSLayer(
			String url,
			String srs,
			String wmslayrs,
			String format,
			String name,
			MapContext mc) {
		
		URL _url = null;
		try {
			_url = new URL(url);
		} catch (Exception ex) {
			logger.error("Bad url: " + url);
			return null;
		}
		
		String _version = "";
		try {
			_version = WMSProtocolHandler.getSuitableWMSVersion(_url);
		} catch (BadWMSResponseException e) {
			logger.error("WMS layer was not created: " + e.getMessage() + ", returned NULL");
			return null;
		}
		
		String _sym = WMSProtocolHandler.getSymbol(url);
		
		WMSProtocolHandler wms_handler = new WMSProtocolHandler();
		wms_handler.setHost(url);
		wms_handler.getCapabilities(_version, _sym);
		WMSLayerDescription d = wms_handler.getRoot();
		Rectangle2D ext = d.getExtentForSRS(srs);
		
		String selected_and_infoable = WMSProtocolHandler.leaveInfoable(wmslayrs, d, false);
		String selected_and_infoable_names = WMSProtocolHandler.leaveInfoable(wmslayrs, d, true);
		ArrayList info_fmts = wms_handler.getInfoFormats();
		
		WmsRasterDriver drv = new WmsRasterDriver(
				url,
				wmslayrs,
				selected_and_infoable,
				selected_and_infoable_names,
				format,
				srs,
				ext,
				info_fmts,
				_version,
				_sym);
		RasterAdapter adapter = new RasterAdapter();
		adapter.setDriver(drv);
		FLayerGenericRaster wmsLayer = new FLayerGenericRaster();
		wmsLayer.setMapContext(mc);
		wmsLayer.setSource(adapter);
		wmsLayer.setVisible(true);
		wmsLayer.setName(name);
		return wmsLayer;
	}
	
	public static FLayerGenericRaster createWKFRasterLayer(File f, String name, MapContext mc){
		WKFRasterDriver drv = new WKFRasterDriver(mc.getViewPort());
		RasterFileAdapter adapter = new RasterFileAdapter(f);
		adapter.setDriver(drv);
		
		logger.debug(Utils.time() + "MainWindow instancia la capa raster WKFormat.");
		FLayerGenericRaster layer = new FLayerGenericRaster();
		layer.setMapContext(mc);
		layer.setName(name);
		layer.setSource(adapter);
		
		try {
			logger.debug(Utils.time() + "MainWindow llama al start() del adapter.");
			adapter.start();
		} catch (FMapDriverException e) {
			logger.error("While starting wk raster file: " + e.getMessage());
			return null;
		}
		return layer;
	}
	
	public static FLayerGenericRaster createEcwLayer(File f, String name, MapContext mc){
		EcwRasterDriver drv = new EcwRasterDriver(mc.getViewPort());
		RasterFileAdapter adapter = new RasterFileAdapter(f);
		adapter.setDriver(drv);
		
		logger.debug(Utils.time() + "MainWindow instancia la capa raster ECW.");
		FLayerGenericRaster ecwLayer = new FLayerGenericRaster();
		ecwLayer.setMapContext(mc);
		
		ecwLayer.setSource(adapter);
		
		try {
			logger.debug(Utils.time() + "MainWindow llama al start() del adapter.");
			adapter.start();
		} catch (FMapDriverException e) {
			logger.error("While starting ecw file: " + e.getMessage(), e);
		}
		
		ecwLayer.setVisible(true);
		ecwLayer.setName(name);
		return ecwLayer;
	}
	public FLayerGenericRaster() {
		id = System.currentTimeMillis();
	}

	public Rectangle2D getFullExtent() throws FMapDriverException {
		return adapter.getFullExtent();
	}

	public void draw(Graphics2D g, ViewPort viewPort) throws FMapDriverException {

		if (isWithinScale(viewPort.getScale())) {
			adapter.draw(g, viewPort);
		}
	}

	public boolean isWritable() {
		// TODO Auto-generated method stub
		return false;
	}

	public RasterAdapter getSource() {
		return adapter;
	}

	public void setSource(RasterAdapter ra) {
		adapter = ra;
	}

	public boolean isTiled() {
		// TODO Auto-generated method stub
		return false;
	}

	public int[] getTileSize() {
		// TODO Auto-generated method stub
		return null;
	}

	public ArrayList getInfo(Point p,
			double tolerance,
			Cancellable cancel, String srs) throws FMapDriverException {
		
		Point2D pReal =
			getMapContext().getViewPort().toMapPoint(new Point2D.Double(p.x, p.y));
		
		ArrayList resp = new ArrayList();
		
		if (adapter.getDriver() instanceof InfoByPoint) {
			
			InfoByPoint ibp = (InfoByPoint) adapter.getDriver();
			resp = ibp.getInfo(p, tolerance, cancel, srs);
			
		} else {
			
			// -------------------------------------------------------
			// driver is not info by point, return pixel's color data:
			int[] pixel = adapter.getDriver().getPixel(pReal.getX(), pReal.getY());
			
			if (pixel == null) {
				resp.add("[No data]");
				resp.add(null);
			} else {
				if (pixel.length == 1) { // special case
					
					resp.add("[Unknown error]");
					resp.add(null);
					switch (pixel[0]) {
					case RasterDriver.ONE_PIXEL_NO_DATA:
						resp.clear();
						resp.add("[No data]");
						resp.add(null);
						break;
					case RasterDriver.ONE_PIXEL_IMAGE_ERROR:
						resp.clear();
						resp.add("[Error while reading image]");
						resp.add(null);
						break;
					}

				} else {
					resp.add("Data:");
					String aux = "[";
					for (int i=0; i<pixel.length; i++) aux = aux + pixel[i] + ", ";
					aux = aux.substring(0, aux.length() - 2) + "]";
					resp.add(aux);
				}
			}

			// -------------------------------------------------------
		}

		return resp;
	}

	public void wakeUp(boolean driver_is_already_open) {
	}

	public long getId() {
		return id;
	}

	public StringBuffer toXML(String name, String projectPath) {
		StringBuffer buff = new StringBuffer();
		buff.append("<" + name +">\n");
		
			buff.append("<type>");
			buff.append(this.getClass().getName());
			buff.append("</type>\n");

			buff.append("<path>");
			String absPath = this.adapter.getDriver().getFilePath();
			String relPath = Utils.relPath(projectPath, absPath);
			if (relPath == null) relPath = absPath;
			buff.append(relPath);
			buff.append("</path>\n");
		
		buff.append("</" + name +">\n");
		return buff;
	}
	
	public String getFilePath() {
		return adapter.getDriver().getFilePath();
	}
	
	public void clean() {
		logger.debug("Cleaning layer: " + getName());
		try {
			adapter.getDriver().close();
			adapter.setDriver(null);
		} catch (IOException e) {
			logger.error("While closing raster driver: " + e.getMessage());
		}
		adapter = null;
	}

	public String getCustomForm() {
		// TODO Auto-generated method stub
		return null;
	}

	public void setCustomFormActive(boolean state) {
		// TODO Auto-generated method stub
		
	}

}
