/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap.layer;

import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.MapContext;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.proj.ICoordTrans;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
//import es.prodevelop.gvsig.mobile.fmap.util.xml.IPersistance;
//import es.prodevelop.gvsig.mobile.fmap.util.xml.XMLEntity;
//import es.prodevelop.gvsig.mobile.fmap.util.xml.XMLException;

/**
 * Implementacin de las caractersticas de alto nivel de las capas:
 * visibilidad, activacin, nombre, ...
 */
public abstract class FLyrDefault implements FLayer {
	
	private static Logger logger = Logger.getLogger(FLyrDefault.class);
	
	protected long id = 0;
	private boolean active = false;
	private boolean dirty = false;
	private boolean visible = false;
	private boolean infoable = true;
	private boolean available = true;
	private boolean inTOC = false;
	private boolean cacheDrawnLayers = false;
	private int numErrors = 0;
	private ArrayList errors = new ArrayList();
	protected boolean isEditing = false;
	private MapContext mContext = null;
	private String customForm = null;
	protected boolean hasCustomForm = false;
	

	private String name;

	private IProjection projection;

//	private boolean visible = true;
//
//	private boolean active;

	private int transparency = 0;

	private ICoordTrans ct;

	private double minScale = -1; // -1 indica que no se usa

	private double maxScale = -1;

//	private boolean isInTOC = true;

	protected ArrayList layerListeners = new ArrayList();

	private Hashtable properties = new Hashtable();

//	private boolean bCacheDrawnLayers;

	private BufferedImage cacheImageDrawnLayers = null;

	protected String memo = "";

//	private boolean bDirty;
	

//	private boolean available = true;

	//by default, all is active, visible and avalaible

	public Object getProperty(Object key) {
		return properties.get(key);
	}

	public void setProperty(Object key, Object val) {
		properties.put(key, val);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#setActive(boolean)
	 */
	public void setActive(boolean selected) {
		//active = selected;
		active = selected;
		callActivationChanged(LayerEvent.createActivationChangedEvent(this,
				"active"));
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#isActive()
	 */
	public boolean isActive() {
//		return active;
		return active;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#setName(java.lang.String)
	 */
	public void setName(String name) {
		this.name = name;
		callNameChanged(LayerEvent.createNameChangedEvent(this, "name"));
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getName()
	 */
	public String getName() {
		return name;
	}

	/*
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#load()
	 */
	public void load() throws FMapDriverException {
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#setVisible(boolean)
	 */
	public void setVisible(boolean visibility) {
//		visible = visibility;
		visible = visibility;
		setDirty(true);
		
		callVisibilityChanged(LayerEvent.createVisibilityChangedEvent(this,
				"visible"));
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#isVisible()
	 */
	public boolean isVisible() {
//		return visible && this.available;
		return (visible && available);
	}

	/**
	 * Devuelve la capa padre de la actual.
	 *
	 * @return FLayers padre.
	 */

	/**
	 * Inserta una proyeccin.
	 *
	 * @param proj
	 *            Proyeccin.
	 */
	public void setProjection(IProjection proj) {
		projection = proj;
	}

	/**
	 * @see org.cresques.geo.Projected#getProjection()
	 */
	public IProjection getProjection() {
		return projection;
	}

	/**
	 * @see org.cresques.geo.Projected#reProject(org.cresques.cts.ICoordTrans)
	 */
//	public void reProject(ICoordTrans arg0) {
//	}

	/**
	 * Devuelve el nivel de transparencia de la capa.
	 *
	 * @return Entero que representa el nivel de transparencia.
	 */
	public int getTransparency() {
		return transparency;
	}

	/**
	 * Inserta el nivel de transparencia de la capa.
	 *
	 * @param trans
	 *            Nivel de transparencia.
	 */
	public void setTransparency(int trans) {
		transparency = trans;
		setDirty(true);
	}

	/**
	 * Devuelve el XMLEntity a partir del objeto.
	 *
	 * @return XMLEntity.
	 * @throws XMLException
	 *//*
	public XMLEntity getXMLEntity() throws XMLException {
		XMLEntity xml = new XMLEntity();
		xml.putProperty("className", this.getClass().getName());

//		xml.putProperty("active", active);
		xml.putProperty("active", active);
		xml.putProperty("name", name);
		xml.putProperty("minScale", minScale);
		xml.putProperty("maxScale", maxScale);

		// TODO xml.addChild(parentLayer.getXMLEntity());
//		xml.putProperty("visible", visible);
		xml.putProperty("visible", visible);
		if (projection != null) {
			xml.putProperty("proj", projection.getAbrev());
		}
		xml.putProperty("transparency", transparency);
//		xml.putProperty("isInTOC", isInTOC);
		xml.putProperty("isInTOC", inTOC);

		// persist Properties hashTable

		Set keyset = properties.keySet();
		xml.putProperty("numProperties", keyset.size());

		Iterator keyitr = keyset.iterator();
	    while (keyitr.hasNext()) {
	      String propName = (String)keyitr.next();
	      Object obj = properties.get(propName);
	      if (obj instanceof IPersistance)
	      {
	    	  IPersistance persistObj = (IPersistance)obj;
	          XMLEntity xmlPropObj = persistObj.getXMLEntity();
              // make sure the node contains the class name
	          if (!xmlPropObj.contains("className")) {
	        	  try {
	        		  String propClassName = persistObj.getClassName();
		        	  // System.out.println("PROP CLASS NAME "+propClassName);
		        	  xmlPropObj.putProperty("className", propClassName);
	        	  } catch (Exception e) {
	        		  e.printStackTrace();
	        	  }
	          }
	          xmlPropObj.putProperty("layerPropertyName", propName);
	          xml.addChild(xmlPropObj);
	      }
	    }

		return xml;
	}*/

	/*
	 * Inserta los valores de los atributos del XMLEntity al objeto.
	 *
	 * @param xml XMLEntity.
	 *
	 * @throws XMLException @throws FMapDriverException @throws DriverIOException
	 *
	 * public void setXMLEntity03(XMLEntity xml) throws XMLException { active =
	 * xml.getBooleanProperty("active"); name = xml.getStringProperty("name");
	 * minScale=xml.getDoubleProperty("minScale");
	 * maxScale=xml.getDoubleProperty("maxScale"); visible =
	 * xml.getBooleanProperty("visible"); if (xml.contains("proj")) {
	 * setProjection(ProjectionPool.get(xml.getStringProperty("proj"))); } if
	 * (xml.contains("transparency")) transparency =
	 * xml.getIntProperty("transparency"); }
	 */

	/**
	 * Inserta los valores de los atributos del XMLEntity al objeto.
	 *
	 * @param xml
	 *            XMLEntity.
	 *
	 * @throws XMLException
	 * @throws FMapDriverException
	 * @throws DriverIOException
	 *//*
	public void setXMLEntity(XMLEntity xml) throws XMLException {
//		active = xml.getBooleanProperty("active");
		active = xml.getBooleanProperty("active");
		name = xml.getStringProperty("name");
		minScale = xml.getDoubleProperty("minScale");
		maxScale = xml.getDoubleProperty("maxScale");
//		visible = xml.getBooleanProperty("visible");
		visible = xml.getBooleanProperty("visible");
		if (xml.contains("proj")) {
			setProjection(CRSFactory.getCRS(xml.getStringProperty("proj")));
		}
		if (xml.contains("transparency"))
			transparency = xml.getIntProperty("transparency");
		if (xml.contains("isInTOC"))
//			isInTOC = xml.getBooleanProperty("isInTOC");
			inTOC = xml.getBooleanProperty("isInTOC");

        // recreate Properties hashTable

		if (xml.contains("numProperties")) {
			int numProps = xml.getIntProperty("numProperties");

            for (int iProp=0; iProp<numProps; iProp++) {
        	    XMLEntity xmlProp = xml.getChild(iProp);
        	    try {
        	    	String className = xmlProp.getStringProperty("className");
        	        Class classProp = Class.forName(className);
	                Object obj = classProp.newInstance();
	                IPersistance objPersist = (IPersistance)obj;
	                objPersist.setXMLEntity(xmlProp);
	                String propName = xmlProp.getStringProperty("layerPropertyName");
	                properties.put(propName, obj);
        		} catch (Exception e) {
        			continue;
        		}
       		// remove Properties children to avoid breaking layers' XML reading logic
        	for (iProp=0; iProp<numProps; iProp++)
                xml.removeChild(numProps-1-iProp);
            }
		}
	}*/

	/**
	 * Inserta los valores de los atributos del XMLEntity al objeto.
	 *
	 * @param xml
	 *            XMLEntity.
	 *
	 * @throws XMLException
	 * @throws FMapDriverException
	 * @throws DriverIOException
	 */
	

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getMapContext()
	 */
	public MapContext getMapContext() {
		return mContext;
	}
	
	public void  setMapContext(MapContext m) {
		mContext = m;;
	}

	/**
	 * Aade a la lista de listener un nuevo LayerListener.
	 *
	 * @param o
	 *            LayerListener.
	 *
	 * @return boolean.
	 */
	public boolean addLayerListener(LayerListener o) {
		if (layerListeners.contains(o))
			return false;
		return layerListeners.add(o);
	}
	public LayerListener[] getLayerListeners() {
		return (LayerListener[])layerListeners.toArray(new LayerListener[0]);
	}
	/**
	 * Borra de la lista de listeners el que se pasa como parmetro.
	 *
	 * @param o
	 *            LayerListener a borrar.
	 *
	 * @return True si ha sido correcto el borrado del Listener.
	 */
	public boolean removeLayerListener(LayerListener o) {
		return layerListeners.remove(o);
	}

	/**
	 * Llamada al metodo nameChanged de los Listeners dados de alta.
	 *
	 * @param e
	 *            LayerEvent.
	 */
	private void callNameChanged(LayerEvent e) {
		for (Iterator iter = layerListeners.iterator(); iter.hasNext();) {
			LayerListener listener = (LayerListener) iter.next();

			listener.nameChanged(e);
		}
	}

	/**
	 * Llamada al mtodo visibilityChanged de los Listeners.
	 *
	 * @param e
	 *            LayerEvent.
	 */
	private void callVisibilityChanged(LayerEvent e) {
		for (Iterator iter = layerListeners.iterator(); iter.hasNext();) {
			LayerListener listener = (LayerListener) iter.next();

			listener.visibilityChanged(e);
		}
	}

	/**
	 * Llamada al mtodo activationChanged de los Listener.
	 *
	 * @param e
	 *            LayerEvent.
	 */
	private void callActivationChanged(LayerEvent e) {
		for (Iterator iter = layerListeners.iterator(); iter.hasNext();) {
			LayerListener listener = (LayerListener) iter.next();

			listener.activationChanged(e);
		}
	}

	/**
	 * Devuelve las capas virtuales.
	 *
	 * @return FLayers.
	 */

	/**
	 * Inserta las capas virtuales.
	 *
	 * @param virtualLayers
	 *            FLayers.
	 */

	/**
	 * Devuelve la capa de texto.
	 *
	 * @return capa de texto.
	 */
	
	/**
	 * Inserta la capa de texto.
	 *
	 * @param layerText
	 *            Capa de texto.
	 */
	
	/**
	 * Inserta la Transformacin de coordenadas.
	 *
	 * @param ct
	 *            Transformacin de coordenadas.
	 */
	public void setCoordTrans(ICoordTrans ct) {
		ct = ct;
	}

	/**
	 * Devuelve las transformacin de coordenadas.
	 *
	 * @return ct.
	 */
	public ICoordTrans getCoordTrans() {
		return ct;
	}

	/**
	 * Mtodo que es llamado por Flayers para notificar a la capa que va a ser
	 * aadida. Esta previa notificacin es util para las capas que necesitan
	 * hacer algo antes de ser aadida. Por ejemplo, el raster necesita volver a
	 * abrir el fichero que ha podido ser cerrado con anterioridad. Si no se
	 * redefine este mtodo no se har nada ya que este es vacio.
	 */
	

	public double getMinScale() {
		return minScale;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#getMaxScale()
	 */
	public double getMaxScale() {
		return maxScale;
	}

	public void setMinScale(double minScale) {
		this.minScale = minScale;
	}

	public void setMaxScale(double maxScale) {
		this.maxScale = maxScale;
	}

	public boolean isWithinScale(double scale) {
		
		boolean bVisible = true;
		if (getMinScale() != -1) {
			if (scale < getMinScale())
				bVisible = false;
		}
		if (getMaxScale() != -1) {
			if (scale > getMaxScale())
				bVisible = false;
		}
		return bVisible;
	}

	public boolean isInTOC() {
		return inTOC;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#isCachingDrawnLayers()
	 */
	public boolean isCachingDrawnLayers() {
		return cacheDrawnLayers;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLayer#setCachingDrawnLayers(boolean)
	 */
	public void setCachingDrawnLayers(boolean bCacheDrawnLayers) {
//		this.bCacheDrawnLayers = bCacheDrawnLayers;
		cacheDrawnLayers = bCacheDrawnLayers;
		if (!cacheDrawnLayers)
			cacheImageDrawnLayers = null;
	}

	public BufferedImage getCacheImageDrawnLayers() {
		return cacheImageDrawnLayers;
	}

	public void setCacheImageDrawnLayers(BufferedImage cacheImageDrawnLayers) {
		this.cacheImageDrawnLayers = cacheImageDrawnLayers;
	}

	public boolean isDirty() {
		return dirty;
	}

	public void setDirty(boolean d) {
		dirty = d;
	}

	public boolean isAvailable() {
		return available;
	}

	public void setAvailable(boolean a) {
		available = a;
	}

	public void reload() throws FMapDriverException {
		setAvailable(true);
	}

	/*
	 * This stuff is to save error's info that causes
	 * unavailable status.
	 * */

	public boolean isOk(){
		return isOk();
	}

	public int getNumErrors(){
		return numErrors;
	}

	public FMapDriverException getError(int i){
		return (FMapDriverException) errors.get(i);
	}

	public List getErrors(){
		return errors;
	}

	public void addError(FMapDriverException error){
		errors.add(error);
	}

	public boolean visibleRequired() {
		return visible;
	}

	public String getInfoString() {
		return null;
	}
	
	public boolean isInfoable() {
		return infoable;
	}
	
	public void setInfoable(boolean inf) {
		infoable = inf;
	}
	
	public void setMemo(String m) {
		memo = m;
	}
	
	public String getMemo() {
		return memo;
	}

//	TODO Adding to edition
	public void setEditing(boolean b) throws Exception {
		isEditing = b;
		setDirty(true);
		setCachingDrawnLayers(b);
	}

	protected void callEditionChanged(LayerEvent e) {
		for (Iterator iter = layerListeners.iterator(); iter.hasNext();) {
			LayerListener listener = (LayerListener) iter.next();

			listener.editionChanged(e);
		}
	}

	public boolean isEditing(){
		return isEditing;
	}
	
	//Custom Form methods
	
	public boolean hasCustomForm(){
		return hasCustomForm;
	}
	
	public String getCustomForm(){
		return customForm; 
	}
	
	public void setCustomForm(String name){
		customForm=name;
		hasCustomForm=true;
	}
}
