/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap.layer;

import java.io.File;
import java.util.TreeMap;

import org.apache.log4j.Logger;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.driverManager.DriverManager;
import com.hardcode.driverManager.WriterManager;
import com.hardcode.gdbms.engine.data.DataSourceFactory;

import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialFileDriver;
import es.prodevelop.gvsig.mobile.fmap.legend.LegendFactory;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;

/**
 * Crea un adaptador del driver que se le pasa como parmetro en los mtodos
 * createLayer. Si hay memoria suficiente se crea un FLyrMemory que pasa todas
 * las features del driver a memoria
 */
public class LayerFactory {
	private static Logger logger = Logger.getLogger(LayerFactory.class
			.getName());

	private static String driversPath = "../FMap 03/drivers";

	private static String writersPath = "../FMap 03/drivers";

	private static DriverManager driverManager;

	private static WriterManager writerManager;

	private static DataSourceFactory dataSourceFactory;

	/**
	 * Map en el que se guarda para cada fuente de datos aadida al sistema, el
	 * adaptador que la maneja. Ha de ser un TreeMap ya que esta clase define la
	 * igualdad entre las claves a traves del mtodo equals de las mismas. Los
	 * objetos FileSource, DBSource y WFSSource tienen definido el mtodo equals
	 * de forma que devuelven true cuando dos objetos apuntan a la misma fuente
	 * de datos
	 */
	private static TreeMap sourceAdapter;

	/*
	 * Crea un RandomVectorialFile con el driver que se le pasa como parmetro y
	 * guardndose el nombre del fichero para realizar los accesos, la capa
	 * tendr asociada la proyeccin que se pasa como parametro tambin
	 * 
	 * @param layerName Nombre de la capa. @param driverName Nombre del driver.
	 * @param f fichero. @param proj Proyeccin.
	 * 
	 * @return FLayer. @throws FMapDriverException
	 * 
	 * @throws FMapDriverException @throws DriverIOException
	 */
	public static FLayer createLayer(String layerName, String driverName,
			File f, IProjection proj) throws FMapDriverException {
		// Se obtiene el driver que lee
		DriverManager dm = getDM();

		try {
			Driver d = dm.getDriver(driverName);

			if (d instanceof VectorialFileDriver) {
				return createLayer(layerName, (VectorialFileDriver) d, f, proj);
			}
		} catch (DriverLoadException e) {
			FMapDriverException exception = 
				new FMapDriverException("While creating layer");
			throw exception;
		}

		return null;
	}

	/**
	 * It creates a FLayer (FLyrVect) which reads its data from a file driver,
	 * projected in the specified projection.
	 * 
	 * @param layerName
	 *            name of the layer
	 * @param d
	 *            vectorial file driver to read layer's data
	 * @param f
	 *            file associated to the driver
	 * @param proj
	 *            layer projection
	 * 
	 * @return FLayer new vectorial layer
	 * 
	 * @throws FMapDriverException
	 */
	public static FLayer createLayer(String layerName, VectorialFileDriver d,
			File f, IProjection proj) 
	
	/*throws FMapDriverException*/ {

		// TODO Comprobar si hay un adaptador ya
		VectorialFileAdapter adapter = new VectorialFileAdapter(f);
		adapter.setDriver((VectorialDriver) d);

		FLyrVect capa = new FLyrVect();
		capa.setName(layerName);

		// TODO Meter esto dentro de la comprobacin de si hay memoria
		if (false) {
		} else {
			capa.setSource(adapter);
			capa.setProjection(proj);
		}
		capa.setLegend(LegendFactory.createSingleSymbolLegend());

		return capa;
	}


	/**
	 * Crea una capa Raster a partir del nombre driver, fichero y proyeccin.
	 * 
	 * @param layerName
	 *            Nombre de la capa.
	 * @param d
	 *            RasterDriver.
	 * @param f
	 *            Fichero.
	 * @param proj
	 *            Proyeccin.
	 * 
	 * @return Nueva capa de tipo raster.
	 * 
	 * @throws DriverIOException
	 */
	

	/**
	 * Crea una capa Raster a partir del nombre driver, fichero y proyeccin y
	 * coordenadas de georeferenciacin. Esta funcin es para georeferenciar
	 * capas raster. Para imagenes que no tienen georeferenciacin hay que
	 * asignarle una temporal, normalmente a partir de la vista activa.
	 * 
	 * @param layerName
	 *            Nombre de la capa.
	 * @param d
	 *            RasterDriver.
	 * @param f
	 *            Fichero.
	 * @param proj
	 *            Proyeccin.
	 * @param extent
	 *            Extent de la vista activa
	 * 
	 * @return Nueva capa de tipo raster.
	 * 
	 * @throws DriverIOException
	 */
	

	
	/**
	 * Crea un RandomVectorialWFS con el driver que se le pasa como parmetro y
	 * guardndose la URL del servidor que se pasa como parmetro
	 * 
	 * @param driver
	 * @param host
	 * @param port
	 * @param user
	 * @param password
	 * @param dbName
	 * @param tableName
	 * @param proj
	 * 
	 * @return Capa creada.
	 * 
	 * @throws UnsupportedOperationException
	 */
	

	
	/**
	 * @param driver
	 * @param layerName
	 * @param object
	 * @return
	 * @throws SQLException
	 * @throws DriverIOException
	 * @throws IOException
	 * @throws DriverLoadException
	 * @throws com.hardcode.gdbms.engine.data.driver.DriverException
	 * @throws NoSuchTableException
	 * @throws ClassNotFoundException
	 * @throws
	 */
	
	/**
	 * Crea una FLyrComplexRaster que ataca al driver que se pasa como
	 * parmetro.
	 * 
	 * @param driver
	 * @param f
	 * @param proj
	 * 
	 * @throws IllegalArgumentException
	 *             Si se pasa un driver que no implementa
	 *             GeorreferencedRasterDriver o NotGeorreferencedRasterDriver
	 */
	

	/**
	 * Devuelve el DriverManager.
	 * 
	 * @return DriverManager.
	 */
	public static DriverManager getDM() {
		initializeDriverManager();

		return driverManager;
	}

	/**
	 * Devuelve el WriterManager.
	 * 
	 * @return WriterManager.
	 */
	public static WriterManager getWM() {
		initializeWriterManager();

		return writerManager;
	}

	/**
	 * Inicializa el DriverManager.
	 */
	private static void initializeDriverManager() {
		if (driverManager == null) {
			driverManager = new DriverManager();
			driverManager.loadDrivers(new File(LayerFactory.driversPath));

			Throwable[] failures = driverManager.getLoadFailures();

			for (int i = 0; i < failures.length; i++) {
				logger.error("", failures[i]);
			}

			getDataSourceFactory().setDriverManager(driverManager);
			getDataSourceFactory().initialize();
		}
	}

	/**
	 * Inicializa el DriverManager.
	 */
	private static void initializeWriterManager() {
		if (writerManager == null) {
			writerManager = new WriterManager();
			writerManager.loadWriters(new File(LayerFactory.writersPath));

			Throwable[] failures = writerManager.getLoadFailures();

			for (int i = 0; i < failures.length; i++) {
				logger.error("", failures[i]);
			}

			getDataSourceFactory().setWriterManager(writerManager);
			getDataSourceFactory().initialize();
			// QueryManager.registerQuery(new ArcJoin());
		}
	}

	/**
	 * sets drivers Directory
	 * 
	 * @param path
	 */
	public static void setDriversPath(String path) {
		LayerFactory.driversPath = path;
		initializeDriverManager();
	}

	/**
	 * sets writers Directory
	 * 
	 * @param path
	 */
	public static void setWritersPath(String path) {
		LayerFactory.writersPath = path;
		initializeWriterManager();
	}

	/**
	 * @return Returns the dataSourceFactory.
	 */
	public static DataSourceFactory getDataSourceFactory() {
		if (dataSourceFactory == null) {
			dataSourceFactory = new DataSourceFactory();
		}
		return dataSourceFactory;
	}
}
