/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.proj;

/**
 * Basic interface for any projection class.
 *  
 * @author jldominguez
 *
 */
public interface IProjection {

	/**
	 * Computes the current scale
	 * 
	 * @param minX min x value (map units)
	 * @param maxX max x value (map units)
	 * @param width map width in pixels
	 * @param dpi dots per inch
	 * @return the current scale (return 1000 if current scale is 1:1000)
	 */
	double getScale(double minX, double maxX, double width, double dpi);
	
	/**
	 * 
	 * @return the projection EPSG abbreviation
	 */
	String getAbrev();
	
	/**
	 * 
	 * @return the abbreviation of the unit used ("m", "", etc)
	 */
	String getUnitsAbbrev();
	
	/**
	 * 
	 * @return the length of 1 map unit.
	 */
	double getMetersPerProjUnit();

}
