/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.util.download;

import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.URL;
import java.nio.ByteBuffer;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;

/**
 * This class is used to download resources in a separate thread, so it can be cancelled.
 * 
 * @see es.prodevelop.gvsig.mobile.fmap.util.download.Downloader
 * 
 * @author jldominguez
 *
 */
public class DownloadTask implements Runnable {
	
	private static Logger logger = Logger.getLogger(DownloadTask.class);
	
	public static final int BYTE_ARRAY = 0;
	public static final int FILE = 1;
	
	private URL url = null;
	private String ext = "tmp";
	private File destFile = null;
	
	private Object returned_obj = null;
	private int down_type = FILE;
	
	private boolean go_on = true;
	private boolean done = false;
	
	/**
	 * 
	 * @return whether the download is finished
	 */
	public boolean isDone() {
		return done;
	}
	
	/**
	 * Constructor
	 * @param u the URL
	 * @param exte the local file extension
	 * @param _type whether it is a file download or an array of bytes
	 * @param dest_file destination file
	 */
	public DownloadTask(URL u, String exte, int _type, File dest_file) {
		url = u;
		ext = exte;
		down_type = _type;
		destFile = dest_file;
	}
	
	/**
	 * Switches the state to cancelled, so the download should be cancelled
	 *
	 */
	public void cancel() {
		go_on = false;
	}

	/**
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		
		// ---------------------------------------- bytes
		if (down_type == BYTE_ARRAY) {
			
			ArrayList byteArraysList = new ArrayList(); 

			try {
				
				byte[] buffer = new byte[1024*4];
				
				logger.debug("Opening stream for: " + url.toString());
				
				DataInputStream is = new DataInputStream(url.openStream());
				int full_size = 0;

				for (int i = is.read(buffer); i>0; i = is.read(buffer)) {
					
					logger.debug("Read: " + i + " bytes.");
					
					if (!go_on) {
						returned_obj = null;
						return;
					}
					
					byteArraysList.add(new Integer(i));
					byteArraysList.add(buffer.clone());
					full_size = full_size + i;

					if (!go_on) {
						returned_obj = null;
						return;
					}
				}
				
				byteArraysList.add(new Integer(full_size));
				
			} catch (Exception e) {
				logger.error("While downloading: " + e.getMessage());
				returned_obj = null;
				done = true;
				return;
			}
			
			returned_obj = mergeBytes(byteArraysList);
			
			byteArraysList.clear();
			byteArraysList = null;
			
			done = true;
			return;
		}
		// ---------------------------------------- file
		if (down_type == FILE) {
			
			logger.debug("Downloading to FILE this url: " + url.toString());
			logger.debug("Destination path is: " + destFile.getAbsolutePath());
			
			DataOutputStream dos;
			
			if (!go_on) {
				returned_obj = null;
				return;
			}

			try {
				logger.debug("Getting file: File does not exist...");
				if (!go_on) {
					returned_obj = null;
					return;
				}

				if (!destFile.getParentFile().exists()) {
					logger.debug("Parent did not exist!");
					destFile.getParentFile().mkdirs();
				} else {
					logger.debug("Parent existed!");
				}
				destFile.createNewFile();

				dos = new DataOutputStream( new BufferedOutputStream(new FileOutputStream(destFile)));
				byte[] buffer = new byte[1024*4];
				// ByteBuffer bff;
				
				logger.debug("Opening stream...");
				InputStream inp_str = url.openStream();
				logger.debug("Opening stream... done");
				
				DataInputStream is = new DataInputStream(inp_str);
				
				if (!go_on) {
					returned_obj = null;
					return;
				}

				long read = 0;
				
				for (int i = is.read(buffer); i>0; i = is.read(buffer)) {
					
					if (!go_on) {
						returned_obj = null;
						return;
					}

					dos.write(buffer, 0, i);
					read += i;
					
					if (!go_on) {
						returned_obj = null;
						return;
					}
				}
				
				inp_str.close();
				dos.close(); is.close();
				is = null; dos = null;
				inp_str = null;

			} catch (Exception e) {
				logger.error("While downloading: " + e.getMessage());
				returned_obj = null;
				done = true;
				return;
			}
			
			returned_obj = destFile;		
			done = true;
			return;
			
		}
		
		returned_obj = null;
		done = true;
		
		
	}
	
	/**
	 * 
	 * @return the result object (file or byte array)
	 */
	public Object getDownloadedObject() {
		return returned_obj;
	}
	
	private byte[] mergeBytes(ArrayList byteArraysList) {
		
		int array_size = byteArraysList.size(); 
		int total_size =  ((Integer) byteArraysList.get(array_size - 1)).intValue();
		int n = array_size / 2;
		
		int item_size = 0;
		byte[] item_bytes = null;

		byte[] resp = new byte[total_size];

		if (! go_on) {
			return null;
		}

		int index = 0;
		for (int i=0; i<n; i++) {
			
			if (! go_on) {
				return null;
			}

			item_size = ((Integer) byteArraysList.get(2 * i)).intValue();
			item_bytes = (byte[]) byteArraysList.get(2 * i + 1);
			System.arraycopy(item_bytes, 0, resp, index, item_size);
			index = index + item_size;
			
			if (! go_on) {
				return null;
			}
			
		}
		item_bytes = null;
		return resp;
	}	

}
