/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.layer;

import org.gvsig.gpe.parser.GPEErrorHandler;
import org.gvsig.xmlschema.som.IXSSchemaDocument;

import es.prodevelop.gvsig.mobile.fmap.MapContext;
import es.prodevelop.gvsig.mobile.fmap.layer.VectorialAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.VectorialDefaultAdapter;
import es.prodevelop.gvsig.mobile.fmap.proj.CRSFactory;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.gpe.fmap.DriverGpeFactory;
import es.prodevelop.gvsig.mobile.gpe.fmap.VectorialGpeDriver;

/********************************************************
 * LayerGpeFactory.										*
 * ----------------										*
 * Factory that creates GPE Layers						*
 *														* 
 * PRODEVELOP S.L. 2008									*
 * @author Carlos Snchez Perin (csanchez)			*
 ********************************************************/

public class LayerGpeFactory {
	//--------------------
	// STATIC CONSTRUCTOR
	//--------------------
	//------------------------------------
	// MapControl and errorHandler needed.
	// The parentLayer, name and srs are
	// needed too.
	//------------------------------------
	public static FlyrGpeVect createVectorialLayer(MapContext mapContext,
			String name,
			String srs,
			GPEErrorHandler errorHandler,
			ILayerGpe parentLayer){
		return createVectorialLayer(mapContext, name, srs,
				null, null, errorHandler, parentLayer);
	}
	
	//------------------------------
	// SECONDARY STATIC CONSTRUCTOR
	//------------------------------
	//------------------------------------
	// MapControl and errorHandler needed.
	// The parentLayer, name and srs are
	// needed too.
	// This needs the Schema.
	//------------------------------------
	public static FlyrGpeVect createVectorialLayer(MapContext mapContext, String layerName,
			String srs, IXSSchemaDocument schemaDocument, String xsElementName, GPEErrorHandler errorHandler,
			ILayerGpe parentLayer){
		IProjection projection = null;
		if (srs != null){
			projection = CRSFactory.getCRS(srs);
		}
		if (projection == null){
			projection = mapContext.getProjection();
		}
		FlyrGpeVect layer = new FlyrGpeVect(mapContext, errorHandler, parentLayer);		
		if (layerName == null){
			layer.setName(layer.getDefaultLayerName());
		}else{
			layer.setName(layerName);
		}
		VectorialGpeDriver driver = DriverGpeFactory.createVectorialDriver(mapContext,
				schemaDocument,
				xsElementName);
		layer.setDriver(driver);
		layer.setProjection(projection);
		VectorialAdapter adapter = new VectorialDefaultAdapter();
		adapter.setDriver(driver);
		layer.setSource(adapter);	
		return layer;		
	}	
}
