/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.model;

import java.awt.geom.Rectangle2D;
import java.io.IOException;

import org.gvsig.gpe.parser.ICoordinateIterator;

/********************************************************
 * GPEBOX.												
 * -------												
 * Class to implement the libGPE BoundingBox for	 	
 * gvsig mobile.									
 * Only fill the content events needed.					
 * 														
 * PRODEVELOP 2008										
 * @author Carlos Snchez (csanchez@prodevelop.es)		
 ********************************************************/
public class GPEBBox {
	private String id = null;
	private double[] x;
	private double[] y;
	private double[] z;
	private String srs;
	
	//------------
	//CONSTRUCTOR
	//------------
	public GPEBBox(String id, ICoordinateIterator coords, String srs) {
		super();
		this.id = id;
		coords.getDimension();
		double[] min = new double[coords.getDimension()];
		double[] max = new double[coords.getDimension()];
	    x = new double[coords.getDimension()];
		y = new double[coords.getDimension()];
		z = new double[coords.getDimension()];
		try {
			if (coords.hasNext()){
				coords.next(min);
				if (coords.hasNext()){
					coords.next(max);
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		x[0]=min[0];	
		x[1]=max[0];
		y[0]=min[1];
		y[1]=max[1];	
		if (coords.getDimension()==3){
			z[0]=min[2];
			z[1]=max[2];
		}
		else{
			z[0]=0.0;
			z[1]=0.0;
		}
		this.srs = srs;
	}
	
	//-------------------------
	//Return a rectangle in 2D.
	//-------------------------
	public Rectangle2D getBbox2D(){
		return new Rectangle2D.Double(x[0],y[0],x[1]-x[0],y[1]-y[0]);
	}
	
	//-------------------------
	//Return the identifier.
	//-------------------------
	public String getId() {
		return id;
	}
	
	//-------------------------
	// Set the identifier.
	//-------------------------
	public void setId(String id) {
		this.id = id;
	}
	
	//----------------------------
	//Get the System of Reference.
	//----------------------------
	public String getSrs() {
		return srs;
	}
	
	//----------------------------
	//Set the System of Reference.
	//----------------------------
	public void setSrs(String srs) {
		this.srs = srs;
	}
	
	//-------------------------
	//Return the X coordinate.
	//-------------------------
	public double[] getX() {
		return x;
	}
	
	//-------------------------
	//Set the X coordinate.
	//-------------------------
	public void setX(double[] x) {
		this.x = x;
	}
	
	//-------------------------
	//Return the Y coordinate.
	//-------------------------
	public double[] getY() {
		return y;
	}
	
	//-------------------------
	//Set the Y coordinate.
	//-------------------------
	public void setY(double[] y) {
		this.y = y;
	}
	
	//-------------------------
	//Return the Z coordinate.
	//-------------------------
	public double[] getZ() {
		return z;
	}
	
	//-------------------------
	//Set the Z coordinate.
	//-------------------------
	public void setZ(double[] z) {
		this.z = z;
	}
}
