/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.model;

import java.util.ArrayList;

import es.prodevelop.gvsig.mobile.fmap.core.FPolyline2D;
import es.prodevelop.gvsig.mobile.fmap.core.GeneralPathX;
import es.prodevelop.gvsig.mobile.fmap.core.Handler;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.ShapeFactory;

/********************************************************
 * GPECURVE.											*
 * -----------											*
 * Class to implement the libGPE Curve for			 	*
 * gvsig mobile.										*
 * Only fill the content events needed.					*
 * 														* 
 * PRODEVELOP 2008										*
 * @author Carlos Snchez (csanchez@prodevelop.es)		*
 ********************************************************/
public class GPECurve extends GPEGeometry{
	private ArrayList segments = null;
	
	public GPECurve(String id, IGeometry geometry, String srs) {
		super(id, geometry, srs);		
		segments = new ArrayList();
	}
	
	public GPECurve(String id, String srs) {
		this(id, null, srs);		
	}
	
	/**
	 * Adds a segment 
	 * @param segment
	 * The segment to add
	 */
	public void addSegment(GPEGeometry segment){
		segments.add(segment);
	}
	
	/**
	 * Gets a segment
	 * @param i
	 * The segment position
	 * @return
	 * A Segment
	 */
	public GPEGeometry getSegmentAt(int i){
		return (GPECurve) segments.get(i);
	}
	
	/**
	 * @return the number of seg
	 */
	public int getSegmentsSize(){
		return segments.size();
	}
	
	/**
	 * @return the geometry
	 */
	public IGeometry getGeometry() {
		IGeometry geom = super.getGeometry();
		if (geom != null){
			return geom;
		}
		GeneralPathX gpx = new GeneralPathX();
		int zLength = 0;
		boolean startPoint = true;
		for (int i=0 ; i<segments.size() ; i++){
			IGeometry segment = ((GPEGeometry) segments.get(i)).getGeometry();
			Handler[] handlers = ((FPolyline2D) segment.getInternalShape()).getSelectHandlers();
			zLength = zLength + handlers.length;
			for (int j=0 ; j<handlers.length ; j++){
				if (startPoint){
					gpx.moveTo(handlers[j].getPoint().getX(),
							handlers[j].getPoint().getY());
					startPoint = false;
				}else{
					gpx.lineTo(handlers[j].getPoint().getX(),
							handlers[j].getPoint().getY());
				}
			}
		}
	geometry = ShapeFactory.createPolyline2D(gpx);
		return geometry;
	}	

}
