/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.model;

import java.awt.geom.PathIterator;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import es.prodevelop.gvsig.mobile.fmap.core.FGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolyline2D;
import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.core.GeneralPathX;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.ShapeFactory;

/********************************************************
 * GPEGEOMETRY.											*
 * ------------											*
 * Class to implement the libGPE geometry for		 	*
 * gvsig mobile.										*
 * Only fill the content events needed.					*
 * 														* 
 * PRODEVELOP 2008										*
 * @author Carlos Snchez (csanchez@prodevelop.es)		*
 ********************************************************/

public class GPEGeometry{
	protected String srs = null;
	protected String id = null;
	protected IGeometry geometry = null;
	protected ArrayList subGeometryList = null;
	protected Rectangle2D bounds = null;
	
	//This geometri is reprojected from the original
	//CRS to the layer CRS.
	private IGeometry reprojectedGeometry = null;
			
	public GPEGeometry(String id, IGeometry geometry, String srs) {
		super();
		this.geometry = geometry;
		this.srs = srs;
		this.id = id;
		this.subGeometryList = new ArrayList();
	}

	/**
	 * @return the geometry
	 */
	public IGeometry getGeometry() {
		return geometry;
	}
	/**
	 * @param the geometry
	 */
	public void setGeometry(IGeometry geom) {
		geometry=geom;
	}
	/**
	 * @return the srs
	 */
	public String getSrs() {
		return srs;
	}
	
	/**
	 * @return the id
	 */
	public String getId() {
		return id;
	}

	/**
	 * @return the reprojectedGeometry
	 */
	public IGeometry getReprojectedGeometry() {
		return reprojectedGeometry;
	}

	/**
	 * @param reprojectedGeometry the reprojectedGeometry to set
	 */
	public void setReprojectedGeometry(IGeometry reprojectedGeometry) {
		this.reprojectedGeometry = reprojectedGeometry;
	}

	/**
	 * @return the bounding box
	 */
	public Rectangle2D getShapeBounds() {
		Rectangle2D subBound = null;
		if (geometry!=null){	
			if (isMultiGeometry()){
				bounds = geometry.getBounds2D();
				for (int i=0; i<getGeometriesSize();i++){
					GPEGeometry subgeometry = getGeometryAt(i);
					if (subgeometry.isMultiGeometry())
						subBound = subgeometry.getShapeBounds();
					else
						subBound = subgeometry.getGeometry().getBounds2D();
					bounds.add(subBound);
				}
			}
			else{
				bounds = geometry.getBounds2D();
			}	
		}
		return bounds;
	}
	
	
	/**
	 * Gets one geometry
	 * @param i
	 * Geometry position
	 * @return
	 * A Geometry
	 */
	public GPEGeometry getGeometryAt(int i){
		return (GPEGeometry) subGeometryList.get(i);
	}
	
	/**
	 * @return the number of geometries
	 */
	public int getGeometriesSize(){
		return subGeometryList.size();
	}
	
	/**
	 * Adds a new geometry
	 * @param geometry
	 * The geometry to add
	 */
	public void addGeometry(GPEGeometry geometry){
		if (bounds == null){
			bounds = geometry.getShapeBounds();
		}else{
			bounds.add(geometry.getShapeBounds());
		}
		subGeometryList.add(geometry);
	}
	
	
	public boolean isMultiGeometry(){
		if (subGeometryList.size()==0)
			return false;
		return true;
	}

	/**
	 * Composition of geometries from sub-features
	 * @param subgeom2 
	 * @param geometry
	 * The geometry to add
	 */
	public void ComposeGeometry(GPEGeometry mainGeometry, GPEGeometry subGeometry) {
		int type1 = mainGeometry.getGeometry().getGeometryType();
		int type2 = subGeometry.getGeometry().getGeometryType();
		if((type1==FShape.POINT)&&(type2==FShape.POINT)){
			FPoint2D point1; 
			FPoint2D point2;
			point1=(FPoint2D)((FGeometry)mainGeometry.getGeometry()).getInternalShape();
			point2=(FPoint2D)((FGeometry)subGeometry.getGeometry()).getInternalShape();
			double x1 = point1.getX();
			double y1 = point1.getY();
			double x2 = point2.getX();
			double y2 = point2.getY();
			GeneralPathX gp = new GeneralPathX();
			gp.moveTo(x1, y1);
			gp.lineTo(x2, y2);
			geometry = ShapeFactory.createPolyline2D(gp);
		}
		else if((type1==FShape.LINE)&&(type2==FShape.POINT)){
			int i=0;
			GeneralPathX gp = new GeneralPathX();
			FPoint2D point = (FPoint2D)((FGeometry)subGeometry.getGeometry()).getInternalShape();
			FPolyline2D line = (FPolyline2D)((FGeometry)mainGeometry.getGeometry()).getInternalShape();
			PathIterator it = line.getPathIterator(null);
			while (!it.isDone()){
				double[] coords = new double[2];
				it.currentSegment(coords);
				if (i == 0)
					gp.moveTo(coords[0],coords[1]);
				else
					gp.lineTo(coords[0],coords[1]);
				i++;
				it.next();
			}	
			double x = point.getX();
			double y = point.getY();
			gp.lineTo(x,y);
			geometry = ShapeFactory.createPolyline2D(gp);
		}
	}
}
