package org.gvsig.gpe.gpx.parser;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

import org.gvsig.exceptions.BaseException;
import org.gvsig.gpe.gpx.exceptions.GpxException;
import org.gvsig.gpe.gpx.exceptions.GpxHeaderParseException;
import org.gvsig.gpe.gpx.parser.profiles.IBindingProfile;
import org.gvsig.gpe.xml.parser.GPEXmlParser;
import org.gvsig.gpe.xml.stream.XmlStreamException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/

/**
 * This is a GPX parser. This class must be registered in 
 * the GPE register to read and write GPX files.
 */
public abstract class GPEDefaultGpxParser extends GPEXmlParser{

private IBindingProfile profile = null;
	
	/**
	 * @return the profile
	 */
	public IBindingProfile getProfile() {
		return profile;
	}

	/**
	 * @param profile the profile to set
	 */
	public void setProfile(IBindingProfile profile) {
		this.profile = profile;
	}
	
	public GPEDefaultGpxParser() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.GPEParser#accept(java.io.File)
	 */
	public boolean accept(URI uri) {
		if ((uri.getPath().toUpperCase().endsWith("GPX"))){
			return true;
		}
		return false;		
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.GPEParser#getFormats()
	 */
	public String[] getFormats() {
		String[] formats = new String[1];
		formats[0] = "GPX";		
		return formats;
	}	

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.GPEParser#getVersions()
	 */
	public String[] getVersions() {
		String[] versions = new String[0];
		versions[0] = "1.1";
		return versions;

	}

	/**
	 * It creates an InputStream. The Gpx file can have the
	 * GPX extension
	 * @return
	 * @throws GpxException
	 */
	protected InputStream createInputStream(File file){		
		//GPX
		try{
			return new FileInputStream(file);
		} catch (FileNotFoundException e) {
			getErrorHandler().addError(e);
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.xml.GPEXmlParser#initParse()
	 */
	protected void initParse() {
		try {
			String namespace = getProfile().getHeaderBinding().parse(getParser(),this);			
			AbstractGpxParser parser = new GpxParsersFactory().createParser(namespace,getParser(), this);
			parser.initParse();
		}catch (XmlStreamException e){
			getErrorHandler().addError(e);		
		} catch (GpxHeaderParseException e) {
			getErrorHandler().addError(e);
		} catch (BaseException e) {
			getErrorHandler().addError(e);
		}		
	}

}
