package org.gvsig.gpe.gpx.parser;

import java.io.IOException;
import javax.xml.namespace.QName;

import org.gvsig.gpe.gpx.exceptions.GpxBodyParseException;
import org.gvsig.gpe.gpx.utils.GpxTags;
import org.gvsig.gpe.xml.stream.IXmlStreamReader;
import org.gvsig.gpe.xml.stream.XmlStreamException;
import org.gvsig.gpe.xml.utils.CompareUtils;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/

/**
 * This is a parser for GPX 1.1
 */
public class GpxParserV1_1 extends AbstractGpxParser{
	private Object layer = null;	
		
	public GpxParserV1_1(IXmlStreamReader parser, GPEDefaultGpxParser handler) {
		super(parser, handler);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.gmlEngine.IFeaturesParser#getVersion()
	 */
	public String getVersion() {
		return "1.0";
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.kml.versions.AbstractKmlParser#initParse()
	 */
	public void initParse() throws GpxBodyParseException {
		boolean endFeature = false;
		int currentTag;		
		
		String id = null;
		String prefix = null;
		
		if (layer == null){
			layer = getHandler().getContentHandler().startLayer(id, GpxTags.NAMESPACE, prefix, null, GpxTags.DEFAULT_SRS, null, null);
		}	

		try {
			QName tag = getParser().getName();
			currentTag = getParser().getEventType();

			while (!endFeature){
				switch(currentTag){
				case IXmlStreamReader.START_ELEMENT:
					if (CompareUtils.compareWithOutNamespace(tag, GpxTags.WPT)){
						Object feature = getHandler().getProfile().getWayPointBinding().parse(getParser(), getHandler());
						getHandler().getContentHandler().addFeatureToLayer(feature, layer);
					}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.RTE)){
						Object feature = getHandler().getProfile().getRouteBinding().parse(getParser(), getHandler());
						getHandler().getContentHandler().addFeatureToLayer(feature, layer);
					}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.TRK)){						
						Object feature = getHandler().getProfile().getTrackBinding().parse(getParser(), getHandler());
						getHandler().getContentHandler().addFeatureToLayer(feature, layer);						
					}else if (CompareUtils.compareWithOutNamespace(tag,GpxTags.EXTENSIONS)){						
						getHandler().getProfile().getElementBinding().parse(getParser(), getHandler(), null, null);						
					}else if (CompareUtils.compareWithOutNamespace(tag,GpxTags.METADATA)){
						getParser().next();
						Object feature = getHandler().getProfile().getMetadataBinding().parse(getParser(), getHandler(), layer);
						getHandler().getContentHandler().addFeatureToLayer(feature, layer);
					}
					
					break;
				case IXmlStreamReader.END_ELEMENT:
					if (CompareUtils.compareWithOutNamespace(tag, GpxTags.ROOT)){						
						endFeature = true;						
					}
				break;
					
				case IXmlStreamReader.END_DOCUMENT:
					endFeature = true;
					
					break;
				}
				if (!endFeature){					
					currentTag = getParser().next();
					tag = getParser().getName();
				}
			}
			if (layer != null){
				getHandler().getContentHandler().endLayer(layer);
			}
		} catch (XmlStreamException e) {
			throw new GpxBodyParseException (e);
		} catch (IOException e) {
			throw new GpxBodyParseException (e);
		}		
	}
}
