package org.gvsig.gpe.gpx.parser;

import java.util.Hashtable;

import javax.xml.namespace.QName;

import org.gvsig.exceptions.BaseException;
import org.gvsig.gpe.gpx.exceptions.GpxImpossibleCreateParser;
import org.gvsig.gpe.gpx.utils.GpxTags;
import org.gvsig.gpe.xml.stream.IXmlStreamReader;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/

/**
 * This class is used to register the differnet gpx 
 * parsers
 */
public class GpxParsersFactory {
	
private static Hashtable supportedVersions = new Hashtable();	
	
	static {
		supportedVersions.put(GpxTags.NAMESPACE,GpxParserV1_1.class);	
		supportedVersions.put(GpxTags.UNKNOWN_VERSION,GpxParserV1_1.class);		
	}	
	
	/**
	 * To create a new GPX parser
	 * @param nameSpace
	 * GPX namespace used to determine the parser to use
	 * @param parser
	 * XML parser used to parse the XML
	 * @param handler
	 * GPE parser that contains the handlers
	 * @return
	 * A GPX parser
	 * @throws BaseException
	 */
	public AbstractGpxParser createParser(String nameSpace,IXmlStreamReader parser, GPEDefaultGpxParser handler) throws BaseException{
		Class clazz = (Class) supportedVersions.get(nameSpace);
		if (clazz == null){
			clazz = (Class) supportedVersions.get(GpxTags.UNKNOWN_VERSION);
		}
		if (clazz != null){
			Class [] args = {IXmlStreamReader.class,GPEDefaultGpxParser.class};
			Object [] params = {parser,handler};
			try {
				return (AbstractGpxParser)clazz.getConstructor(args).newInstance(params);
			} catch (Exception e) {
				throw new GpxImpossibleCreateParser(nameSpace,e);
			}
		}
		return null ;		
	}

}
