package org.gvsig.gpe.gpx.parser.v11.features;

import java.io.IOException;

import javax.xml.namespace.QName;

import org.gvsig.gpe.gpx.parser.GPEDefaultGpxParser;
import org.gvsig.gpe.gpx.utils.GPXUtilsParser;
import org.gvsig.gpe.gpx.utils.GpxTags;
import org.gvsig.gpe.xml.stream.IXmlStreamReader;
import org.gvsig.gpe.xml.stream.XmlStreamException;
import org.gvsig.gpe.xml.utils.CompareUtils;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * It parses an element. An element is a tag inside of a 
 * metadata tag
 */
public class ElementBinding {
	/**
	 * It parses a xml element
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @param parentElement
	 * The parent element
	 * @return
	 * A Element
	 * @throws XmlStreamException
	 * @throws IOException
	 */
	public  Object parse(IXmlStreamReader parser,GPEDefaultGpxParser handler, Object feature, Object parentElement) throws XmlStreamException, IOException {
		boolean endFeature = false;
		int currentTag;		
		Object element = null;	
		Object value = null;
		boolean isInitialized = false;
		//Used to finish to parse the current element
		QName elementRootType = parser.getName();

		String type = getType(elementRootType.getLocalPart());

		QName tag = parser.getName();
		currentTag = parser.getEventType();

		while (!endFeature){
			switch(currentTag){
			case IXmlStreamReader.START_ELEMENT:
				if (!(CompareUtils.compareWithOutNamespace(tag,elementRootType))){
					if (!isInitialized){						
						element = handler.getContentHandler().startElement(elementRootType.getNamespaceURI(),
								GPXUtilsParser.removeBlancSymbol(elementRootType), 
								null,
								parentElement);
						isInitialized = true;
					}					
					handler.getProfile().getElementBinding().parse(parser, handler, feature, element);
				}

				break;
			case IXmlStreamReader.END_ELEMENT:
				if (CompareUtils.compareWithOutNamespace(tag,elementRootType)){						
					endFeature = true;					
					if (!isInitialized){
						element = handler.getContentHandler().startElement(
								elementRootType.getNamespaceURI(),
								GPXUtilsParser.removeBlancSymbol(elementRootType), 
								value,								
								parentElement);
						isInitialized = true;
					}
					handler.getContentHandler().endElement(element);
				}
				break;
			case IXmlStreamReader.CHARACTERS:					
				value = getValue(type, parser.getText());				
				break;
			}
			if (!endFeature){	
				//if the element tag has attributes we parse them
				if(parser.getAttributeCount() > 0){
					value = getAttributes(parser, handler);
				}
				currentTag = parser.next();
				tag = parser.getName();
			}
		}
		return element;		
	}

	/**
	 * This method has to calculate the element type
	 * @param name
	 * Element name
	 * @return
	 * Element type
	 */
	private  String getType(String name){
		return "xs:string";
	}

	/**
	 * This method must return the element value
	 * @param type
	 * XS Element type
	 * @param value
	 * Element value like a String
	 * @return
	 * Element value
	 */
	private  Object getValue(String type, String value){
		return value;
	}
	
	/**
	 * This method must return a string containing
	 * the values of the attributes of an element tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * String with attributes' values separate with ","
	 */
	private String getAttributes(IXmlStreamReader parser,GPEDefaultGpxParser handler) throws XmlStreamException{
		
		String attributes = null;
		if(parser.getAttributeCount() > 0){
			attributes = new String();
			for(int i = 0; i<parser.getAttributeCount(); i++){
				attributes += parser.getAttributeValue(i);
				attributes += GpxTags.COORDINATES_SEPARATOR;
			}
		}
		
		if(attributes != null){
			if(attributes.endsWith(GpxTags.COORDINATES_SEPARATOR)){
				attributes = attributes.substring(0, attributes.length()-1);
			}
		}
			
		return attributes;
	}

}
