package org.gvsig.gpe.gpx.parser.v11.features;

import java.io.IOException;

import javax.xml.namespace.QName;

import org.gvsig.gpe.gpx.parser.GPEDefaultGpxParser;
import org.gvsig.gpe.gpx.utils.GpxTags;
import org.gvsig.gpe.xml.stream.IXmlStreamReader;
import org.gvsig.gpe.xml.stream.XmlStreamException;
import org.gvsig.gpe.xml.utils.CompareUtils;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/

/**
 * This class parsers a Metadata tag.
 * @see http://www.topografix.com/GPX/1/1/#type_metadataType
 */
public class MetadataBinding {
	
	/**
	 * It parses the Metadata tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @throws XmlStreamException 
	 * @throws IOException 
	 */
	public static Object parse(IXmlStreamReader parser,GPEDefaultGpxParser handler, Object layer) throws XmlStreamException, IOException{
		boolean endFeature = false;
		int currentTag;			
		
		String id = null;
		Object feature = handler.getContentHandler().startFeature(id, GpxTags.NAMESPACE, null, null);

		QName tag = parser.getName();
		currentTag = parser.getEventType();

		while (!endFeature){
			switch(currentTag){
			case IXmlStreamReader.START_ELEMENT:	
				if (CompareUtils.compareWithOutNamespace(tag,GpxTags.NAME)){
					parser.next();
					handler.getContentHandler().addNameToLayer(parser.getText(), layer);
					//TO GET AS AN ELEMENT
					//Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature,null);
					//handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag,GpxTags.DESC)){
					parser.next();
					handler.getContentHandler().addDescriptionToLayer(parser.getText(), layer);
					//TO GET AS AN ELEMENT
					//Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature,null);
					//handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag,GpxTags.AUTHOR)){
					//parser.next();
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature,null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag,GpxTags.COPYRIGHT)){
					//parser.next();
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag,GpxTags.LINK)){
					//parser.next();
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag,GpxTags.TIME)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, null, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag,GpxTags.KEYWORDS)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, null, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.BOUNDS)){					
					Object bBox = handler.getProfile().getLatLonBoxBinding().parse(parser, handler);
					handler.getContentHandler().addBboxToLayer(bBox, layer);
				}else if (CompareUtils.compareWithOutNamespace(tag,GpxTags.EXTENSIONS)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, null, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}

				break;
			case IXmlStreamReader.END_ELEMENT:
				if (CompareUtils.compareWithOutNamespace(tag, GpxTags.METADATA)){
					endFeature = true;
					handler.getContentHandler().endFeature(feature);
				}
				break;
			case IXmlStreamReader.CHARACTERS:					

				break;
			}
			if (!endFeature){					
				currentTag = parser.next();
				tag = parser.getName();
			}
		}		
		return feature;
	}
}
