package org.gvsig.gpe.gpx.parser.v11.features;

import java.io.IOException;
import java.util.Date;

import javax.xml.namespace.QName;

import org.gvsig.gpe.gpx.parser.GPEDefaultGpxParser;
import org.gvsig.gpe.gpx.utils.GpxTags;
import org.gvsig.gpe.xml.stream.IXmlStreamReader;
import org.gvsig.gpe.xml.stream.XmlStreamException;
import org.gvsig.gpe.xml.utils.CompareUtils;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/

/**
 * This class parses a WPT tag. Example:
 * @see http://www.topografix.com/GPX/1/1/#type_wptType
 */
public class WayPointBinding {

	/**
	 * It parses a wptType tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @throws XmlStreamException 
	 * @throws IOException 
	 */
	public static Object parse(IXmlStreamReader parser,GPEDefaultGpxParser handler) throws XmlStreamException, IOException{
		boolean endFeature = false;
		int currentTag;			

		//TODO: Generar uniqueID
		String id = null;
		Object feature = handler.getContentHandler().startFeature(id, GpxTags.NAMESPACE, "wayPoint", null);
		
		Object point = handler.getProfile().getPointBinding().parse(parser, handler);
		handler.getContentHandler().addGeometryToFeature(point, feature);
		
		QName tag = parser.getName();
		currentTag = parser.getEventType();		

		while (!endFeature){
			switch(currentTag){
			case IXmlStreamReader.START_ELEMENT:	
				if (CompareUtils.compareWithOutNamespace(tag, GpxTags.ELE)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.TIME)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.MAGVAR)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.GEOIDHEIGHT)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.NAME)){	
					parser.next();
					handler.getContentHandler().addNameToFeature(parser.getText(), feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.CMT)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.DESC)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.SRC)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.LINK)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.SYM)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.TYPE)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.FIX)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.SAT)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.HDOP)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.VDOP)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.PDOP)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.AGEOFDGPSDATA)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.DGPSID)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}else if (CompareUtils.compareWithOutNamespace(tag, GpxTags.EXTENSIONS)){					
					Object element = handler.getProfile().getElementBinding().parse(parser, handler, feature, null);
					handler.getContentHandler().addElementToFeature(element, feature);
				}				

				break;
			case IXmlStreamReader.END_ELEMENT:
				if (CompareUtils.compareWithOutNamespace(tag, GpxTags.WPT) ||
					CompareUtils.compareWithOutNamespace(tag, GpxTags.RTEPT) ||
					CompareUtils.compareWithOutNamespace(tag, GpxTags.TRKPT)){
					endFeature = true;
					handler.getContentHandler().endFeature(feature);
				}
				break;
			case IXmlStreamReader.CHARACTERS:					

				break;
			}
			if (!endFeature){					
				currentTag = parser.next();
				tag = parser.getName();
			}
		}
		
		return feature;
	}	
}
