package org.gvsig.gpe.gpx.parser.v11.geometries;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.xml.namespace.QName;

import org.gvsig.gpe.gpx.parser.GPEDefaultGpxParser;
import org.gvsig.gpe.gpx.utils.GPXUtilsParser;
import org.gvsig.gpe.gpx.utils.GpxTags;
import org.gvsig.gpe.gpx.parser.v11.coordinates.CoordinatesTypeIterator;
import org.gvsig.gpe.xml.stream.IXmlStreamReader;
import org.gvsig.gpe.xml.stream.XmlStreamException;
import org.gvsig.gpe.xml.utils.CompareUtils;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/

/**
 *  It parsers attributes lat and lon inside a wptType tag. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;wpt&gt;lan="60.0" lon="60.0"&lt;/wpt&gt;
 * </code>
 * </pre>
 * </p> 
 * @see http://www.topografix.com/GPX/1/1/#type_wptType  
 * 
 */
public class PointBinding {
	
	/**
	 * It parsers attributes lat and lon inside a wptType tag. Example:
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * A point
	 * @throws IOException 
	 * @throws XmlStreamException 
	 */
	public Object parse(IXmlStreamReader parser,GPEDefaultGpxParser handler) throws XmlStreamException, IOException {
		boolean endFeature = false;
		int currentTag;
		Object point = null;
		
		String id = null;			
			
		CoordinatesTypeIterator coordinatesIteartor = handler.getProfile().getCoordinatesTypeBinding();
		coordinatesIteartor.initialize(parser, handler, GpxTags.WPT);		
		
		point = handler.getContentHandler().startPoint(id,
				coordinatesIteartor,
				GpxTags.DEFAULT_SRS);

		handler.getContentHandler().endPoint(point);		
		
		return point;	
	}
	
	/**
	 * This method takes values from attributes of a wptType tag
	 * and returns a string containing them separated by ','
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * A String containing the wptType coordinates separated by ','
	 * @throws XmlStreamException
	 * @throws IOException
	 */
	public static String parseCoordinates(IXmlStreamReader parser,GPEDefaultGpxParser handler) throws XmlStreamException, IOException{

		String coordinates = "";
		String latitude = "";
		
		Hashtable attributes = GPXUtilsParser.getAttributes(parser);		
		
		if(attributes.containsKey(GpxTags.LAT.getLocalPart())){			
			latitude += (String)attributes.get(GpxTags.LAT.getLocalPart());
			latitude += (GpxTags.COORDINATES_SEPARATOR);		
		}
		
		if(attributes.containsKey(GpxTags.LON.getLocalPart())){
			coordinates += (String)attributes.get(GpxTags.LON.getLocalPart());
			coordinates += GpxTags.COORDINATES_SEPARATOR;
			//add latitude after longitude.
			coordinates += latitude;
		}		
		
		//z coordinate
		coordinates += "0.0";
		
		if(coordinates.endsWith(GpxTags.COORDINATES_SEPARATOR))
		{
			coordinates = coordinates.substring(0, coordinates.length()-2).toString();
		}
		
		return coordinates;
	}	

}
