package org.gvsig.gpe.gpx.utils;

import java.util.Hashtable;

import javax.xml.namespace.QName;

import org.gvsig.gpe.GPEDefaults;
import org.gvsig.gpe.utils.StringUtils;
import org.gvsig.gpe.xml.stream.IXmlStreamReader;
import org.gvsig.gpe.xml.stream.XmlStreamException;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 * @author Carlos Snchez Perin (sanchez_carper@gva.es)
 */
public class GPXUtilsParser {

	/**
	 * It returns a HashTable with the XML attributes. It has been 
	 * created because the parser doesn't has a getAttribiute(AttributeName)
	 * method.

	 * @param parser
	 * @return
	 * @throws XmlStreamException 
	 */
	public static Hashtable getAttributes(IXmlStreamReader parser) throws XmlStreamException{
		Hashtable hash = new Hashtable();
		//parser.next();
		int num_atributos = parser.getAttributeCount();
		while(parser.getAttributeCount() < 0)
		{
			QName tag = parser.getName();			
			parser.next();
		}
		for (int i=0 ; i<parser.getAttributeCount() ; i++){
			QName atributo = parser.getAttributeName(i);
			String valor=parser.getAttributeValue(i);
			if (valor!=null)
				hash.put(atributo.getLocalPart(), valor);
		}
		return hash;
	}	
	
	/**
	 * Remove the blanc symbol from a tag
	 * @param tag
	 * Tag name
	 * @return
	 * The tag without blancs
	 */
	public static String removeBlancSymbol(QName tag){
		if (tag == null){
			return null;
		}
		String blancSpace = GPEDefaults.getStringProperty(GPEDefaults.DEFAULT_BLANC_SPACE);
		if (blancSpace == null){
			blancSpace = GpxTags.DEFAULT_BLANC_SPACE;
		}
		// PROBLEM WITH COMPATIBILITY OF "replaceAll()" WITH IBM J9 JAVA MICROEDITION
		return StringUtils.replaceAllString(tag.getLocalPart(), blancSpace, " ");
		// return tag.replaceAll(blancSpace," ");		
	}
	
	/**
	 * Replace the blancs of a tag with the
	 * deafult blanc symbol
	 * @param tag
	 * @return
	 * A tag with blancs
	 */
	public static String addBlancSymbol(QName tag){
		if (tag == null){
			return null;
		}
		String blancSpace = GPEDefaults.getStringProperty(GPEDefaults.DEFAULT_BLANC_SPACE);
		if (blancSpace == null){
			blancSpace = GpxTags.DEFAULT_BLANC_SPACE;
		}
		// PROBLEM WITH COMPATIBILITY OF "replaceAll()" WITH IBM J9 JAVA MICROEDITION
		return StringUtils.replaceAllString(tag.getLocalPart()," ",blancSpace);
		//return tag.replaceAll(" ",blancSpace);		
	}
}

