package org.gvsig.gpe.kml.writer.v21.geometries;

import java.io.IOException;

import org.gvsig.gpe.kml.utils.KmlTags;
import org.gvsig.gpe.kml.writer.GPEKmlWriterHandlerImplementor;
import org.gvsig.gpe.parser.ICoordinateIterator;
import org.gvsig.gpe.xml.writer.IWriter;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: CoordinatesWriter.java 361 2008-01-10 08:41:21Z jpiera $
 * $Log$
 * Revision 1.3  2007/05/16 09:30:09  jorpiell
 * the writting methods has to have the errorHandler argument
 *
 * Revision 1.2  2007/05/08 07:53:08  jorpiell
 * Add comments to the writers
 *
 * Revision 1.1  2007/04/14 16:08:07  jorpiell
 * Kml writing support added
 *
 *
 */
/**
 * This class writes a coordinates Kml tag. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;coordinates&gt;60.0,60.0 60.0,90.0 90.0,90.0&lt;/coordinates&gt;
 * </code>
 * </pre>
 * </p> 
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 * @see http://code.google.com/apis/kml/documentation/kml_tags_21.html#coordinates
 */
public class CoordinatesWriter {
	
	/**
	 * It writes an array of coordinates written using
	 * the kml coordinates tag
	 * @param writer
	 * Writer to write the labels
	 * @param handler
	 * The writer handler implementor
	 * @param coords
	 * A coordinates iterator. 
	 * @throws IOException
	 */
	public void write(IWriter writer, GPEKmlWriterHandlerImplementor handler,
			ICoordinateIterator coords) throws IOException{
		writer.write("\n");
		writer.write("<" + KmlTags.COORDINATES.getLocalPart() + ">");
		double[] buffer = new double[coords.getDimension()];
		//The first coordinate
		if (coords.hasNext()){
			coords.next(buffer);		
			writeOneCoordinate(writer,handler,buffer);			
		}
		//The other coordinates
		while (coords.hasNext()){
			coords.next(buffer);		
			writer.write(KmlTags.TUPLES_SEPARATOR);
			writeOneCoordinate(writer,handler,buffer);			
		}		
		writer.write("</" + KmlTags.COORDINATES.getLocalPart() + ">");
	}
	
	/**
	 * Writes the content of a coordinates kml tag
	 * @param writer
	 * Writer to write the labels
	 * @param handler
	 * The writer handler implementor
	 * @param buffer
	 * An array with the coordinates
	 * @throws IOException
	 */
	private void writeOneCoordinate(IWriter writer, GPEKmlWriterHandlerImplementor handler,
			double[] buffer) throws IOException{	
		for (int i=0 ; i<buffer.length ; i++){
			handler.getProfile().getDoubleWriter().write(writer,handler,buffer[i]);
			if (i<buffer.length -1){
				writer.write(KmlTags.COORDINATES_SEPARATOR);
			}
		}
	}
}
