package org.gvsig.gpe.kml.writer.v21.geometries;

import java.io.IOException;

import org.gvsig.gpe.kml.utils.KmlTags;
import org.gvsig.gpe.kml.writer.GPEKmlWriterHandlerImplementor;
import org.gvsig.gpe.parser.ICoordinateIterator;
import org.gvsig.gpe.xml.writer.IWriter;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: LatLonAltBoxWriter.java 361 2008-01-10 08:41:21Z jpiera $
 * $Log$
 * Revision 1.3  2007/05/16 09:30:09  jorpiell
 * the writting methods has to have the errorHandler argument
 *
 * Revision 1.2  2007/05/09 08:36:23  jorpiell
 * Add the bbox to the layer
 *
 * Revision 1.1  2007/05/08 07:53:08  jorpiell
 * Add comments to the writers
 *
 *
 */
/**
 * This class writes a LatLonAltBox Kml tag. Example:
 * <p>
 * <pre>
 * <code> 
 * &lt;LatLonAltBox&gt;
 * &lt;north&gt;43.374&lt;/north&gt;
 * &lt;south&gt;42.983&lt;/south&gt;
 * &lt;east&gt;-0.335&lt;/east&gt;
 * &lt;west&gt;-1.423&lt;/west&gt;
 * &lt;minAltitude&gt;0&lt;/minAltitude&gt;
 * &lt;maxAltitude&gt;0&lt;/maxAltitude&gt;
 * &lt;/LatLonAltBox&gt;
 * </code>
 * </pre>
 * </p> 
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 * @see http://code.google.com/apis/kml/documentation/kml_tags_21.html#latlonaltbox
 */
public class LatLonAltBoxWriter {
	
	/**
	 * It writes the LatLonAltBox init tag and its fields
	 * @param writer
	 * Writer to write the labels
	 * @param handler
	 * The writer handler implementor
	 * @param coords
	 * A coordinates iterator. 
	 * @param north
	 * Specifies the latitude of the north edge of the bounding box,
	 * in decimal degrees from 0 to 90.
	 * @param south
	 * Specifies the latitude of the south edge of the bounding box,
	 * in decimal degrees from 0 to 90.
	 * @param east
	 * Specifies the longitude of the east edge of the bounding box,
	 * in decimal degrees from 0 to 180.
	 * @param west
	 * Specifies the longitude of the west edge of the bounding box, 
	 * in decimal degrees from 0 to 180.
	 * @param minAltitude
	 * Specified in meters above sea level
	 * @param maxAltitude
	 * Specified in meters above sea levell 
	 * @throws IOException
	 */
	public void start(IWriter writer, GPEKmlWriterHandlerImplementor handler, 
			ICoordinateIterator coords) throws IOException{
		double[] minBuffer = new double[coords.getDimension()];
		double[] maxBuffer = new double[coords.getDimension()];
		//TODO what happends if the length is < 3!!! If the iterator only has 1 element
		if (coords.hasNext()){
			coords.next(minBuffer);
		}
		if (coords.hasNext()){
			coords.next(maxBuffer);
		}
		writer.write("\n");
		writer.write("<" + KmlTags.LATLONALTBOX.getLocalPart() + ">");
		writer.write("\n");
		writer.write("<" + KmlTags.NORTH.getLocalPart() + ">");
		handler.getProfile().getDoubleWriter().write(writer, handler, maxBuffer[0]);
		writer.write("</" + KmlTags.NORTH.getLocalPart() + ">");
		writer.write("\n");
		writer.write("<" + KmlTags.SOUTH.getLocalPart() + ">");
		handler.getProfile().getDoubleWriter().write(writer, handler, minBuffer[0]);
		writer.write("</" + KmlTags.SOUTH.getLocalPart() + ">");
		writer.write("\n");
		writer.write("<" + KmlTags.EAST.getLocalPart() + ">");
		handler.getProfile().getDoubleWriter().write(writer, handler, maxBuffer[1]);
		writer.write("</" + KmlTags.EAST.getLocalPart() + ">");
		writer.write("\n");
		writer.write("<" + KmlTags.WEST.getLocalPart() + ">");
		handler.getProfile().getDoubleWriter().write(writer, handler, minBuffer[1]);
		writer.write("</" + KmlTags.WEST.getLocalPart() + ">");
		writer.write("\n");
		writer.write("<" + KmlTags.MINALTITUDE.getLocalPart() + ">");
		handler.getProfile().getDoubleWriter().write(writer, handler, maxBuffer[2]);
		writer.write("</" + KmlTags.MINALTITUDE.getLocalPart() + ">");
		writer.write("\n");
		writer.write("<" + KmlTags.MAXALTITUDE.getLocalPart() + ">");
		handler.getProfile().getDoubleWriter().write(writer, handler, minBuffer[2]);
		writer.write("</" + KmlTags.MAXALTITUDE.getLocalPart() + ">");
	}
	
	/**
	/**
	 * It writes the LatLonAltBox end tag and its fields
	 * @param writer
	 * Writer to write the labels
	 * @param handler
	 * The writer handler implementor
	 * @throws IOException
	 */
	public void end(IWriter writer, GPEKmlWriterHandlerImplementor hanlder) throws IOException{
		writer.write("\n");
		writer.write("</" + KmlTags.LATLONALTBOX.getLocalPart() + ">");
	}
}
