package org.gvsig.gpe;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.xml.parsers.SAXParserFactory;

import org.gvsig.gpe.exceptions.GPEParserCreationException;
import org.gvsig.gpe.exceptions.GPEWriterHandlerCreationException;
import org.gvsig.gpe.parser.GPEParser;
import org.gvsig.gpe.writer.GPEWriterHandler;
import org.gvsig.gpe.writer.IGPEWriterHandlerImplementor;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GPERegister.java 282 2007-12-20 11:55:56Z jpiera $
 * $Log$
 * Revision 1.16  2007/06/28 13:04:33  jorpiell
 * The Qname has been updated to the 1.5 JVM machine. The schema validation is made in the GPEWriterHandlerImplementor class
 *
 * Revision 1.15  2007/06/20 09:35:37  jorpiell
 * Add the javadoc comments
 *
 * Revision 1.14  2007/05/16 12:34:55  csanchez
 * GPEParser Prototipo final de lectura
 *
 * Revision 1.13  2007/05/09 06:54:07  jorpiell
 * Change the File by URI
 *
 * Revision 1.12  2007/05/08 12:57:14  jorpiell
 * Add the register exceptions
 *
 * Revision 1.11  2007/05/07 07:06:26  jorpiell
 * Add a constructor with the name and the description fields
 *
 * Revision 1.10  2007/04/19 11:50:20  csanchez
 * Actualizacion protoripo libGPE
 *
 * Revision 1.9  2007/04/19 07:23:20  jorpiell
 * Add the add methods to teh contenhandler and change the register mode
 *
 * Revision 1.8  2007/04/18 12:54:45  csanchez
 * Actualizacion protoripo libGPE
 *
 * Revision 1.7  2007/04/17 07:53:55  jorpiell
 * Before to start a new parsing process, the initialize method of the content handlers is throwed
 *
 * Revision 1.6  2007/04/17 06:26:54  jorpiell
 * Fixed one problem with a index
 *
 * Revision 1.5  2007/04/14 16:06:13  jorpiell
 * The writer handler has been updated
 *
 * Revision 1.4  2007/04/12 17:06:42  jorpiell
 * First GML writing tests
 *
 * Revision 1.3  2007/04/11 11:10:27  jorpiell
 * Cambiado el nombre de getDriver a GetParser
 *
 * Revision 1.2  2007/04/11 08:54:24  jorpiell
 * Aadidos algunos comentarios
 *
 * Revision 1.1  2007/04/11 08:52:55  jorpiell
 * Se puede registrar una clase por nombre
 *
 * Revision 1.1  2007/04/11 08:22:41  jorpiell
 * GPE clase para registrar los drivers
 *
 *
 */
/**
 * API entry point to create parsers and writers based on MIME-Type identification.
 * <p>
 * This factory uses a SPI (Service Provider Interface) mechanism to look up for {@link GPEParser}
 * and {@link IGPEWriterHandlerImplementor} implementations that can deal with a requested
 * MIME-Type.
 * </p>
 * <h2>Implementation Lookup</h2>
 * <p>
 * The SPI mechanism keeps parser implementations decoupled from client applications allowing the
 * transparent interchange of implementation, thus defining a format plugin system.
 * </p>
 * <p>
 * Parser and Writer implementations registers themselves lazily by providing a small configuration
 * file bundled together with the rest of the implementation resources. This configuration file
 * consist of a text file at a specific location and with an specific file name, this factory look
 * up mechanism will search for in order to find out the implementation class names.
 * </p>
 * <p>
 * To register a GPEParser implememntation, a file named <code>org.gvsig.gpe.parser.GPEParser</code>
 * shall exist in the class path (hence the ability to have multiple files equally namded budled in
 * different jar files), in the implementation's <code>META-INF/services</code> folder.
 * </p>
 * <p>
 * To register a Writer handler implementation, the same approach shall be followed with a file
 * named <code>META-INF/services/org.gvsig.gpe.writer.IGPEWriterHandlerImplementor</code>
 * </p>
 * <p>
 * The content of both files for a given implementation consists of full qualified class names, one
 * per line. For example, an hypotetical <code>MyFormatParser</code> in the package
 * <code>org.mycompany.gpe</code> and bundled in a jar file called <code>myformat.jar</code>
 * shall provide the following resources:
 * 
 * <pre>
 * <code>
 * $jar tvf myformat.jar
 * META-INF/services/org.gvsig.gpe.parser.GPEParser
 * org/mycompany/gpe/MyFormatParser.class
 * </code>
 * </pre>
 * 
 * And the content of the file <code>META-INF/services/org.gvsig.gpe.parser.GPEParser</code> shall
 * be a single line of text with the <code>org.mycompany.gpe.MyFormatParser</code> class name.
 * 
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 * @author Carlos Snchez Perin (sanchez_carper@gva.es)
 */
public class GPEFactory {

    /**
     * Looks up the available parser implementations and returns the list of supported MIME-Type
     * identifiers.
     * 
     * @return the list of available parsing MIME types
     */
//    public static Set getAvailableParserTypes() {
//        Set mimeTypes = new HashSet();
//        Iterator availableParsers = availableParsers();
//        while(availableParsers.hasNext()){
//            GPEParser parser = (GPEParser) availableParsers.next();
//            mimeTypes.add(parser.getMimeType());
//        }
//        return mimeTypes;
//    }
//
//    public static Set getAvailableWriterTypes() {
//        throw new UnsupportedOperationException();
//    }

    /**
     * Returns an iterator over instances of the registered parsers.
     * <p>
     * NOTE: since GPEParser is an abstract and statefull class rather than an interface, this
     * method shall stay private in order to ensure we return a new instance for the required format
     * every time.
     * </p>
     * 
     * @return all the registered parsers
     */
    private static Iterator availableParsers() {
        Iterator providers = sun.misc.Service.providers(GPEParser.class);
        return providers;
    }

    /**
     * Returns an iterator over instances of the registered writers.
     * <p>
     * NOTE: since GPEWriter is an abstract and statefull class rather than an interface, this
     * method shall stay private in order to ensure we return a new instance for the required format
     * every time.
     * </p>
     * 
     * @return all the registered writers
     */
    private static Iterator availableWriters() {
        Iterator providers = sun.misc.Service.providers(IGPEWriterHandlerImplementor.class);
        return providers;
    }

    public static GPEParser createParserByClass(String prefferredImplClassName) {
        Class prefferredImplClass = null;
        try {
            prefferredImplClass = Class.forName(prefferredImplClassName);
        } catch (ClassNotFoundException e) {
            return null;
        }

        Iterator providers = availableParsers();
        while (providers.hasNext()) {
            GPEParser next = (GPEParser) providers.next();
            if (prefferredImplClass.isInstance(next)) {
                return next;
            }
        }
        return null;
    }

    /**
     * Create a new parser from a name
     * 
     * @param name GPEParser name
     * @param contenHandler Application contenHandler usett to throw the parsing events
     * @param errorHandler Application errror handler used to put errors and warnings
     * @throws GPEParserCreationException
     */
    public static GPEParser createParser(String name) throws GPEParserCreationException {
        Iterator providers = availableParsers();
        if (providers.hasNext()) {
            GPEParser parser = (GPEParser) providers.next();
            if (parser.getName().compareTo(name) == 0) {
                return parser;
            }
        }
        return null;
    }

    /**
     * Gets the parser that can open the file (if it exists)
     * 
     * @param uri File to open
     * @return Null if the driver doesn't exist
     * @throws GPEParserCreationException
     * @throws SecurityException
     * @throws IllegalArgumentException
     */
    public static GPEParser createParser(URI uri) throws GPEParserCreationException {
        Iterator providers = availableParsers();
        if (providers.hasNext()) {
            GPEParser parser = (GPEParser) providers.next();
            while ((!(parser.accept(uri)))&&(providers.hasNext())){
            	parser = (GPEParser) providers.next();	
            }
            if (parser.accept(uri)) {
                return parser;
            }
        }
        return null;
    }

    /**
     * Create a new content writer from a name
     * 
     * @param name GPEWriterHandler name GPEParser name
     * @param contenHandler Application contenHandler usett to throw the parsing events
     * @param errorHandler Application errror handler used to put errors and warnings
     * @throws GPEWriterHandlerCreationException
     */
    public static GPEWriterHandler createWriter(String name)
            throws GPEWriterHandlerCreationException {
        Iterator providers = availableWriters();
        if (providers.hasNext()) {
            IGPEWriterHandlerImplementor implementor = (IGPEWriterHandlerImplementor) providers
                    .next();
            if (implementor.getName().compareTo(name) == 0) {
                return new GPEWriterHandler(implementor);
            }
        }
        return null;
    }

    public static GPEWriterHandler createWriterByClass(String prefferredImplClassName) {
        Class prefferredImplClass = null;
        try {
            prefferredImplClass = Class.forName(prefferredImplClassName);
        } catch (ClassNotFoundException e) {
            return null;
        }

        Iterator providers = availableWriters();
        while (providers.hasNext()) {
            IGPEWriterHandlerImplementor next = (IGPEWriterHandlerImplementor) providers.next();
            if (prefferredImplClass.isInstance(next)) {
                return new GPEWriterHandler(next);
            }
        }
        return null;
    }

    /**
     * Gets all the writers that can
     * 
     * @param format
     * @return
     */
    public static ArrayList getWriterHandlerByFormat(String format) {
        Iterator providers = availableWriters();
        ArrayList possibleWriters = new ArrayList();
        if (providers.hasNext()) {
            IGPEWriterHandlerImplementor implementor = (IGPEWriterHandlerImplementor) providers
                    .next();
            String[] formats = implementor.getFormats();
            for (int i = 0; i < formats.length; i++) {
                if (formats[i].toLowerCase().compareTo(format.toLowerCase()) == 0) {
                    possibleWriters.add(new GPEWriterHandler(implementor));
                }
            }
        }
        return possibleWriters;
    }

    /**
     * Return true if exists a driver that can open the file
     * 
     * @param uri File to open
     * @return true if the driver exists
     */
    public static boolean accept(URI uri) {
        Iterator providers = availableParsers();
        if (providers.hasNext()) {
            GPEParser parser = (GPEParser) providers.next();
            while((parser.accept(uri)==false)&&(providers.hasNext()==true)){
            	parser = (GPEParser) providers.next();	
            }
            if (parser.accept(uri)) {
                return true;
            }
        }
        return false;
    }
}
