/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package gnu.io;

import java.io.FileDescriptor;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Vector;

/**
 * Communications port management. CommPortIdentifier is the central class for 
 * controlling access to communications ports. It includes methods for:
 *
 * - Determining the communications ports made available by the driver.
 * - Opening communications ports for I/O operations.
 * - Determining port ownership.
 * - Resolving port ownership contention.
 * - Managing events that indicate changes in port ownership status. 
 *
 * An application first uses methods in CommPortIdentifier to negotiate with 
 * the driver to discover which communication ports are available and then 
 * select a port for opening. It then uses methods in other classes like CommPort, 
 * ParallelPort and SerialPort to communicate through the port.
 *  
 * @author juangui
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */

public class CommPortIdentifier {
	public static int PORT_SERIAL = 1;
	public static int PORT_PARALLEL = 2;
	public static int TOTAL_PORT_NUMBER = 9;
	public static String[] PORT_NAMES={"COM0","COM1","COM2","COM3","COM4",
		"COM5","COM6","COM7","COM8"};
	
	private String portName;
	private String owner;    
	private int portType;
	private CommPort commPort;
	private static boolean debug = false;
	private static CommDriver RXTXDriver;
	
//	private boolean Available = true;    
// 	static CommPortIdentifier CommPortIndex;
//	CommPortIdentifier next;
//	static Object Sync;
//	Vector ownershipListener;

	/*------------------------------------------------------------------------------
	static {}   aka initialization
	accept:       -
	perform:      load the rxtx driver
	return:       -
	exceptions:   Throwable
	comments:     static block to initialize the class
------------------------------------------------------------------------------*/
//	initialization only done once....
	static 
	{
		RXTXDriver = new RXTXCommDriver();		
		RXTXDriver.initialize();
	}
	
	/**
	 * Constructor
	 */
	CommPortIdentifier (String pn, CommPort cp, int pt) 
	{
		portName = pn;
		commPort = cp;
		portType = pt;
	}

	/**
	 * Adds portName to the list of ports.
	 * 
	 * @param portName - The name of the port being added
	 * @param portType - The type of the port being added
	 * @param driver - The driver representing the port being added
	 */
	public static void addPortName(java.lang.String portName, int portType, CommDriver driver) {
		//Adds portName to the list of ports.
	}

	public void addPortOwnershipListener(CommPortOwnershipListener listener) {
		//Registers an interested application so that it can receive notification of changes in port ownership.
	}

	/**
	 * Returns the owner of the port, that is, the name of the application
	 * owning the port.
	 * @return current owner of the port.
	 */
	public String getCurrentOwner() {
		//Returns the owner of the port.
		return (owner);
	}
	
	/**
	 * Returns the name of the port. The port name should be identifiable by the user.
	 * Ideally, it should be the label on the hardware. For example, "COM1" and 
	 * "COM2" on PCs; "Serial A" and "Serial B" on Sun Ultra workstations. 
	 * The port name may be stored by an application and subsequently used 
	 * to create a CommPortIdentifier object using 
	 * getPortIdentifier(String portName) method.
	 * @return
	 */
	public String getName() {
		//Returns the name of the port.
		return (portName);
	}
	
	public static CommPortIdentifier getPortIdentifier(CommPort port)
													throws NoSuchPortException {
		//Obtains the CommPortIdentifier object corresponding to a port that has already been opened by the application.
		return null;
	}
	
	/**
	 * Obtains a CommPortIdentifier object by using a port name. 
	 * The port name may have been stored in persistent storage by the application.
	 * 
	 * @param _portName - name of the port to open
	 * @return a CommPortIdentifier object
	 * @throws NoSuchPortException - if the port does not exist
	 */
	public static CommPortIdentifier getPortIdentifier(String _portName) 
													throws NoSuchPortException {
		//if(debug) System.out.println("CommPortIdentifier:getPortIdentifier(" + _portName +")");
		
		int index = -1;
		for (int i=0; i<TOTAL_PORT_NUMBER; i++)
			if (PORT_NAMES[i].equals(_portName)) 
				{
				index = i;
				break;			
				}
		
		if (index < 0) throw new NoSuchPortException();
		else return(new CommPortIdentifier(_portName,null,PORT_SERIAL));
	}
	
	/**
	 * Obtains an enumeration object that contains a CommPortIdentifier object 
	 * for each port in the system.
	 * @return
	 */
	public static Enumeration getPortIdentifiers() {
		List ports = new ArrayList();
		for (int i = 1; i<=16; i++){
			//quickfix, not complete
			CommPortIdentifier port = new CommPortIdentifier("COM" + i,null,0);
			ports.add(port);
		}			
		return new CommPortEnumerator(ports);
	}

	/**
	 * Returns the port type
	 * @return - PORT_SERIAL or PORT_PARALLEL
	 */
	public int getPortType() {
		//Returns the port type.
		return (portType);
	}
	
	public boolean isCurrentlyOwned() {
		//Checks whether the port is owned.
		return false;
	}

	/**
	 * Opens the communications port using a FileDescriptor object on platforms
	 * that support this technique.
	 * @param fd - The FileDescriptor object used to build a CommPort.
	 * @return a CommPort object.
	 * @throws UnsupportedCommOperationException - is thrown on platforms which 
	 * do not support this functionality.
	 */
	public CommPort open(FileDescriptor fd) throws UnsupportedCommOperationException {
		//Opens the communications port using a FileDescriptor object on platforms that support this technique.
		return null;
	}

	/**
	 * Opens the communications port. open obtains exclusive ownership of the port. 
	 * If the port is owned by some other application, a PORT_OWNERSHIP_REQUESTED 
	 * event is propagated using the CommPortOwnershipListener event mechanism. 
	 * If the application that owns the port calls close during the event processing,
	 * then this open will succeed. There is one InputStream and one OutputStream 
	 * associated with each port. After a port is opened with open, then all calls 
	 * to getInputStream will return the same stream object until close is called.
	 * 
	 * @param appname - Name of application making this call. This name will become 
	 * the owner of the port. Useful when resolving ownership contention.
	 * @param timeout - time in milliseconds to block waiting for port open.
	 * @return a CommPort object
	 * @throws PortInUseException - if the port is in use by some other application
	 * that is not willing to relinquish ownership
	 */
	public CommPort open(String appname, int timeout) throws PortInUseException {
		owner = appname;
		commPort = RXTXDriver.getCommPort(portName,portType); 
		return commPort;
	}
	
	public void removePortOwnershipListener(CommPortOwnershipListener listener) {
		//Deregisters a CommPortOwnershipListener registered using addPortOwnershipListener
	}
}

